/*jslint indent:4, browser:true */
/*global define, module_, class_, include_, use_, extend_, static_, properties_, property_, globals_ */
/**
 * @projectDescription Dictionary module
 *
 * @author Julien Polo<julien.polo@gmail.com>
 * @version 1.0.1
 *
 * @link http://wiki.commonjs.org/wiki/Promises
 * @link https://github.com/kriszyp/node-promise
 * @link https://github.com/kriskowal/q
 */
define("stj/Deferred", [ "stj" ], function (stj) {
    "use strict";


    /**
     * @class Deferred
     */
    class_('Deferred', null, function (Deferred) {
        /**
         * Shortcuts
         */
        var
            isUndefined = Object.isUndefined,
            isInstanceOf = Object.isInstanceOf,
            isFunction = Object.isFunction,
            isArray = Object.isArray,

            size = Object.size,
            forEach = Object.forEach,
            objectGet = Object.get,
            objectSet = Object.set,

            //Filters
            filter = use_('filter'),
            validateFunction = filter.validateFunction,
            validateObject = filter.validateObject,

            setNextTick = use_('event').setNextTick,

            defer = Deferred.create,
            proxy = function (deferred) {
                return deferred;
            },
            argsArray = function (args) {
                return isArray(args[0]) ? args[0] : args;
            },

            //Errors
            TimeoutError = Error,
            RuntimeError = use_('error.RuntimeError'),
            StopIteration = use_('error.StopIteration');

        /**
         * Static
         */
        extend_({
            UNKNOWN: "unknown",
            SUCCESS: "success",
            ERROR: "error",

            /**
             * Return true if object implements then
             *
             * @param {*} object
             * @return {boolean}
             */
            isPromise: function isPromise(object) {
                return object && isFunction(object.then);
            },

            /**
             * Registers an observer on a promise.
             *
             * @param {*} value promise or value to observe
             * @param {Function} onSuccess function to be called with the resolved value
             * @param {Function} onError function to be called with the rejection reason
             * @param {*=} thisp object passed to this
             * @return {Promise} promise
             */
            when: function when(value, onSuccess, onError/*, thisp*/) {
                var result = this,
                    args = arguments,
                    thisp = args[3];
                if (Deferred.isPromise(value)) {
                    result = value.then(onSuccess, onError, thisp);
                } else {
                    result = defer();
                    result.then(onSuccess, onError, thisp);
                    result.emitSuccess(value);
                }
                return proxy(result);
            },

            /**
             * Return onSuccess and onError will be called on every promise result.
             *
             * @param {Array} arrayOfPromises promise or value to observe
             * @param {Function} onSuccess(value, index, isError) -> result
             * @param {Function} onError(error, index, isError) -> result
             * @param {*=} thisp object passed to this
             * @return {Promise} promise
             */
            forEach: (function /*factory*/() {
                var _forEach = forEach,
                    onAny = function (value, isError) {
                        var env = this,
                            index = env.index,
                            parent = env.parent,
                            deferred = parent.deferred,
                            result;

                        if (!parent.finished) {
                            parent.done += 1;

                            try {
                                result = (isError ? parent.onSuccess : parent.onError).call(parent.thisp, value, index, isError);
                            } catch (e) {
                                if (!isInstanceOf(e, StopIteration)) {
                                    throw e;//rethrow
                                } else {
                                    parent.finished = true;
                                }
                            }

                            if (!isUndefined(result)) {
                                parent.result = result;
                            }

                            if (parent.finished || parent.done === parent.todo) {
                                deferred.emitSuccess(parent.result);
                            }
                        }
                    },
                    iteratorAsync = function (promiseOrValue, index) {
                        Deferred.when(promiseOrValue, onAny, onAny, {
                            parent: this,
                            index: index
                        });
                    };

                return function forEach(arrayOfPromises, onSuccess, onError/*, thisp*/) {
                    validateFunction(onSuccess);
                    validateFunction(onError);
                    var deferred = defer(),
                        length = size(arrayOfPromises);
                    if (length === 0) {
                        deferred.emitSuccess([]);
                    } else {
                        _forEach(arrayOfPromises, iteratorAsync, {
                            deferred: deferred,
                            done: 0,
                            todo: length,
                            finished: false,
                            onSuccess: onSuccess,
                            onError: onError,
                            thisp: arguments[3]
                        });
                    }
                    return proxy(deferred);
                };
            }()),

            /**
             * Return promise that emit success array of all promise results
             * or emit error at first error encountered
             *
             * @param {Array} arrayOfPromises promise or value to observe
             * @return {Promise} promise
             */
            every: (function /*factory*/() {
                var onAny = function (value, index, isError) {
                        var env = this,//deferred, results
                            deferred = env[0],
                            results = env[1];
                        if (isError) {
                            deferred.emitError(value);
                            throw new StopIteration();
                        }
                        results[index] = value;
                        return results;
                    };
                return function every(arrayOfPromises) {
                    var deferred = defer(),
                        array = argsArray(arguments),
                        length = size(array);
                    if (!length) {
                        deferred.emitSuccess([]);
                    } else {
                        Deferred
                        .forEach(array, onAny, onAny, [deferred, new Array(length)])
                        .onSuccess(deferred.emitSuccess, deferred);
                    }
                    return proxy(deferred);
                };
            }()),

            /**
             * Return promise that emit success at first result
             * or emit error at last error encountered
             *
             * @param {Array} arrayOfPromises promise or value to observe
             * @return {Promise} promise
             */
            some: (function /*factory*/() {
                var onAny = function (value, index, isError) {
                        var env = this;//deferred, results
                        if (!isError) {
                            env[0].emitSuccess(value);
                            throw new StopIteration();
                        }
                        return value;
                    };
                return function some(arrayOfPromises) {
                    var deferred = defer(),
                        array = argsArray(arguments);
                    if (!size(array)) {
                        deferred.emitSuccess([]);
                    } else {
                        Deferred
                        .forEach(array, onAny, onAny, [deferred])
                        .onSuccess(deferred.emitError, deferred);
                    }
                    return proxy(deferred);
                };
            }()),

            /**
             * Takes an array of asynchronous functions (that return promises) and executes
             * them sequentially. Each function is called with the return value of the last
             * function
             *
             * @param {Array} array The array of function
             * @param {*} startingValue The value to pass to the first function
             * @return the value returned from the last function
             */
            reduce: function reduce(array, startingValue) {
                array = array.slice(); // make a copy
                var deferred = defer(),
                    next = function (value) {
                        var nextAction = array.shift();
                        if (nextAction) {
                            Deferred.when(nextAction(value), next, deferred.emitError, deferred);
                        } else {
                            deferred.emitSuccess(value);
                        }
                    };
                next(startingValue);
                return proxy(deferred);
            },

            /**
             * Delays for a given amount of time and then fulfills the returned promise.
             *
             * @param {int} milliseconds The number of milliseconds to delay
             * @return {Promise} A promise that will be fulfilled after the delay
             */
            delay: function delay(milliseconds) {
                var deferred,
                    timer = setTimeout(function () {
                        deferred.emitSuccess();
                    }, milliseconds, deferred);
                deferred = defer({
                    cancel: function () {
                        clearTimeout(timer);
                    }
                });
                return proxy(deferred);
            },

            /**
             * Run block callback and return promise.
             * If block throw an error, promise will emit error.
             *
             * @param {Function} block
             * @param {*=} thisp
             * @param {Object|Array=} args
             * @return {Promise}
             */
            run: function run(block/*, thisp, args*/) {
                var deferred = defer(),
                    result,
                    args = arguments;
                try {
                    result = block.apply(args[1], args[2]);
                    if (Deferred.isPromise(result)) {
                        result.then(deferred.emitSuccess, deferred.emitError, deferred);
                    } else {
                        deferred.emitSuccess(result);
                    }
                } catch (e) {
                    deferred.emitError(e);
                }
                return proxy(deferred);
            }
        });

        /**
         * Member
         */
        properties_({

            /**
             * Deferred constructor
             *
             * @constructor
             * @param {Object} options
             */
            initialize: function initialize(options) {
                this.canceller = null;
                this.timer = null;

                this.status = Deferred.UNKNOWN;
                this.handled = false;
                this.handlers = [];

                this.configure(options);

                /*this.value = undefined*/
                //this.__defineGetter__('promise', this.getPromise);
            },

            /**
             * Destructor
             */
            finalize: function finalize() {
                this.cancel();
            },

            /**
             * Configure deferred
             *
             * @param {Object} options
             *   - timeout: milliseconds
             *   - cancel: function called when cancelled
             */
            configure: function configure(options) {
                var current;
                if (options) {
                    current = options.cancel;
                    if (current) {
                        this.canceller = validateFunction(current);
                    }
                    current = options.timeout;
                    if (current) {
                        this.setTimeout(current);
                    }
                }
                return this;
            },

            /**
             * Return true if this deferred is finished
             *
             * @return {boolean}
             */
            isFinished: function isFinished() {
                return this.status !== Deferred.UNKNOWN;
            },

            /**
             * Return true if this deferred is successful
             *
             * @return {boolean}
             */
            isSuccess: function isSuccess() {
                return this.status === Deferred.SUCCESS;
            },

            /**
             * Return true if this deferred had an error
             *
             * @return {boolean}
             */
            isError: function isError() {
                return this.status === Deferred.ERROR;
            },

            /**
             * Add a listener on success
             *
             * @param {Function} block
             * @param {*=} thisp
             * @return {Deferred}
             */
            onSuccess: function onSuccess(block/*, thisp*/) {
                return this._addHandler(block, null, arguments[1]);
            },

            /**
             * Add a listener on error
             *
             * @param {Function} block
             * @param {*=} thisp
             * @return {Deferred}
             */
            onError: function onError(block/*, thisp*/) {
                return this._addHandler(null, block, arguments[1]);
            },

            /**
             * Add a listener on success & error
             *
             * @param {Function} block(error, result)
             * @param {*=} thisp
             * @return {Deferred}
             */
            onAny: (function /*factory*/() {
                var callback = function (value, isError) {
                        var env = this,
                            result = null,
                            error = null;
                        if (isError) {
                            error = value;
                        } else {
                            result = value;
                        }
                        env[0].call(env[1], error, result);
                    };
                return function onAny(block/*, thisp*/) {
                    var handler = block ? callback : null;
                    return this._addHandler(handler, handler, arguments);
                };
            }()),

            /**
             * Add a listener on success and the second argument on error
             *
             * @param {Function=} onSuccess
             * @param {Function=} onError
             * @param {*=} thisp
             * @return {Deferred}
             */
            then: function then(onSuccess, onError/*, thisp*/) {
                return proxy(this._addHandler(onSuccess, onError, arguments[2]));
            },

            /**
             * Return the value corresponding to [propertyName]
             *
             * @param {string|int} propertyName
             * @return {*}
             */
            __get__: function __get__(propertyName) {
                return this.onSuccess(function (object) {
                    return objectGet(object, propertyName);
                });
            },

            /**
             * Set the value corresponding to [propertyName]
             *
             * @param {string} propertyName
             * @return {*}
             */
            __set__: function __set__(propertyName, value) {
                return this.onSuccess(function (object) {
                    objectSet(object, propertyName,  value);
                    return value;
                });
            },

            /**
             * Emit success signal
             *
             * @param {*} value
             * @return this
             */
            emitSuccess: function emitSuccess(value) {
                return this._setStatus(Deferred.SUCCESS, value);
            },
            /*callback: function callback(value) {
                return this.emitSuccess(value);
            },
            resolve: function resolve(value) {
                return this.emitSuccess(value);
            },*/

            /**
             * Emit error signal
             *
             * @param {*} value
             * @return this
             */
            emitError: function emitError(value) {
                return this._setStatus(Deferred.ERROR, value);
            },
            /*errback: function errback(error) {
                return this.emitError(error);
            },
            reject: function reject(error) {
                return this.emitError(error);
            },*/


            /**
             * Set a timeout that will emitError if no emitSuccess or emitError was called
             *
             * @param {*} milliseconds
             * @return this
             */
            setTimeout: (function /*factory*/() {
                var _setTimeout = globals_('setTimeout'),
                    onTimeout = function (self) {
                        self.timer = null;
                        if (!self.isFinished()) {
                            var error = new TimeoutError();
                            if (self.canceller) {
                                self.cancel(error);
                            } else {
                                self.emitError(error);
                            }
                        }
                    };
                return function setTimeout(milliseconds) {
                    if (!this.isFinished()) {
                        this.unsetTimeout();
                        if (milliseconds && milliseconds !== Infinity) {
                            this.timer = _setTimeout(onTimeout, milliseconds, this);
                        }
                    }
                    return this;
                };
            }()),

            /**
             * Cancel the deferred timeout
             *
             * @return this
             */
            unsetTimeout: function unsetTimeout() {
                var timeout = this.timer;
                if (timeout) {
                    clearTimeout(timeout);
                    this.timer = null;
                }
                return this;
            },

            /**
             * Cancel the promise
             *
             * @return {boolean} true if error was handled
             */
            cancel: function cancel() {
                if (!this.isFinished() && this.canceller) {
                    return this.emitError(this.canceller());
                }
                return false;
            },

            _addHandler: function _addHandler(onSuccess, onError, thisp) {
                if (onSuccess) {
                    validateFunction(onSuccess);
                }
                if (onError) {
                    validateFunction(onError);
                }
                var deferredChild = defer(this.canceller);

                //append to be notified
                this.handlers.push([deferredChild, onSuccess, onError, thisp || null]);

                //notify all at next tick if finished
                if (this.isFinished()) {
                    this._notifyAll();
                }
                return deferredChild;
            },

            _setStatus: function _setStatus(status, args) {
                if (this.isFinished()) {
                    throw new RuntimeError("This deferred has already been resolved");
                }
                this.status = status;
                this.value = args;
                this.unsetTimeout();
                this._notifyAll();
                return this;
            },

            _notifyAll: (function /*factory*/() {
                var notifyAllNextTick = function () {
                        var isError = this.isError(),
                            value = this.value,
                            handlers = this.handlers,
                            handler = handlers.pop(),
                            deferredChild, callback, thisp,
                            result;
                        //Reset handlers
                        //this.handlers = [];

                        while (handler) {
                            deferredChild = handler[0];
                            callback = isError ? handler[2] : handler[1];
                            thisp = handler[3];

                            if (callback) {
                                this.handled = true;
                                try {
                                    result = isFunction(callback) ? callback.call(thisp, value, isError) : callback;
                                    if (Deferred.isPromise(result)) {
                                        result.then(deferredChild.emitSuccess, deferredChild.emitError, deferredChild);
                                    } else {
                                        deferredChild.emitSuccess(result);
                                    }
                                } catch (e) {
                                    deferredChild.emitError(e);
                                }
                            } else {
                                if (isError) {
                                    deferredChild.emitError(value);
                                    this.handled = deferredChild.handled;
                                } else {
                                    deferredChild.emitSuccess(value);
                                }
                            }
                            //next handler
                            handler = handlers.pop();
                        }
                        if (isError && !this.handled) {
                            throw value;
                        }

                        this.isNotifying = false;
                    };

                return function _notifyAll() {
                    if (!this.isNotifying && size(this.handlers)) {
                        this.isNotifying = true;
                        setNextTick(notifyAllNextTick, this);
                    }
                };
            }())

            /*getPromise: function () {
                if (!this._promise) {
                    this._promise = new Promise();

                    if (this._canceller) {
                        this._promise.cancel = function () {
                            this.cancel();
                        };
                    }

                    this._promise.then = function (resolvedCallback, errorCallback, progressCallback) {
                        return this.then(resolvedCallback, errorCallback, progressCallback);
                    };
                }
                return this._promise;
            }*/
        });
    });
    return use_('Deferred');
});