/*jslint indent:4 */
/*global define, module_, class_, include_, use_, extend_, static_, properties_, property_, globals_ */
/**
 * @projectDescription Interface module
 *
 * @author Julien Polo<julien.polo@gmail.com>
 * @version 1.0.1
 */
define("stj/Interface", [ "stj" ], function (stj) {
    "use strict";

    /**
     * @class Interface
     */
    class_("Interface", null, function (Interface) {
        var
            //Messages
            MESSAGE_NOT_INTERFACE = ' is not an interface',
            MESSAGE_ALREADY_DECLARED = ' is not already declared',

            //shortcuts
            isArray = Object.isArray,
            isFunction = Object.isFunction,
            isUndefined = Object.isUndefined,
            isEmpty = Object.isEmpty,
            isInstanceOf = Object.isInstanceOf,
            size = Object.size,
            hasOwn = Object.has,
            forEach = Object.forEach,

            namespaceNormalize = stj.namespaceNormalize,
            moduleMeta = stj.meta_,
            moduleCurrent = stj.moduleCurrent,
            assertNotRegistered = function assertNotRegistered(path) {
                if (path.length > 0 && !isUndefined(moduleMeta(moduleCurrent()).path(path))) {
                    throw new TypeError(path + MESSAGE_ALREADY_DECLARED);
                }
            },

            ValueError = use_('error.ValueError'),
            NotImplementedError = use_('error.NotImplementedError');

        //include_("stj.Object");
        properties_({

            /**
             * Interface constructor
             *
             * @constructor
             * @param {Object} definition
             * @param {string=} name
             */
            initialize: function initialize(definition, name) {
                this.displayName = name || "";
                this.properties = {};

                var properties = this.properties, property, value;
                for (property in definition) {
                    if (hasOwn(definition, property)) {
                        value = definition[property];
                        if (value !== true && !isFunction(value)) {
                            throw new TypeError("method should be true or function");
                        }
                        properties[property] = value;
                    }
                }
            },

            /**
             * Validate `target` object
             *
             * @param {Function|Object} target
             */
            validate: function validate(target) {
                forEach(this, function (_, property) {
                    this._validateProperty(target, property);
                }, this);
                return this;
            },

            __size__: function __size__() {
                return size(this.properties);
            },

            __forEach__: function __forEach__(iterator, thisp) {
                var properties = this.properties, property;
                for (property in properties) {
                    if (hasOwn(properties, property) && property !== 'displayName') {
                        iterator.call(thisp, properties[property], property, this);
                    }
                }
            },

            __get__: function __get__(property) {
                return this.properties[property];
            },

            _validateProperty: function (target, property) {
                var properties = this.properties,
                    targetValue = target[property],
                    displayName = !isEmpty(this.displayName) ? this.displayName : '<anonymous>',
                    definition = properties[property];

                if (
                    !isFunction(targetValue) ||
                    (isFunction(definition) && +definition.length > +targetValue.length)
                ) {
                    throw new NotImplementedError(displayName + '#' + property + '() is not implemented.');
                }
            }

            //TODO: toRepresentation

        });

        module_(stj, function () {

            properties_(stj.Meta, {
                implement: function implement(newInterface) {
                    if (!isInstanceOf(newInterface, Interface)) {
                        throw new ValueError(newInterface + MESSAGE_NOT_INTERFACE);
                    }
                    var interfaces = this.interfaces;
                    if (!interfaces) {
                        interfaces = this.interfaces = [];
                    }

                    if (interfaces.indexOf(newInterface) < 0) {
                        interfaces.push(newInterface);
                    }
                },

                _validateInterfaces: function _validateInterfaces(target) {
                    var interfaces = this.interfaces,
                        i = interfaces.length;

                    while (i >= 0) {
                        interfaces[i].validate(target);
                        i -= 1;
                    }
                }
            });


            properties_({
                /**
                 * Shortcut a new Interface
                 *
                 * @param {string} path
                 * @param {Object} definition
                 * @return {Interface}
                 */
                interface_: function interface_(path, definition) {
                    path = namespaceNormalize(path);
                    var iface = new Interface(definition, path);
                    if (path.length > 0) {
                        assertNotRegistered(path);
                        moduleMeta(moduleCurrent()).path(path, iface);
                    }
                    return iface;
                },

                /**
                 * Add interface to `thisModule`
                 *
                 * @param {Function|Object|string} thisModule
                 * @param {Interface} definition
                 * @return {Function|Object|string}
                 */
                implement_: function implement_(thisModule, ifaces) {
                    if (isUndefined(ifaces)) {
                        ifaces = thisModule;
                        thisModule = moduleCurrent();
                    }
                    var meta = moduleMeta(thisModule);
                    forEach(isArray(ifaces) ? ifaces : [ifaces], function (iface) {
                        meta.implement(stj.use_(iface));
                    });
                    return thisModule;
                }
            });

            globals_('implement_', stj.implement_);
            globals_('interface_', stj.interface_);
        });


    });



    return use_('Interface');
});