/*jslint indent:4 */
/*global escape*/
/*global define, module_, class_, include_, use_, extend_, static_, properties_, property_, globals_ */
/**
 * @projectDescription url module
 *
 * @author Julien Polo<julien.polo@gmail.com>
 * @version 1.0.1
 */
define("stj/Url", [ "stj" ], function () {
    "use strict";

    //=========================================================================
    // Url
    //=========================================================================
    class_("Url", null, function (Url) {

        var merge = Object.update,
            size = Object.size,
            objectHas = Object.has,
            objectGet = Object.get,
            objectSet = Object.set,
            objectClone = Object.clone,
            instanceOf = Object.isInstanceOf,
            isArray = Object.isArray,
            isBoolean = Object.isBoolean,
            isString = Object.isString,
            isNumber = Object.isNumber,
            isObject = Object.isObject,
            isUndefined = Object.isUndefined,
            forEach = Object.forEach,
            stringify = String,


            PATH_SEPARATOR = '/',
            HOSTNAME_LENGTH_LIMIT = 255,
            REGEXP_PORT = /:[0-9]+$/,

            // RFC 2396: characters reserved for delimiting URLs.
            DELIMITERS = {
                '<' : true,
                '>' : true,
                '"' : true,
                '`' : true,
                ' ' : true,
                '\r' : true,
                '\n' : true,
                '\t' : true
            },

            // RFC 2396: characters not allowed for various reasons.
            UNWISE = merge({
                '{' : true,
                '}' : true,
                '|' : true,
                '\\' : true,
                '^' : true,
                '~' : true,
                '[' : true,
                ']' : true,
                '`' : true
            }, DELIMITERS),

            // Allowed by RFCs, but cause of XSS attacks. Always escape
            // these.
            AUTOESCAPE = {
                '\'' : true
            },

            // Characters that are never ever allowed in a hostname.
            // Note that any invalid chars are also handled, but these
            // are the ones that are *expected* to be seen, so we
            // fast-path
            // them.
            NONHOST = merge(merge({
                '%' : true,
                '/' : true,
                '?' : true,
                ';' : true,
                '#' : true
            }, UNWISE), DELIMITERS),

            NONAUTH = merge({
                '/' : true,
                '@' : true,
                '?' : true,
                '#' : true
            }, DELIMITERS),

            PROTOCOL_SLASHED = {
                'http' : true,
                'https' : true,
                'ftp' : true,
                'gopher' : true,
                'file' : true
            },

            _setters = {},
            _getters = {},


            indexOfAny = function indexOfAny(string, charsAsObject) {
                var i, l;
                for (i = 0, l = size(string); i < l; i += 1) {
                    if (charsAsObject[objectGet(string, i)]) {
                        return i;
                    }
                }
                return -1;
            },

            matchOne = function matchOne(regexp, string, index) {
                var result = regexp.exec(string);
                if (result) {
                    return result[index || 0];
                }
            };

        //=============================================================
        //                           static
        //=============================================================
        extend_({
            /**
             * Alias to decodeURIComponent
             *
             * @return {string}
             */
            decodeComponent: function decodeComponent(string, decodeSpaces) {
                return decodeURIComponent(string/*, decodeSpaces*/);
            },

            /**
             * Alias to encodeURIComponent
             *
             * @return {string}
             */
            encodeComponent: function encodeComponent(string) {
                return encodeURIComponent(string);
            },

            /**
             * Return a formatted querystring
             *
             * @param {Object} object,
             * @param {string=} separator ('&')
             * @param {string=} equal ('=')
             * @param {string=} name
             * @return {string}
             */
            encodeQuery: (function /*factory*/() {
                var stringifyPrimitive = function stringifyPrimitive(value) {
                        switch (typeof value) {
                        case 'string':
                            return value;

                        case 'boolean':
                            return value ? 'true' : 'false';

                        case 'number':
                            return isFinite(value) ? value : '';

                        default:
                            return '';
                        }
                    },
                    encodeKeyValue = function encodeKeyValue(key, value, equal) {
                        return encodeURIComponent(stringifyPrimitive(key)) +
                        equal +
                        encodeURIComponent(stringifyPrimitive(value));
                    },
                    iteratorKeyValue = function (value, key) {
                        var env = this, equal = env.equal, encodedValue = (
                        /*
                         isArray(value) ? value.map(function (v) {
                             return encodeKeyValue(k, v, equal);
                         }).join(separator) :
                         */
                        encodeKeyValue(key, value, equal));
                        env.__return__ += (size(env.__return__) ? env.separator : '') + encodedValue;
                    };

                return function encodeQuery(object, separator, equal, name) {
                    object = (object === null) ? undefined : object;

                    if (isObject(object)) {
                        return forEach(object, iteratorKeyValue, {
                            __return__ : '',
                            separator : separator || '&',
                            equal : equal || '='
                        });
                    } else {
                        return !name ? '' : encodeKeyValue(name,
                                object, equal);
                    }
                };
            }()),

            /**
             * Return a decoded query object
             *
             * @param {Object} object,
             * @param {string=} separator ('&')
             * @param {string=} equal ('=')
             * @return {Object}
             */
            decodeQuery: (function /*factory*/() {
                var decode = decodeURIComponent,
                    iterator = function (kvp) {
                        var env = this,
                            result = env.__return__,
                            parts = kvp.partition(env.equal),
                            key = decode(parts[0], true),
                            value = decode(parts[2], true),
                            objValue = result[key];

                        if (isUndefined(objValue)) {
                            result[key] = value;
                        } else if (!isArray(objValue)) {
                            result[key] = [ objValue, value ];
                        } else {
                            objValue.push(value);
                        }
                    };
                return function decodeQuery(string, separator, equal) {
                    if (isString(string) && size(string) > 0) {
                        //equal = equal || '=';
                        return forEach(string.split(separator || '&'), iterator, {
                                __return__ : {},
                                equal: equal || '='
                            });
                    }
                    return string;
                };
            }()),

            format: function format(urlOrString) {
                return stringify(Url.parse(urlOrString));
            },

            parse: function parse(uriOrString) {
                return instanceOf(uriOrString, Url) ? uriOrString
                        : new Url(uriOrString);
            }
        });

        //=============================================================
        //                           methods
        //=============================================================
        properties_({

            /**
             * Url constructor
             *
             * @constructor
             * @param {Object} options
             * - protocol
             * - auth
             * - hostname
             * - port
             * - path (Array)
             * - hash
             * - query (Object)
             */
            initialize : function initialize(options) {
                this.protocol = null;
                this.auth = null;
                this.hostname = null;
                this.port = null;
                this.path = [ '' ];
                this.hash = null;
                this.query = {};
                if (options) {
                    objectSet(this, 'href', options);
                }
            },

            /**
             * Return a copy of this url
             *
             * @return {Url}
             */
            clone : function clone(deep) {
                return new Url(this);
            },

            /**
             * Return string representation of url
             *
             * @return {string}
             */
            toString : function toString() {
                return objectGet(this, 'href');
            },

            /**
             * Return string representation of url
             *
             * @return {string}
             */
            valueOf : function valueOf() {
                return stringify(this);
            },

            /**
             * Return true if this is equal to `other`
             *
             * @return {boolean}
             */
            isEqual : function isEqual(other) {
                return +this === +other;
            },

            __hash__ : function __hash__() {
                return stringify(this);
            },

            __set__ : function __set__(key, value) {
                var methodName = _setters[key];
                if (!methodName) {
                    methodName = _setters[key] = '__set' + key.capitalize() + '__';
                }
                if (this[methodName]) {
                    this[methodName](value);
                } else if (objectHas(this, key)) {
                    this[key] = value;
                }
                return this;
            },

            __get__ : function __get__(key) {
                var methodName = _getters[key];
                if (!methodName) {
                    methodName = _getters[key] = '__get' + key.capitalize() + '__';
                }
                return this[methodName] ?
                        this[methodName]() :
                            objectHas(this, key) ? this[key] : null;
            },

            __getHref__ : (function /*factory*/() {
                var encoderIterator = function (c) {
                        this.__return__ = this.__return__.replace(c, encodeURIComponent(c));
                    };

                return function __getHref__() {
                    // ensure it's an object, and not a string url.
                    // If it's an obj, this is a no-op.
                    // this way, you can call url_format() on strings
                    // to clean up potentially wonky urls.

                    var auth = objectGet(this, 'auth'),
                        protocol = objectGet(this, 'protocol') || '',
                        host = objectGet(this, 'host') || false,
                        pathname = objectGet(this, 'pathname') || '',
                        search = objectGet(this, 'search') || '',
                        hash = objectGet(this, 'hash') || '',
                        i, l;

                    if (auth) {
                        auth = forEach(NONAUTH, encoderIterator, {
                            __return__: auth.replace('@', '%40')
                        });
                    }

                    // only the slashedProtocols get the //. Not
                    // mailto:, xmpp:, etc.
                    // unless they had them to begin with.
                    if ((!protocol || PROTOCOL_SLASHED[protocol]) &&
                        host !== false
                    ) {
                        host = '//' + (host || '');
                        if (pathname && !pathname.startsWith('/')) {
                            pathname = '/' + pathname;
                        }
                    } else if (!host) {
                        host = '';
                    }

                    if (protocol && protocol.substr(-1) !== ':') {
                        protocol += ':';
                    }
                    if (search && !search.startsWith('?')) {
                        search = '?' + search;
                    }
                    if (hash && !hash.startsWith('#')) {
                        hash = '#' + hash;
                    }

                    return protocol + host + pathname + search + hash;
                };
            }()),

            __setHref__ : (function /*factory*/() {

                var REGEXP_HOST = /^([a-z0-9]+):/i,


                  //hostnamePartPattern = /^[a-zA-Z0-9][a-z0-9A-Z_\-]{0,62}$/,
                    //hostnamePartStart = /^([a-zA-Z0-9][a-z0-9A-Z_\-]{0,62})(.*)$/,

                    // protocols that can allow "unsafe" and "unwise" chars.
                    PROTOCOL_UNSAFE = {
                        'javascript' : true
                    },
                    // protocols that never have a hostname.
                    PROTOCOL_HOSTLESS = {
                        'javascript' : true
                    }
                    // protocols that always have a path component.
                    /*
                     * PROTOCOL_PATHED = { 'http': true, 'https': true, 'ftp':
                     * true, 'gopher': true, 'file': true, 'http:': true,
                     * 'ftp:': true, 'gopher:': true, 'file:': true },
                     */
                    // protocols that always contain a // bit.
                    ;

                return function __setHref__(href, slashesDenoteHost) {

                    var parts,
                        rest = href,
                        protocol,
                        slashes = false,
                        i, l,

                        atSign,
                        firstNonHost;

                    if (isString(href)) {
                        // cut off any delimiters. This is to support parse stuff like "<http://foo.com>"
                        i = rest.substr(indexOfAny(rest, DELIMITERS));
                        if (i > 0) {
                            rest = rest.substr(i);
                        }

                        protocol = matchOne(REGEXP_HOST, rest, 1);
                        if (protocol) {
                            protocol = protocol.toLowerCase();
                            objectSet(this, 'protocol', protocol);
                            rest = rest.substr(size(protocol) + 1);
                        }

                        // figure self if it's got a host user@server is *always* interpreted as a
                        // hostname, and url resolution will treat //foo/bar as
                        // host=foo,path=bar because that's how the browser resolves relative URLs.
                        if (slashesDenoteHost || protocol || rest.match(/^\/\/[^@\/]+@[^@\/]+/)) {
                            slashes = rest.startsWith('//');
                            if (slashes && !(protocol && PROTOCOL_HOSTLESS[protocol])) {
                                rest = rest.substr(2);
                                // this.slashes = true;
                            }
                        }

                        if (
                            !PROTOCOL_HOSTLESS[protocol] &&
                            (slashes || (protocol && !PROTOCOL_SLASHED[protocol]))
                        ) {

                            // the first instance of /, ?, ;, or # ends the host.
                            atSign = rest.indexOf('@');
                            if (~atSign) {
                                // there *may be* an auth
                                if (indexOfAny(rest, NONAUTH) < atSign) {
                                    // pluck off the auth portion.
                                    objectSet(this, 'auth', rest.substr(0, atSign));
                                    rest = rest.substr(atSign + 1);
                                }
                            }

                            firstNonHost = indexOfAny(rest, NONHOST);
                            if (~firstNonHost) {
                                objectSet(this, 'host', rest.substr(0, firstNonHost));
                                rest = rest.substr(firstNonHost);
                            } else {
                                objectSet(this, 'host', rest);
                                rest = '';
                            }
                        }

                        // now rest is set to the post-host stuff.
                        // chop off any delim chars.
                        if (!PROTOCOL_UNSAFE[protocol]) {

                            // First, make 100% sure that any "AUTOESCAPE" chars get
                            // escaped, even if encodeURIComponent doesn't think they need to be.
                            forEach(AUTOESCAPE, function (autoEscape) {
                                var escaped = encodeURIComponent(autoEscape);
                                if (escaped === autoEscape) {
                                    escaped = escape(autoEscape);
                                    rest = rest.replace(autoEscape, escaped);
                                }

                            });

                            // Now make sure that delims never appear in a url.
                            i = indexOfAny(rest, DELIMITERS);
                            if (i >= 0) {
                                rest = rest.substr(0, i);
                            }

                        }

                        // Hash
                        parts = rest.partition('#');
                        objectSet(this, 'hash', parts[2]);
                        rest = parts[0];

                        //pathname/query
                        parts = rest.partition('?');
                        objectSet(this, 'query', parts[2]);
                        objectSet(this, 'pathname', parts[0]);

                        //if (PROTOCOL_SLASHED[protocol] && this.hostname
                        //        && !this.pathname) {
                        //    this.pathname = '/';
                        //}
                    } else {
                        forEach(this, function (_, key, self) {
                            objectSet(self, key, href[key]);
                        });
                    }

                };
            }()),

            __setProtocol__ : function __setProtocol__(value) {
                this.protocol = stringify(value).toLowerCase();
            },

            __setHostname__ : function __setHostname__(value) {
                value = stringify(value);
                if (size(value) > HOSTNAME_LENGTH_LIMIT) {
                    value = '';
                } else {

                    // lower case
                    value = value.toLowerCase();

                    // IDNA Support: Returns a puny coded representation of "domain".
                    // It only converts the part of the domain name that has non ASCII characters. I.e. it dosent
                    // matter if you call it with a domain that already is in ASCII.
                    /*
                     * var domainArray = host.split('.'), newOut =
                     * [], i, l, s; for (i = 0, l =
                     * domainArray.length; i < l; i += 1) { s =
                     * domainArray[i];
                     * newOut.push(s.match(/[^A-Za-z0-9_-]/) ?
                     * 'xn--' + punycode.encode(s) : s); }
                     * this.hostname = newOut.join('.');
                     */
                }
                this.hostname = value;
            },

            __setPort__ : function __setPort__(value) {
                this.port = isString(value) ? parseInt(value, 10) : value;
            },

            __setQuery__ : function __setQuery__(value) {
                this.query = isObject(value) ? objectClone(value) : Url.decodeQuery(value);
                return this;
            },

            __getHost__ : function __getHost__() {
                var hostname = this.hostname,
                    auth = this.auth,
                    port = this.port;
                return hostname ?
                        ((auth ? auth + '@' : '') +
                        hostname +
                        (port ? ':' + port : ''))
                        : null;
            },
            __setHost__ : function __setHost__(host) {
                var port = matchOne(REGEXP_PORT, host);
                if (port) {
                    objectSet(this, 'port', port.substr(1));
                    host = host.substr(0, size(host) - size(port));
                }
                if (host) {
                    objectSet(this, 'hostname', host);
                }
                return this;
            },

            __getPathname__ : function __getPathname__() {
                return this.path.join(PATH_SEPARATOR);
            },
            __setPathname__ : function __setPathname__(path) {
                this.path = isArray(path) ? path.slice(0) : path.split(PATH_SEPARATOR);
                return this;
            },

            __getSearch__ : function __getSearch__() {
                var queryString = Url.encodeQuery(this.query);
                return queryString ? '?' + queryString : '';
            },
            __setSearch__ : function __setSearch__(search) {
                this.query = isString(search) ? Url.decodeQuery(search) : search;
                return this;
            }

        });
    });

    return use_('Url');
});
