/*jslint indent:4 */
/*global define, module_, class_, include_, use_, extend_, static_, properties_, property_, globals_ */
/**
 * @projectDescription Assertion module (http://wiki.commonjs.org/wiki/Unit_Testing/1.0.1)
 *
 * @author Julien Polo<julien.polo@gmail.com>
 * @version 1.0.1
 */
define("stj/assert", [ "stj" ], function () {
    "use strict";

    //=========================================================================
    //                                Assert
    //=========================================================================
    module_("assert", function (assert) {

        properties_({
            AssertionError: use_('error.AssertionError'),

            /**
             * Throw a new AssertionError
             *
             * @param {*} actual
             * @param {*} expected
             * @param {string=} message
             * @param {string=} operator
             * @param {Function=} stackStartFunction
             */
            fail: function fail(actual, expected, message, operator, stackStartFunction) {
                var assertError = new assert.AssertionError({
                    message : message || "{expected} {operator} {actual}",
                    actual : actual,
                    expected : expected,
                    operator : operator
                });
                if (Error.captureStackTrace) {
                    Error.captureStackTrace(assertError, stackStartFunction || assert.fail);
                }
                throw assertError;
            },

            /**
             * Throw new AssertionError if `actual` is not true
             *
             * @param {*} actual
             * @param {string=} message
             */
            ok: function ok(actual, message) {
                if (!actual) {// !!! ?
                    assert.fail(actual, true, message, '==', ok);
                }
            },

            /**
             * Throw new AssertionError if `actual` is not equal (==) to
             * `expected`
             *
             * @param {*} actual
             * @param {*} expected
             * @param {string=} message
             */
            equal: function equal(actual, expected, message) {
                /*jslint eqeqeq:false */
                if (actual == expected) {
                    return;
                }
                /*jslint eqeqeq:true */
                assert.fail(actual, expected, message, '==', equal);
            },

            /**
             * Throw AssertionError if `actual` is equal (!=) to `expected`
             *
             * @param {*} actual
             * @param {*} expected
             * @param {string=} message
             */
            notEqual: function notEqual(actual, expected, message) {
                /*jslint eqeqeq:false */
                if (actual != expected) {
                    return;
                }
                /*jslint eqeqeq:true */
                assert.fail(actual, expected, message, '!=', notEqual);
            },

            /**
             * Throw AssertionError if `actual` is not strictly equal (===)
             * to `expected`
             *
             * @param {*} actual
             * @param {*} expected
             * @param {string=} message
             */
            strictEqual: function strictEqual(actual, expected, message) {
                if (actual === expected) {
                    return;
                }
                assert.fail(actual, expected, message, '===', strictEqual);
            },

            /**
             * Throw AssertionError if `actual` is strictly equal (!==) to
             * `expected`
             *
             * @param {*} actual
             * @param {*} expected
             * @param {string=} message
             */
            notStrictEqual: function notStrictEqual(actual, expected, message) {
                if (actual !== expected) {
                    return;
                }
                assert.fail(actual, expected, message, '!==', notStrictEqual);
            },

            /**
             * Throw AssertionError if `actual` is not recursively equal to
             * `expected`
             *
             * @param {*} actual
             * @param {*} expected
             * @param {string=} message
             */
            deepEqual: function deepEqual(actual, expected, message) {
                if (Object.isEqual(actual, expected)) {
                    return;
                }
                assert.fail(actual, expected, message, 'deepEqual', deepEqual);
            },

            /**
             * Throw AssertionError if `actual` is recursively equal to
             * `expected`
             *
             * @param {*} actual
             * @param {*} expected
             * @param {string=} message
             */
            notDeepEqual: function notDeepEqual(actual, expected, message) {
                if (!Object.isEqual(actual, expected)) {
                    return;
                }
                assert.fail(actual, expected, message, 'notDeepEqual', notDeepEqual);
            },

            /**
             * Throw AssertionError if `block` does not throw error or if
             * throwed error is not instance of `expected`
             *
             * @param {Function} block
             * @param {RegExp|Function=} expected
             * @param {string=} message
             */
            error: function error(block, expected, message) {
                var actual,
                    isUndefined = Object.isUndefined,
                    isInstanceOf = Object.isInstanceOf;
                try {
                    block();
                } catch (e) {
                    actual = e;
                }
                if (!message) {
                    message = 'Missing expected exception' + (expected && expected.name ? ' (' + expected.name + ').' : '.');
                }


                // Check error throwed
                if (!actual) {
                    assert.fail(isUndefined(actual.constructor) ? actual.constructor : actual, expected, message, 'error', error);
                }

                // Check error throwed
                if (expected) {
                    if (
                        (isInstanceOf(expected, RegExp) && expected.test(actual)) ||
                        isInstanceOf(actual, expected) || (expected.call({}, actual) === true)
                    ) {
                        return;
                    }
                    assert.fail(!isUndefined(actual.constructor) ? actual.constructor : actual, expected, message, 'error', error);
                }
            }
        });

    });

    return use_('assert');
});