/*jslint indent:4 */
/*global define, module_, class_, include_, use_, extend_, static_, properties_, property_, globals_ */

/**
 * @projectDescription ECMA ES5 shim
 *
 * @author Julien Polo<julien.polo@gmail.com>
 * @version 1.0.1
 *
 * @link https://github.com/kriskowal/es5-shim.git
 */
define("stj/es5", [ "stj" ], function (stj) {
    "use strict";

    /**
     * Shorcuts
     */
    var Function = use_('Function'),
        Void = Function.Void,

        Object = use_('Object'),
        ObjectProto = Object.prototype,
        isUndefined = Object.isUndefined,
        isNull = Object.isNull,
        isString = Object.isString,
        isObject = Object.isObject,
        isFunction = Object.isFunction,
        forEach = Object.forEach,
        hasOwn = Object.has,
        toString = ObjectProto.toString.bindToCall(),
        toObject = Object,

        Array = use_('Array'),
        ArrayProto = Array.prototype,
        slice = ArrayProto.slice.bindToCall(),

        filter = use_('filter'),
        validateObject = filter.validateObject,
        validateFunction = filter.validateFunction,

        has = use_('has'),
        HAS_ACCESSORS = hasOwn(ObjectProto, '__defineGetter__'),
        HAS_DEFINE_PROPERTY = has['object-defineproperty'],

        defineGetter = HAS_ACCESSORS ? ObjectProto.__defineGetter__.bindToCall() : Void,
        defineSetter = HAS_ACCESSORS ? ObjectProto.__defineSetter__.bindToCall() : Void,
        lookupGetter = HAS_ACCESSORS ? ObjectProto.__lookupGetter__.bindToCall() : Void,
        lookupSetter = HAS_ACCESSORS ? ObjectProto.__lookupSetter__.bindToCall() : Void,

        shim = function (target, properties) {
            forEach(properties, function (value, property) {
                if (!target[property]) {
                    stj.property_(target, property, value);
                }
            });
        },
        protoProperty = '__proto__',
        proto = function (object, newProto) {
            if (newProto) {
                object[protoProperty] = newProto;
            }
            return object[protoProperty];
        };

    /**
     * Function
     */
    shim(Function.prototype, {
        // ES-5 15.3.4.5
        // http://www.ecma-international.org/publications/files/drafts/tc39-2009-025.pdf
        bind: function bind(thisp) { // .length is 1
            var target = this, args, bound;
            if (!isFunction(target.apply) || !isFunction(target.call)) {
                throw new TypeError();
            }
            args = slice(arguments, 1); // for normal call
            bound = function () {

                if (this instanceof bound) {
                    var F = function () {}, self, result;
                    F.prototype = target.prototype;
                    self = new F();
                    result = target.apply(
                        self,
                        args.concat(slice(arguments))
                    );
                    if (!isNull(result) && toObject(result) === result) {
                        return result;
                    }
                    return self;

                } else {
                    return target.apply(
                        thisp,
                        args.concat(slice(arguments))
                    );
                }
            };
            return bound;
        }
    });


    /**
     * Array
     */
    shim(Array, {
        isArray: function isArray(obj) {
            return toString(obj) === "[object Array]";
        }
    });
    shim(ArrayProto, {
        //forEach,
        //map,
        //filter,
        //reduce,
        //TODO reduceRight
        //every,
        //some,
        //indexOf,
        //lastIndexOf: see stj.js
    });

    /**
     * Object
     */
    shim(Object, {
        // ES5 15.2.3.2
        getPrototypeOf: (function /*factory*/() {
            return (hasOwn({}, proto) ? function prototypeOf(object) {
                return proto(object);
            } : function prototypeOf(object) {
                return object.constructor.prototype;
            });
        }()),

        // ES5 15.2.3.4
        getOwnPropertyNames: function getOwnPropertyNames(object) {
            return Object.keys(object);
        },

        // ES5 15.2.3.5
        create: function create(prototype, properties) {
            var object, Type;
            if (isNull(prototype)) {
                object = {
                    "__proto__": null
                };
            } else {
                if (typeof prototype !== "object") {
                    throw new TypeError("typeof prototype[" + (typeof prototype) + "] != 'object'");
                }
                Type = function () {};
                Type.prototype = prototype;
                object = new Type();
                // IE has no built-in implementation of `Object.getPrototypeOf`
                // neither `__proto__`, but this manually setting `__proto__` will
                // guarantee that `Object.getPrototypeOf` will work as expected with
                // objects created using `Object.create`
                proto(object, prototype);
            }
            if (!isUndefined(properties)) {
                Object.defineProperties(object, properties);
            }
            return object;
        },

        // ES5 15.2.3.3
        getOwnPropertyDescriptor: function getOwnPropertyDescriptor(object, property) {
            validateObject(object);
            // If object does not owns property return undefined immediately.
            if (!hasOwn(object, property)) {
                return;
            }
            // If object has a property then it's for sure both `enumerable` and
            // `configurable`.
            var descriptor =  {
                    enumerable: true,
                    configurable: true
                },
                prototype,
                getter,
                setter;


            // If JS engine supports accessor properties then property may be a
            // getter or setter.
            if (HAS_ACCESSORS) {
                // Unfortunately `__lookupGetter__` will return a getter even
                // if object has own non getter property along with a same named
                // inherited getter. To avoid misbehavior we temporary remove
                // `__proto__` so that `__lookupGetter__` will return getter only
                // if it's owned by an object.
                prototype = proto(object);
                proto(object, ObjectProto);

                getter = lookupGetter(object, property);
                setter = lookupSetter(object, property);

                // Once we have getter and setter we can put values back.
                proto(object, prototype);

                if (getter || setter) {
                    if (getter) {
                        descriptor.get = getter;
                    }
                    if (setter) {
                        descriptor.set = setter;
                    }

                    // If it was accessor property we're done and return here
                    // in order to avoid adding `value` to the descriptor.
                    return descriptor;
                }
            }

            // If we got this far we know that object has an own property that is
            // not an accessor so we set it as a value and return descriptor.
            descriptor.value = object[property];
            return descriptor;
        },

        // ES5 15.2.3.6
        defineProperty: (function /*factory*/() {
            var definePropertyNative = Object.defineProperty,
                ERR_NON_OBJECT_DESCRIPTOR = "Property description must be an object: ",
                ERR_ACCESSORS_NOT_SUPPORTED = "getters & setters can not be defined on this javascript engine";
            if (!HAS_DEFINE_PROPERTY) {
                return function defineProperty(object, property, descriptor) {
                    validateObject(object);
                    if (typeof descriptor !== "object" || descriptor === null) {
                        throw new TypeError(ERR_NON_OBJECT_DESCRIPTOR + descriptor);
                    }
                    // make a valiant attempt to use the real defineProperty
                    // for I8's DOM elements.
                    if (definePropertyNative && object.nodeType) {
                        return definePropertyNative(object, property, descriptor);
                    }

                    // If it's a data property.
                    if (hasOwn(descriptor, "value")) {
                        // fail silently if "writable", "enumerable", or "configurable"
                        // are requested but not supported
                        /*
                        // alternate approach:
                        if ( // can't implement these features; allow false but not true
                            !(owns(descriptor, "writable") ? descriptor.writable : true) ||
                            !(owns(descriptor, "enumerable") ? descriptor.enumerable : true) ||
                            !(owns(descriptor, "configurable") ? descriptor.configurable : true)
                        )
                            throw new RangeError(
                                "This implementation of Object.defineProperty does not " +
                                "support configurable, enumerable, or writable."
                            );
                        */

                        if (HAS_ACCESSORS && (lookupGetter(object, property) ||
                                                  lookupSetter(object, property)))
                        {
                            // As accessors are supported only on engines implementing
                            // `__proto__` we can safely override `__proto__` while defining
                            // a property to make sure that we don't hit an inherited
                            // accessor.
                            var prototype = proto(object);
                            proto(object, ObjectProto);
                            // Deleting a property anyway since getter / setter may be
                            // defined on object itself.
                            delete object[property];
                            object[property] = descriptor.value;
                            // Setting original `__proto__` back now.
                            proto(object, prototype);
                        } else {
                            object[property] = descriptor.value;
                        }
                    } else {
                        if (!HAS_ACCESSORS) {
                            throw new TypeError(ERR_ACCESSORS_NOT_SUPPORTED);
                        }
                        // If we got that far then getters and setters can be defined !!
                        if (hasOwn(descriptor, "get")) {
                            defineGetter(object, property, descriptor.get);
                        }
                        if (hasOwn(descriptor, "set")) {
                            defineSetter(object, property, descriptor.set);
                        }
                    }
                    return object;
                };
            }
        }()),

        // ES5 15.2.3.7
        defineProperties: function defineProperties(object, properties) {
            var property,
                defineProperty = Object.defineProperty;
            for (property in properties) {
                if (hasOwn(properties, property)) {
                    defineProperty(object, property, properties[property]);
                }
            }
            return object;
        },

        // ES5 15.2.3.8
        seal: function seal(object) {
            return object;
        },

        // ES5 15.2.3.9
        freeze: function freeze(object) {
            return object;
        },

        // ES5 15.2.3.10
        preventExtensions: function preventExtensions(object) {
            return object;
        },

        // ES5 15.2.3.11
        isSealed: function isSealed(object) {
            return false;
        },

        // ES5 15.2.3.12
        isFrozen: function isFrozen(object) {
            return false;
        },

        // ES5 15.2.3.13
        isExtensible: function isExtensible(object) {
            return true;
        },

        // ES5 15.2.3.14
        // http://whattheheadsaid.com/2010/10/a-safer-object-keys-compatibility-implementation
        keys: (function /*factory*/() {

            var hasDontEnumBug = true,
                dontEnums = [
                    "toString",
                    "toLocaleString",
                    "valueOf",
                    "hasOwnProperty",
                    "isPrototypeOf",
                    "propertyIsEnumerable",
                    "constructor"
                ],
                key;

            //detect bug
            /*jslint forin:true */
            for (key in {"toString": null}) {
                hasDontEnumBug = false;
            }
            /*jslint forin:false*/

            return function keys(object) {
                validateObject(object);

                var results = [], name, dontEnum, i;
                for (name in object) {
                    if (hasOwn(object, name)) {
                        results.push(name);
                    }
                }

                if (hasDontEnumBug) {
                    i = dontEnums.length - 1;
                    while (i >= 0) {
                        dontEnum = dontEnums[i];
                        if (hasOwn(object, dontEnum)) {
                            results.push(dontEnum);
                        }
                        i -= 1;
                    }
                }

                return keys;
            };
        }())
    });

    /**
     * Date
     */
    shim(Date, {
        // ES5 15.9.4.4
        now: function now() {
            return (new Date()).getTime();
        }
    });
    shim(Date.prototype, {
        // ES5 15.9.5.43
        // Format a Date object as a string according to a simplified subset of the ISO 8601
        // standard as defined in 15.9.1.15.
        toISOString: function toISOString() {
            if (!isFinite(this)) {
                throw new RangeError();
            }
            // the date time string format is specified in 15.9.1.15.
            var result = [this.getUTCFullYear(), this.getUTCMonth() + 1, this.getUTCDate(),
                          this.getUTCHours(), this.getUTCMinutes(), this.getUTCSeconds()],
                length = result.length,
                value;

            while (length >= 0) {
                value = result[length];
                // pad months, days, hours, minutes, and seconds to have two digits.
                if (value < 10) {
                    result[length] = '0' + value;
                }
                length -= 1;
            }
            // pad milliseconds to have three digits.
            return result.slice(0, 3).join('-') + 'T' + result.slice(3).join(':') + '.' +
                ('000' + this.getUTCMilliseconds()).slice(-3) + 'Z';
        },

        // ES5 15.9.5.44
        toJSON: function toJSON(key) {
            // This function provides a String representation of a Date object for
            // use by JSON.stringify (15.12.3). When the toJSON method is called
            // with argument key, the following steps are taken:

            // 1.  Let O be the result of calling ToObject, giving it the this
            // value as its argument.
            // 2. Let tv be ToPrimitive(O, hint Number).
            // 3. If tv is a Number and is not finite, return null.
            // XXX
            // 4. Let toISO be the result of calling the [[Get]] internal method of
            // O with argument "toISOString".
            // 5. If IsCallable(toISO) is false, throw a TypeError exception.
            // XXX this gets pretty close, for all intents and purposes, letting
            // some duck-types slide
            var toISOString = this.toISOString;
            validateFunction(toISOString.call);

            // 6. Return the result of calling the [[Call]] internal method of
            // toISO with O as the this value and an empty argument list.
            return toISOString.call(this);

            // NOTE 1 The argument is ignored.

            // NOTE 2 The toJSON function is intentionally generic; it does not
            // require that its this value be a Date object. Therefore, it can be
            // transferred to other kinds of objects for use as a method. However,
            // it does require that any such object have a toISOString method. An
            // object is free to use the argument key to filter its
            // stringification.
        }
    });

    if (isNaN(Date.parse("2011-06-15T21:40:05+06:00"))) {
        // XXX global assignment won't work in embeddings that use
        // an alternate object for the context.
        globals_('Date', (function (NativeDate) {

            // Date.length === 7
            var
            NewDate = function Date(Y, M, D, h, m, s, ms) {
                var args = arguments,
                    length = args.length,
                    date;
                if (this instanceof NativeDate) {
                    date = length === 1 && isString(Y) ?
                        // We explicitly pass it through parse:
                        new NativeDate(NewDate.parse(Y)) :
                        // We have to manually make calls depending on argument
                        // length here
                        length >= 7 ? new NativeDate(Y, M, D, h, m, s, ms) :
                        length >= 6 ? new NativeDate(Y, M, D, h, m, s) :
                        length >= 5 ? new NativeDate(Y, M, D, h, m) :
                        length >= 4 ? new NativeDate(Y, M, D, h) :
                        length >= 3 ? new NativeDate(Y, M, D) :
                        length >= 2 ? new NativeDate(Y, M) :
                        length >= 1 ? new NativeDate(Y) :
                                      new NativeDate();
                    // Prevent mixups with unfixed Date object
                    date.constructor = NewDate;
                    return date;
                }
                return NativeDate.apply(this, args);
            },

            // 15.9.1.15 Date Time String Format. This pattern does not implement
            // extended years ((15.9.1.15.1), as `Date.UTC` cannot parse them.
            isoDateExpression = new RegExp("^" +
                "(\\d{4})" + // four-digit year capture
                "(?:-(\\d{2})" + // optional month capture
                "(?:-(\\d{2})" + // optional day capture
                "(?:" + // capture hours:minutes:seconds.milliseconds
                    "T(\\d{2})" + // hours capture
                    ":(\\d{2})" + // minutes capture
                    "(?:" + // optional :seconds.milliseconds
                        ":(\\d{2})" + // seconds capture
                        "(?:\\.(\\d{3}))?" + // milliseconds capture
                    ")?" +
                "(?:" + // capture UTC offset component
                    "Z|" + // UTC capture
                    "(?:" + // offset specifier +/-hours:minutes
                        "([-+])" + // sign capture
                        "(\\d{2})" + // hours offset capture
                        ":(\\d{2})" + // minutes offest capture
                    ")" +
                ")?)?)?)?" +
            "$");

            // Copy any custom methods a 3rd party library may have added
            Object.update(NewDate, NativeDate, true);

            // Copy "native" methods explicitly; they may be non-enumerable
            NewDate.now = NativeDate.now;
            NewDate.UTC = NativeDate.UTC;
            NewDate.prototype = NativeDate.prototype;
            NewDate.prototype.constructor = NewDate;

            // Upgrade Date.parse to handle simplified ISO 8601 strings
            NewDate.parse = function parse(string) {
                var match = isoDateExpression.exec(string),
                    i,
                    minutesOffset,
                    hourOffset,
                    sign,
                    offset = 0;
                if (match) {
                    match.shift(); // kill match[0], the full match
                    // parse months, days, hours, minutes, seconds, and milliseconds
                    for (i = 1; i < 7; i += 1) {
                        // provide default values if necessary
                        match[i] = +(match[i] || (i < 3 ? 1 : 0));
                        // match[1] is the month. Months are 0-11 in JavaScript
                        // `Date` objects, but 1-12 in ISO notation, so we
                        // decrement.
                        if (i === 1) {
                            match[i] -= 1;
                        }
                    }

                    // parse the UTC offset component
                    minutesOffset = +match.pop();
                    hourOffset = +match.pop();
                    sign = match.pop();

                    // compute the explicit time zone offset if specified
                    if (sign) {
                        // detect invalid offsets and return early
                        if (hourOffset > 23 || minutesOffset > 59) {
                            return NaN;
                        }

                        // express the provided time zone offset in minutes. The offset is
                        // negative for time zones west of UTC; positive otherwise.
                        offset = (hourOffset * 60 + minutesOffset) * 6e4 * (sign === "+" ? -1 : 1);
                    }

                    // compute a new UTC date value, accounting for the optional offset
                    return NativeDate.UTC.apply(this, match) + offset;
                }
                return NativeDate.parse.apply(this, arguments);
            };
            return NewDate;
        }(Date)), true);
    }

    /**
     * String
     */
    shim(String.prototype, {
        // ES5 15.5.4.20
        trim: function trim() {
            return this.replace(/^\s+|\s+$/g, '');
        }
    });
});