/*jslint indent:4 */
/*global define, module_, class_, include_, use_, extend_, static_, properties_, property_, globals_ */
/**
 * @projectDescription inflection module
 *
 * @author Julien Polo<julien.polo@gmail.com>
 * @version 1.0.1
 */
define("stj/inflect", [ "stj" ], function () {
    "use strict";

    /**
     * @module inflect
     */
    module_("inflect", function (inflect) {
        var isUndefined = Object.isUndefined,
            stringify = String,
            REGEXP_DASH = /_/g;

        properties_({
            /**
             * Return an camelized string
             *
             * Example:
             *     inflect.camelize('foo-bar-baz'); //-> fooBarBaz
             *
             * @param {string} string
             * @return {string}
             */
            camelize: (function /*factory*/() {
                var matcher = /[\-_]+(\D?)/g,
                    replacer = function (_, char) {
                        return char.toUpperCase();
                    };
                return function camelize(string) {
                    return stringify(string).replace(matcher, replacer);
                };
            }()),

            /**
             * Return an capitalized string
             *
             * Example:
             *     inflect.capitalize('foo-bar-baz'); //-> Foo-bar-baz
             *
             * @param {string} string
             * @return {string}
             */
            capitalize: function capitalize(string) {
                return stringify(string).capitalize();
            },

            /**
             *
             *
             * @param {string} string
             * @return {string}
             */
            dasherize: function dasherize(string) {
                return stringify(string).replace(REGEXP_DASH, '-');
            },

            /**
             * Return an lower case string
             *
             * Example:
             *     inflect.lower('Foo-bAR-baz'); //-> foo-bar-baz
             *
             * @param {string} string
             * @return {string}
             */
            lower: function lower(string) {
                return stringify(string).toLowerCase();
            },

            /**
             * Truncate the string to length and append suffix if
             * truncated
             *
             * Example:
             *     inflect.truncate('Foo-bAR-baz'); //-> foo-bar-baz
             *
             * @param {string} string
             * @param {number} length
             * @param {string} suffix
             * @return {string}
             */
            truncate: function truncate(string, length, suffix) {
                if (isUndefined(length)) {
                    length = 30;
                }

                suffix = stringify(suffix || '...');
                string = stringify(string);
                return string.length > length ? string.slice(0, length - suffix.length) + suffix : string;
            },

            /**
             *
             *
             * @param {string} string
             * @return {string}
             */
            underscore: function underscore(string) {
                return string.replace(/::/g, '/')
                        .replace(/([A-Z]+)([A-Z][a-z])/g, '$1_$2')
                        .replace(/([a-z\d])([A-Z])/g, '$1_$2')
                        .replace(/-/g, '_')
                        .toLowerCase();
            },

            /**
             * Return an upper case string
             *
             * Example:
             *     inflect.lower('Foo-bAR-baz'); //-> FOO-BAR-BAZ
             *
             * @param {string} string
             * @return {string}
             */
            upper: function upper(string) {
                return stringify(string).toUpperCase();
            }
        });
    });

    /**
     * Extend string
     */
    module_(String, function () {
        var inflection = use_('inflect');
        properties_({
            toCamelCase: function toCamelCase() {
                return inflection.camelize(this);
            }
        });
    });

    return use_('inflect');
});