/*jslint nodejs:true, indent:4 */
/*global define, module_, class_, include_, use_, extend_, static_, properties_, property_, globals_ */
/**
 * @projectDescription MIME type module
 *
 * @author Julien Polo<julien.polo@gmail.com>
 * @version 1.0.1
 */
define("stj/mime", [ "stj" ], function () {
    "use strict";

    /**
     * @module mime
     */
    module_("mime", function () {

        properties_({
            /**
             * Default value for unknown extensions
             */
            defaultType: "application/octet-stream",

            /**
             * Map of extension to mime type
             */
            _extensionToType: {},

            /**
             * Map of mime type to extension
             */
            _mimeToExtension : {},

            /**
             * Define mimetype -> extension mappings.  Each key is a mime-type that maps
             * to an array of extensions associated with the type.  The first extension is
             * used as the default extension for the type.
             *
             * e.g. mime.add({'audio/ogg': ['oga', 'ogg', 'spx']});
             *
             * @param {Object} map (Object) type definitions
             * @return this
             */
            add: function add(map) {
                var types = this._types,
                    extensions = this._extensions;
                Object.forEach(map, function (newExtensions, type) {
                    type = this._sanitizeMimetype(type);

                    var n = newExtensions.length - 1;
                    while (n >= 0) {
                        types[this._sanitizeExtension(newExtensions[n])] = type;
                        n -= 1;
                    }

                    //first extension is default extension
                    extensions[type] = newExtensions[0];
                });
                return this;
            },

            /**
             * Lookup a mime type based on extension
             *
             * @param {string} fullFileName
             * @param {string=} defaultValue
             * @return {string}
             */
            lookup: function lookup(extensionName, defaultValue) {
                return this._extensionToType[this._sanitizeExtension(extensionName)] || defaultValue || this.defaultType;
            },

            /**
             * Return file extension associated with a mime type
             *
             * @param {string} mimeType
             * @param {string}
             */
            extension: function extension(mimeType) {
                return this._mimeToExtension[this._sanitizeMimeType(mimeType)];
            },

            /**
             * Lookup a charset based on mime type.
             */
            /*charsets: {
              lookup: function (mimeType, fallback) {
                // Assume text types are utf8.  Modify mime logic as needed.
                return /^text\//.test(mimeType) ? 'UTF-8' : fallback;
              }
            },*/

            _sanitizeExtension: function (extensionName) {
                return extensionName.toLowerCase();
            },

            _sanitizeMimetype: function (mimeType) {
                return mimeType.toLowerCase();
            }
        });
    });

    return use_('mime');
});
