/*jslint nodejs:true, browser: true, indent:4 */
/*global module_, class_, include_, use_, extend_, static_, properties_, property_, globals_ */
/*global window, define, dojo, Components, MessageChannel */
/**
 * The MIT License
 *
 * Copyright (c) 2010 Julien Polo (author)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
/**
 * @projectDescription STD library module
 *
 * @author Julien Polo<julien.polo@gmail.com>
 * @version 1.2.0
 */
(function (define, Object, Array, Boolean, Date, Function, Math, Number, RegExp, String, Error, TypeError) {

    //=========================================================================
    //                                STD definition
    //=========================================================================
    define("stj", [], function define_stj() {
        "use strict";

        //=====================================================================
        //                              CONSTANTS
        //=====================================================================
        var
            ValueError = Error,//redefined in error module
            stj,

            TYPE_UNDEFINED = 'undefined',
            TYPE_STRING = 'string',
            TYPE_BOOLEAN = 'boolean',
            TYPE_NUMBER = 'number',
            TYPE_OBJECT = 'object',
            TYPE_FUNCTION = 'function',
            HAS_CONSOLE = typeof console !== TYPE_UNDEFINED,
            HAS_PROCESS = typeof process !== TYPE_UNDEFINED,//for node
            HAS_WINDOW = typeof window !== TYPE_UNDEFINED,
            HAS_DEFINE_PROPERTY = (function /*factory*/() {
                var obj = {};
                try {
                    Object.defineProperty(obj, "t", {
                        get: function () {
                            return true;
                        }
                    });
                    //check getter
                    return (obj.t);
                } catch (e) {
                    //just fail and return false
                }

                return false;
            }()),
            HAS_FUNCTION_NAME = (function /*factory*/() {
                function t() {}
                return t.name === "t";
            }()),

            // RegExp escape matcher
            EOL = "\n",
            GLOBAL = HAS_WINDOW ? window : global,

            NAMESPACE_SEPARATORS = [ '.', '/' ], //first one . is the default that should be used
            NAMESPACE_SEPARATOR = NAMESPACE_SEPARATORS[0],
            NAMESPACE_SEPARATOR_METHOD = '#',

            //=============================================================
            //                          messages
            //=============================================================
            MESSAGE_ALREADY_DECLARED = " is already declared",
            MESSAGE_NOT_DEFINED = ' is not defined',
            MESSAGE_NOT_OBJECT = ' is not an object',
            MESSAGE_NOT_STRING = ' is not a string',
            MESSAGE_NOT_FUNCTION = ' is not a function',
            MESSAGE_NOT_MODULE = ' is not a module',
            MESSAGE_NOT_CLASS = ' is not a class',

            //=============================================================
            //                          utils
            //=============================================================
            /**
             * Shortcuts to math functions
             */
            abs = Math.abs,
            floor = Math.floor,
            ceil = Math.ceil,
            min = Math.min,
            max = Math.max,
            random = Math.random,

            /**
             * Shortcut to Object(anyObject)
             */
            toObject = Object,

            /**
             * Convert to integer
             */
            toInteger = function toInteger(number) {
                number = +number;
                return (!number || !isFinite(number)) ? (number || 0) : ((number > 0 || -1) * floor(abs(number)));
            },

            /**
             * Shortcut to String(anyObject)
             */
            stringify = String,

            /**
             * Return an object representing enum choices (from string or array)
             */
            choices = function choices(values) {
                var result = {},
                    length = values.length - 1;
                while (length >= 0) {
                    result[values[length]] = true;
                    length -= 1;
                }
                return result;
            },

            /**
             * Return true if obj has own property
             */
            hasOwn = function (obj, key) {
                return obj && obj.hasOwnProperty(key);
            },

            /**
             * Return obj own property value
             */
            getOwn = function getOwn(obj, key, defaultValue) {
                return hasOwn(obj, key) ? obj[key] : defaultValue;
            },

            /**
             * Shortcut/shim to indexOf
             */
            indexOf = function indexOf(list, value, fromIndex) {
                if (list.indexOf) {
                    return list.indexOf(value, fromIndex);
                } else {
                    var self = toObject(list),
                        length = toInteger(self.length),
                        i = fromIndex || 0;
                    if (i < length) {
                        if (i < 0) {
                            i += length;
                        }
                        while (i < length) {
                            if (hasOwn(self, i) && value === self[i]) {
                                return i;
                            }
                            i += 1;
                        }
                    }

                    return -1;
                }
            },

            /**
             * Shortcut/shim to lastIndexOf
             */
            lastIndexOf = function lastIndexOf(enumerable, item, fromIndex) {
                return enumerable.lastIndexOf(item, fromIndex);
            },

            isBoolean = function isBoolean(obj) {
                return (typeof obj === TYPE_BOOLEAN);
            },

            isArray =  function isArray(obj) {
                return ((typeof obj === TYPE_OBJECT) && obj instanceof Array);
            },

            isNumber = function isNumber(obj) {
                return (typeof obj === TYPE_NUMBER) && isFinite(obj);
            },

            isInteger = function isInteger(obj) {
                return (typeof obj === TYPE_NUMBER) && floor(obj) === obj;
            },

            isString = function isString(obj) {
                return (typeof obj === TYPE_STRING);
            },

            isFunction = function isFunction(obj) {
                return (typeof obj === TYPE_FUNCTION);
            },

            isNull = function isNull(obj) {
                return obj === null;
            },

            isUndefined = function isUndefined(obj) {
                return typeof obj === TYPE_UNDEFINED;
            },

            isPrimitive = (function /*factory*/() {
                var ENUM_TYPE = choices(["function", "object"]);
                return function isPrimitive(obj) {
                    return (obj === null) || !ENUM_TYPE[typeof obj];//not function, object or undefined
                };
            }()),

            isObject = function isObject(obj) {
                return !isPrimitive(obj);
            },

            validateFunction = function validateFunction(obj) {
                if (!isFunction(obj)) {
                    throw new TypeError(obj + MESSAGE_NOT_FUNCTION);
                }
                return obj;
            },

            /**
             * Return function name
             */
            functionName = (function /*factory*/() {
                var REGEXP_FUNCTION_HEADER = /^\s+function\s+/;
                return HAS_FUNCTION_NAME ?
                    function functionName(func) {
                        return func.name;
                    }:
                    function functionName(func) {
                        var name = func.name, scriptContent;
                        if (!name) {
                            scriptContent = stringify(func).replace(REGEXP_FUNCTION_HEADER, "");

                            name = scriptContent.slice(0, indexOf(scriptContent, '('));
                            try {
                                func.name = name;
                            } catch (e) {
                                //ignore errors
                            }
                        }
                        return name;
                    };
            }()),

            /**
             * Shortcut to function bind
             */
            functionCall = Function.prototype.call,
            functionBindThis = (function factory(functionBind) {
                return functionBind ? function (func, self) {
                    return functionBind.call(func, self);
                } : function (func, object) {
                    return function () {
                        return func.apply(object, arguments);
                    };
                };
            }(Function.prototype.bind)),
            functionBindToCall = function (func) {
                return functionBindThis(functionCall, func);
            },

            /**
             * Return an array of keys for obj
             */
            objectKeys = (function /*factory*/() {
                return Object.keys ? Object.keys : function objectKeys(obj) {
                    var result = [], property;
                    for (property in obj) {
                        if (hasOwn(obj, property)) {
                            result.push(property);
                        }
                    }
                    return result;
                };
            }()),

            /**
             * Shortcut to copy or slice a part of an array
             */
            slice = functionBindToCall(Array.prototype.slice),

            /**
             * Shortcut to iterate generically in an object
             */
            StopIteration = function () {}, //defined in error module
            stopIteration = function stopIteration() {
                throw new StopIteration();
            },
            returnSet = function (args, returnValue) {
                args.__return__ = returnValue;
                return args;
            },
            returnGet = function (args, defaultValue) {
                return getOwn(args, '__return__', defaultValue);
            },
            forEach = function forEach(obj, iterator/*, thisp, options */) {
                var key,
                    thisp = arguments[2];
                validateFunction(iterator);
                if (obj) {
                    try {
                        if (obj.__forEach__) {
                            obj.__forEach__(iterator, thisp, arguments[3]);
                        } else {
                            obj = toObject(obj);
                            for (key in obj) {
                                if (hasOwn(obj, key)) {
                                    iterator.call(thisp, obj[key], key, obj);
                                }
                            }
                        }
                    } catch (e) {
                        if (!(e instanceof StopIteration)) {
                            throw e;//rethrow
                        }
                    }
                }
                return returnGet(thisp, obj);
            },

            /**
             * Mixin utility
             */
            merge = (function /*factory*/() {
                var
                    mergeOverwrite = function (_, otherValue) {
                        return otherValue;
                    },
                    mergeKeep = function (value, _) {
                        return value;
                    },
                    mergeIterator = function (extensionValue, property, extension) {
                        var
                            object = this.o,
                            conflictHandler = this.h,
                            objectValue;

                        //Possible conflict?
                        if (hasOwn(object, property)) {
                            objectValue = object[property];
                            extensionValue = conflictHandler(objectValue, extensionValue, property);
                            if (extensionValue !== objectValue) {
                                object[property] = extensionValue;
                            }

                        //No conflict
                        } else {
                            object[property] = extensionValue;
                        }
                    };

                return function merge(object, extension, _conflictHandler, _useForEach) {
                    if (extension) {
                        var env = {
                                o: object,
                                h: (isUndefined(_conflictHandler) || _conflictHandler === true) ? mergeOverwrite :
                                    (!_conflictHandler ? mergeKeep : _conflictHandler)
                            },
                            property;

                        if (!_useForEach) {
                            for (property in extension) {
                                if (hasOwn(extension, property)) {
                                    mergeIterator.call(env, extension[property], property, extension);
                                }
                            }
                        } else {
                            forEach(extension, mergeIterator, env);
                        }

                    }
                    return object;
                };
            }()),


            /**
             * Improved shortcut to script eval
             */
            runScript = function runScript(javascriptCode, uriOrFilename) {
                if (uriOrFilename) {
                    javascriptCode += EOL + "//@ sourceURL=" + uriOrFilename + EOL;// Firebug hint
                }

                var result;
                /*jslint evil:true*/
                result = GLOBAL.eval ? GLOBAL.eval(javascriptCode) : eval(javascriptCode);
                /*jslint evil:false*/
                return result;
            },

            /**
             * Return a new constructor function with optional `methods`
             */
            createConstructor = function (baseName, methods) {
                //TODO : filter non allowed characters in baseName
                var key = "_eval" + ceil(random() * 9999999),
                    source =
                        //t: this, a: arguments, C: constructor, i: initialize, r: result,
                        key + "=(function " + (baseName || '') + "($1) {" + EOL +
                        "  var t = this, a = arguments, C = a.callee, i, r, flag='-_..&}{/@(';" + EOL + //flag is a marker to pass initialize
                        "  if (!(t instanceof C)) {" + EOL +
                        "    t = new C(flag);" + EOL + // constructor was called directly without new operator
                        "  }" + EOL +
                        "  r = t;" + EOL +
                        "  i = t.initialize;" + EOL +
                        "  if (i && $1 !== flag) {" + EOL +
                        "    r = i.apply(t, a) || t;" + EOL +
                        "  }" + EOL +
                        "  return r;" + EOL +
                        "});",
                    result;

                runScript(source, "stj.js");
                result = GLOBAL[key];
                GLOBAL[key] = null;

                //add methods
                merge(result.prototype, methods);
                return result;
            },

            /**
             * Shortcut/shim to Object.defineProperty
             */
            propertyDefine = HAS_DEFINE_PROPERTY ? Object.defineProperty : function propertyDefine(object, name, definition) {
                if (hasOwn(definition, 'value')) {
                    object[name] = definition.value;
                }
            },

            /**
             * Return object property descriptor
             */
            propertyDescriptor = HAS_DEFINE_PROPERTY ?
                function propertyDescriptor(object, property) {
                    var descriptor = Object.getOwnPropertyDescriptor(object, property);
                    return descriptor ? descriptor :
                        hasOwn(object, property) ? {
                            value: object[property],
                            enumerable: true,
                            configurable: true,
                            writable: (!isFunction(object) || property !== 'name')
                        } : undefined;
                } :
                function propertyDescriptor(object, property) {
                    if (isNull(object) || !isObject(object)) {
                        throw new TypeError(object + MESSAGE_NOT_OBJECT);
                    }
                    // If object does not owns property return undefined immediately.
                    return hasOwn(object, property) ? {
                        enumerable: true,
                        configurable: true,
                        writable: (!isFunction(object) || property !== 'name'),
                        value: object[property]
                    } : undefined;
                },

            /**
             * Define non enumerable property
             */
            propertyHidden = function (obj, propertyName, value) {
                if (obj[propertyName] !== value) {
                    propertyDefine(obj, propertyName,
                        merge({
                            value: value,
                            enumerable: false
                        },
                        propertyDescriptor(obj, propertyName),
                        false
                        )
                    );

                }
            },

            /**
             * Return an array of property names
             */
            propertyNames = HAS_DEFINE_PROPERTY ? Object.getOwnPropertyNames : objectKeys,

            /**
             * Shortcut to return the prototype of an object
             */
            prototypeOf = (function /*factory*/() {
                //Default implementation
                var _native = Object.getPrototypeOf,
                    proto = '__proto__';

                return _native ? _native :
                    //shim
                    (hasOwn({}, proto) ? function getPrototypeOf(object) {
                        return object[proto];
                    } : function getPrototypeOf(object) {
                        return object.constructor.prototype;
                    });
            }());


        // Build global (this is the only one)
        if (GLOBAL.__STD__) {
            //Do not declare twice, maybe should be handled
            throw new Error('stj' + MESSAGE_ALREADY_DECLARED);
        }

        (function define_oop() {
            var

            //=====================================================================
            //                              NAMESPACES
            //=====================================================================
            /**
             * Return a normalized path to module
             *
             * @param {string} ns path to normalize
             * @return {string}
             */
            namespaceNormalize = function namespaceNormalize(ns) {
                ns = ns.replace(NAMESPACE_SEPARATORS, NAMESPACE_SEPARATOR);

                if (ns.charAt(0) === NAMESPACE_SEPARATOR) {
                    ns = ns.substr(1);
                }
                if (ns.charAt(ns.length - 1) === NAMESPACE_SEPARATOR) {
                    ns = ns.substr(0, ns.length - 1);
                }
                return ns;
            },

            /**
             * Split arguments on first path separator occurence.
             * Return a [part1, separator, part2] array.
             *
             * @return {Array}
             */
            namespacePartition = function namespacePartition(ns) {
                var separatorPosition = indexOf(ns, NAMESPACE_SEPARATOR);
                return ~separatorPosition ?
                       [ns.substr(0, separatorPosition), NAMESPACE_SEPARATORS, ns.substr(separatorPosition + 1)] :
                       [ns, '', ''];
            },

            /**
             * Join all parts with namespace separator
             *
             * @param {Array} parts
             * @param {string=} methodName optional method suffix
             * @return {string}
             */
            namespaceJoin = function namespaceJoin(parts, methodName) {
                var paths = slice(parts) || [],
                    filtered = [], i = 0, length = paths.length, part;
                while (i < length) {
                    part = paths[i];
                    if (isString(part)) {
                        filtered.push(part);
                    }
                    i += 1;
                }

                return namespaceNormalize(filtered.join(NAMESPACE_SEPARATOR)) +
                           (methodName ? NAMESPACE_SEPARATOR_METHOD + methodName : '');
            },

            /**
             * Return last part of a `fullName` namespace
             *
             * @return {string}
             */
            namespaceBase = function namespaceBase(fullName) {
                var parts = fullName.split(NAMESPACE_SEPARATOR);
                return parts[parts.length - 1];
            },


            //=====================================================================
            //                              Module
            //=====================================================================
            Meta,

            moduleRoot = {},//root of all modules
            moduleToId = [],//Array of id => module
            moduleRegistry = {}, //Object of id => meta


            moduleQueue = [], //module resolution queue
            moduleCurrentId = 0, // current module id
            moduleCurrent = moduleRoot,
            moduleCurrentIsStatic = false,


            /**
             * Return the `target` module identifier
             *
             * @return {number}
             */
            moduleId = function moduleId(target) {
                if (HAS_DEFINE_PROPERTY) {
                    var meta = getOwn(target, '__stjMeta__');
                    return meta ? meta._id : -1;
                } else {
                    return indexOf(moduleToId, target);
                }
            },

            /**
             * Return true if obj is a module
             *
             * @return {boolean}
             */
            isModule = function isModule(obj) {
                return (isFunction(obj) || isObject(obj)) && (moduleId(obj) >= 0);
            },

            /**
             * Return true if obj is a class
             *
             * @return {boolean}
             */
            isClass = function isClass(obj) {
                return isFunction(obj) && (moduleId(obj) >= 0);
            },

            /**
             * Return the meta information object for `target`
             *
             * @param {Module} target
             * @return {Meta}
             */
            moduleMeta = function moduleMeta(target) {
                var id = moduleId(target);
                if (!~id) {
                    throw new TypeError(target + MESSAGE_NOT_MODULE);
                }
                return HAS_DEFINE_PROPERTY ? target.__stjMeta__ : moduleRegistry[id];
            },

            /**
             * Create a new module from `target`
             *
             * @param {Object|Function} target
             * @param {string} name
             * @param {boolean=} _isPrototype
             * @param {boolean=} _resolve
             * @return {Object|Function}
             */
            moduleCreate = function moduleCreate(target, name, _isPrototype, _resolve) {
                if (!isModule(target)) {
                    var moduleId = moduleCurrentId,
                        meta = new Meta(target, moduleId, name, _isPrototype);
                    if (HAS_DEFINE_PROPERTY) {
                        propertyDefine(target, '__stjMeta__', {
                            value: meta,
                            enumerable: false,
                            writable: false
                        });
                    } else {
                        moduleToId.push(target);
                        moduleRegistry[moduleId] = meta;
                    }

                    moduleCurrentId += 1;
                }
                return target;
            },

            /**
             * Resolve all module queue
             */
            moduleResolve = function moduleResolve(mod) {
                mod.resolve();
            },

            /**
             * Resolve all module queue
             */
            moduleResolveAll = function () {
                for (var i = 0, l = moduleQueue.length; i < l; i += 1) {
                    moduleResolve(moduleQueue[i]);
                }
                moduleQueue = [];
            },

            /**
             * Throw error if module is already registered
             *
             * @param {string} path
             * @return {boolean}
             */
            assertNotRegistered = function assertNotRegistered(path) {
                if (path.length > 0 && !isUndefined(moduleMeta(moduleCurrent).path(path))) {
                    throw new Error(path + MESSAGE_ALREADY_DECLARED);
                }
                return path;
            },

            /**
             * Register resolving `path` from currentModule
             *
             * @param {string} path
             * @param {*}
             * @return {*}
             */
            moduleRegister = function moduleRegister(path, moduleObject) {
                if (path.length > 0) {
                    assertNotRegistered(path);
                    moduleMeta(moduleCurrent).path(path, moduleObject);
                }
                return moduleObject;
            },




            //=========================================================================
            //                                Class
            //=========================================================================

            /**
             * Create a class from `target`
             *
             * @param {Function} target
             * @param {string} name
             * @param {boolean=} _resolve
             * @return {Function}
             */
            classCreate = function classCreate(target, name, _resolve) {
                validateFunction(target);
                name = name || functionName(target);

                //Class
                moduleCreate(target, name, false, false);

                //Prototype
                moduleCreate(target.prototype, name, true, false);


                //resolution
                if (_resolve) {
                    moduleResolveAll();
                }
                return target;
            },

            /**
             * Make `self` inherits from `parentClass`
             *
             * @param {Function} self
             * @param {Function=} parentClass
             * @return {Function}
             */
            classInherits = function classInherits(self, parentClass) {
                if (parentClass) {
                    validateFunction(parentClass);
                    var /** @constructor */
                        TempClass = function () {},
                        prototypeOld = self.prototype,
                        prototypeNew;

                    //Create a fake new constructor
                    TempClass.prototype = parentClass.prototype;
                    prototypeNew = new TempClass();

                    //Restore methods
                    merge(prototypeNew, prototypeOld);
                    propertyHidden(prototypeNew, 'constructor', getOwn(prototypeOld, 'constructor', self));

                    //Change prototype
                    self.prototype = prototypeNew;
                }
                return self;
            },

            /**
             * Convert object or function to a module
             *
             * @param {string} targetNamespace,
             * @param {Object|Function} objectOrFunction
             * @param {Array=} properties array of property names to force import
             */
            toModule = function toModule(targetNamespace, objectOrFunction, properties) {
                targetNamespace = targetNamespace ? namespaceNormalize(targetNamespace) : '';

                var objectIsFunction = isFunction(objectOrFunction),
                    objectMeta,
                    objectMetaProto,
                    objectProto = objectIsFunction ? objectOrFunction.prototype : null;

                if (!isModule(objectOrFunction)) {
                    (objectIsFunction ? classCreate : moduleCreate)(objectOrFunction, targetNamespace, false);
                    objectMeta = moduleMeta(objectOrFunction);
                    objectMetaProto = objectIsFunction ? moduleMeta(objectProto) : null;

                    //force import of properties
                    if (properties) {
                        forEach(properties, function (property) {
                            //check proto
                            if (objectIsFunction) {
                                objectMetaProto.importProperty(property);
                            }
                            //object
                            objectMeta.importProperty(property);
                        });
                    }

                    if (objectIsFunction && stj) {
                        objectMeta.include(stj.use_("stj.Class"));
                        if (objectOrFunction !== Object) {
                            objectMetaProto.include(stj.use_("stj.Object"));
                        }
                    }
                }

                if (targetNamespace.length > 0) {
                    moduleMeta(moduleRoot).path(targetNamespace, objectOrFunction, false);
                }

                moduleResolveAll();
                return objectOrFunction;
            },

            _setName = function _setName(self, name) {
                if (isModule(self)) {
                    moduleMeta(self).displayName = name;
                    if (isClass(self)) {
                        _setName(self.prototype, name);
                    }
                } else {

                    /*if (!hasOwn(self, 'displayName')) {
                        objectDefineHidden(self, {
                            displayName : name
                        });
                    } else {
                        self.displayName = name;
                    }*/
                }
            },

            _getName = function _getName(obj) {
                return getOwn(obj, 'displayName');
            };



            //=========================================================================
            //                                Meta Module
            //=========================================================================
            Meta = createConstructor('Meta', {
                initialize: function initialize(mod, id, name, isPrototype) {
                    this._id = id;
                    this.module = mod;
                    this.displayName = name || '';
                    this.properties = {};
                    this.children = {};
                    this.includes = [];
                    this.isPrototype = !!isPrototype;//TODO detect is prototype
                    this.isClass = isFunction(mod);

                    //Import
                    var properties = propertyNames(mod),
                        i = properties.length - 1;
                    while (i >= 0) {
                        this.importProperty(properties[i]);
                        i -= 1;
                    }
                },

                set: function set(name, descriptor, _resolve) {
                    if (!name.length || this._isPropertyInternal(name)) {
                        throw new ValueError('"' + name + '" is not a valid property name.');
                    }
                    if (!isObject(descriptor) || isFunction(descriptor)) {
                        throw new TypeError(descriptor);
                    }

                    var moduleName = this.displayName,
                        //valueName = _getName(value),
                        currentValue = this.get(name),
                        value = descriptor.value;

                    //is already set?
                    if (!isUndefined(currentValue) && (isModule(currentValue) || isModule(value))) {
                        throw new Error(name + MESSAGE_ALREADY_DECLARED);
                    }

                    if (isModule(value)) {
                        moduleMeta(value).displayName = namespaceJoin([ moduleName, name ]);
                    }
                    merge(descriptor, {
                        configurable: true,
                        enumerable: true
                    }, false);
                    descriptor.enumerable = descriptor.enumerable && !this.isPrototype && !isFunction(this.module);
                    this.properties[name] = descriptor;

                    /*if (!valueName) {
                        if (isModule(value) || isClass(value)) {
                            if (
                                !isModule(value) &&
                                !isFunction(module) &&
                                moduleIsPrototype
                            ) {
                                _setName(value, namespaceJoin(moduleName, name));
                            } else {
                                _setName(value, namespaceJoin([ moduleName, name ]));
                            }
                        } else if (isFunction(value)) {
                            _setName(value, namespaceJoin(moduleName, name));
                        }
                    }*/
                    //notify change
                    this._onChange(_resolve);
                    return value;
                },

                get: function get(name) {
                    return getOwn(this.module, name) || getOwn(this.properties[name], 'value');
                },

                path: function path(pathString, createElement, _resolve) {
                    var parts = namespacePartition(pathString),
                        isLeaf = parts[1] === '',
                        first = parts[0],
                        moduleSub = this.get(first);
                    if (createElement && !moduleSub) {
                        moduleSub = isLeaf ? createElement : moduleCreate({});
                        this.set(first, {
                            value: moduleSub
                        }, false);
                    }

                    if (isLeaf || !moduleSub) {
                        if (_resolve) {
                            moduleResolveAll();
                        }
                        return moduleSub;
                    }

                    return moduleMeta(moduleSub).path(parts[2], createElement, _resolve);
                },

                update: function update(extension, _resolve) {
                    for (var property in extension) {
                        if (hasOwn(extension, property) && !isUndefined(extension[property])) {
                            this.set(property, {
                                value: extension[property]
                            }, false);
                        }
                    }
                    return this._onChange(_resolve);
                },

                isIncluding: function isIncluding(extension) {
                    return indexOf(this.includes, extension) > 0;
                },

                include: function include(extension, _resolve) {
                    var module = this.module,
                        includes = this.includes;

                    if (indexOf(includes, extension) < 0 && extension !== module) {
                        includes.push(extension);
                        moduleMeta(extension)._onInclude(module);//Notify inclusion to children
                        this._onChange(_resolve);
                    }
                    return this;
                },

                resolve: function resolve() {
                    var includes = this.includes,
                        include, includeProperties,
                        i = includes.length - 1, j = 0,
                        resolved = {},
                        property;

                    // Resolve dependencies
                    merge(resolved, this.properties, false);
                    while (i >= 0) {
                        include = includes[i];
                        includeProperties = propertyNames(include);
                        j = includeProperties.length - 1;

                        while (j >= 0) {
                            property = includeProperties[j];
                            if (!hasOwn(resolved, property) &&
                                hasOwn(include, property) &&
                                !this._isPropertyInternal(property)
                            ) {
                                resolved[property] = propertyDescriptor(include, property);
                            }
                            j -= 1;
                        }
                        i -= 1;
                    }

                    // Resolve own dependencies
                    for (property in resolved) {
                        if (hasOwn(resolved, property)) {
                            this.exportProperty(property, resolved[property]);
                        }
                    }
                },

                baseName: function () {
                    return namespaceBase(this.displayName);
                },

                _onChange: function _onChange(resolve) {
                    if (!~indexOf(moduleQueue, this)) {
                        moduleQueue.push(this);
                        var children = this.children,
                            id = children.length - 1;
                        for (id in children) {
                            if (hasOwn(children, id)) {
                                moduleMeta(children[id])._onChange(false);
                            }
                        }
                    }

                    if (resolve) {
                        moduleResolveAll();
                    }
                    return this;
                },

                _onInclude: function _onInclude(target) {
                    var hook = this.get(isFunction(target) ? '__extended__' : '__included__');
                    this.children[moduleId(target)] = target;
                    if (hook) {
                        hook.call(this.module, target);
                    }
                },

                _isPropertyInternal: function _isPropertyInternal(propertyName) {
                    return hasOwn({
                        //'name': true,
                        'caller': true,
                        'arguments': true,
                        /*length: true,*/
                        prototype: true,
                        constructor: true,
                        displayName: true,
                        __stjMeta__: true
                    }, propertyName);
                },

                importProperty: function importProperty(property) {
                    var target = this.module,
                        properties = this.properties;

                    if (!this._isPropertyInternal(property) && !isUndefined(target[property])) {
                        properties[property] = propertyDescriptor(target, property) || {
                            value: target[property],
                            configurable: true
                        };
                    }
                },

                exportProperty: function exportProperty(property, descriptor) {
                    var target = this.module,
                        targetDescriptor = propertyDescriptor(target, property);

                    if (
                        getOwn(targetDescriptor, 'configurable', true) &&
                        (getOwn(targetDescriptor, 'value', getOwn(target, property)) !== descriptor.value/* ||
                         getOwn(targetDescriptor, 'get') !== descriptor.get*/)
                    ) {
                        propertyDefine(target, property, descriptor);
                    }


                }
            });

            //Initialize modules
            moduleCreate(moduleRoot, '', false, false);

            //=========================================================================
            //                                stj
            //=========================================================================
            (function (convertModule, convertModules) {
                var validateModule = function validateModule(obj) {
                        if (!isModule(obj)) {
                            throw new TypeError(obj + MESSAGE_NOT_MODULE);
                        }
                        return obj;
                    },

                    validateClass = function validateClass(obj) {
                        if (!isClass(obj)) {
                            throw new TypeError(obj + MESSAGE_NOT_CLASS);
                        }
                        return obj;
                    },

                    // Add the given methods to the object.
                    genericDefinition = function genericDefinition(withObject, definition, _isStatic) {
                        if (withObject && definition) {
                            //Remember previous state
                            var oldObject = moduleCurrent,
                                oldStatic = moduleCurrentIsStatic,
                                meta = moduleMeta(withObject),
                                exception;

                            moduleCurrent = withObject;
                            moduleCurrentIsStatic = !!_isStatic;


                            if (isModule(definition)) {
                                meta.include(definition, true);
                            } else if (isFunction(definition)) {
                                //if class the try to find the parent class

                                try {
                                    definition.call(
                                        withObject, //this
                                        withObject, //first arg the module/class
                                        meta
                                    );
                                } catch (e) {
                                    exception = e;
                                }
                            } else {
                                meta.update(definition, true);
                            }

                            moduleCurrent = oldObject;
                            moduleCurrentIsStatic = oldStatic;

                            if (exception) {
                                throw exception;
                            }
                        }
                        return withObject;
                    };

                stj = convertModule("stj", {
                    cache: {},

                    /**
                     * Return the module
                     *
                     * @global
                     * @param {String|Object|Function} pathOrModule
                     * @param {boolean=} _noError
                     * @return {Object|Function}
                     */
                    use_: function use_(pathOrModule, _noError) {
                        var result = pathOrModule,
                            cache = stj.cache;
                        if (isString(pathOrModule)) {
                            pathOrModule = namespaceNormalize(pathOrModule);

                            result = cache[pathOrModule];
                            if (!result) {
                                result = moduleMeta(moduleRoot).path(pathOrModule);
                                result = isModule(result) ? result : undefined;
                                if (result) {
                                    cache[pathOrModule] = result;
                                }
                            }

                            if (isUndefined(result) && !_noError) {
                                throw new Error(pathOrModule + MESSAGE_NOT_DEFINED);
                            }
                        } else {
                            validateModule(result);
                        }
                        return result;
                    },

                    /**
                     * Create a new module
                     *
                     * @global
                     * @param {String|Object|Function} pathOrModule
                     * @param {Function} definition
                     * @return {Object|Function}
                     */
                    module_: function module_(pathOrModule, definition) {
                        var result = pathOrModule;

                        if (isString(pathOrModule)) {
                            pathOrModule = assertNotRegistered(namespaceNormalize(pathOrModule));
                            result = moduleRegister(
                                pathOrModule,
                                moduleCreate({}/*, '', false, false*/)
                            );
                        }

                        //module definition
                        return genericDefinition(validateModule(result), definition, false);
                    },

                    /**
                     * Create a new module
                     *
                     * @global
                     * @param {String|Object|Function} pathOrModule
                     * @param {String|Function=} parent
                     * @param {Function=} definition
                     * @return {Function}
                     */
                    class_: function class_(pathOrModule, parent, definition) {
                        var result;
                        if (isString(pathOrModule)) {
                            pathOrModule = assertNotRegistered(namespaceNormalize(pathOrModule));
                            parent = parent ? stj.use_(parent) : null;

                            result = createConstructor(namespaceBase(pathOrModule));
                            result = moduleRegister(pathOrModule, classCreate(classInherits(result, parent)));

                            stj.extend_(result, "stj.Class");
                            if (parent) {
                                stj.extend_(result, parent);
                            }
                            stj.include_(result.prototype, "stj.Object");
                        } else {
                            result = pathOrModule;
                            definition = parent;
                        }

                        //Call factory
                        return genericDefinition(validateClass(result), definition, false);
                    },

                    /**
                     * Getter/Setter an attribute for module if no value is specified
                     *
                     * @param {Module} thisModule
                     * @param {string} name
                     * @param {*} value
                     * @return {*}
                     */
                    property_: function property_(thisModule, name, value, descriptor) {
                        if (isUndefined(value) && isString(thisModule)) {
                            descriptor = value;
                            value = name;
                            name = thisModule;
                            thisModule = isClass(moduleCurrent) && !moduleCurrentIsStatic ? moduleCurrent.prototype : moduleCurrent;
                        }
                        descriptor = descriptor || {};
                        descriptor.value = value;
                        return isUndefined(value) ? thisModule[name] : moduleMeta(thisModule).set(name, descriptor, true);
                    },

                    /**
                     * Define many properties
                     *
                     * @param {Module} thisModule
                     * @param {Object} properties
                     * @return {*}
                     */
                    properties_: function properties_(thisModule, properties) {
                        if (isUndefined(properties)) {
                            properties = thisModule;
                            thisModule = isClass(moduleCurrent) && !moduleCurrentIsStatic ? moduleCurrent.prototype : moduleCurrent;
                        }
                        return moduleMeta(thisModule).update(properties, true);
                    },

                    /**
                     * Return include
                     *
                     * @param {Function|Object|string} thisModule
                     * @param {Object} definition
                     * @return {Function|Object}
                     */
                    include_: function include_(thisModule, definition) {
                        if (isUndefined(definition)) {
                            definition = thisModule;
                            thisModule = moduleCurrent;
                        }

                        if (!moduleCurrentIsStatic && isClass(thisModule)) {
                            thisModule = thisModule.prototype;
                        }

                        var meta = moduleMeta(thisModule), i = 0, l;
                        if (isArray(definition) || isString(definition) || isModule(definition)) {
                            definition = isArray(definition) ? definition : [ definition ];
                            l = definition.length - 1;
                            while (i <= l) {
                                meta.include(stj.use_(definition[i]), false);
                                i += 1;
                            }
                            moduleResolveAll();
                        } else {
                            validateModule(definition);//will throw error
                        }
                        return thisModule;
                        //return genericDefinition(thisModule, definition, false);
                    },

                    /**
                     * Extend the type module (prototype)
                     *
                     * @param {Function|Object|string} thisModule
                     * @param {Function|Object} definition
                     * @return {Function}
                     */
                    extend_: function extend_(thisClass, definition) {
                        if (isUndefined(definition)) {
                            definition = thisClass;
                            thisClass = moduleCurrent;
                        }
                        if (isString(definition)) {
                            definition = stj.use_(definition);
                        }
                        return genericDefinition(validateClass(thisClass), definition, true);
                    },

                    /**
                     * Getter/Setter for globals. Throw an error if global is already defined
                     * unless `_allowOverwrite` is true
                     *
                     * Example as getter:
                     *     globals_();//-> return all globals
                     *     globals_('foo'); //-> return global value named foo
                     *
                     * Example as setter:
                     *     globals_('foo', 'bar');//sets foo global to "bar"
                     *     globals_({
                     *         foo: 'bar',
                     *         baz: 'helloword'//sets foo to "bar" and baz to "helloword
                     *     });
                     *
                     * @param {Object|string=} key
                     * @param {*=} value
                     * @param {boolean=} _allowOverwrite (default false)
                     * @return {*}
                     */
                    globals_: (function /*factory*/() {
                        var setterOverwrite = function (myValue, newValue, property) {
                                return newValue;
                            },
                            setterKeep = function (myValue, newValue, property) {
                                if (!isUndefined(myValue) && myValue !== newValue) {
                                    throw new Error(property + MESSAGE_ALREADY_DECLARED);
                                }
                                return newValue;
                            };
                        return function globals_(key, value/*, _allowOverwrite*/) {
                            //work as getter
                            if (!key) {
                                return GLOBAL;
                            }


                            var property, globalValue,
                                _allowOverwrite = isObject(key) ? value : arguments[2],
                                setter = !_allowOverwrite ? setterKeep : setterOverwrite;

                            if (isObject(key)) {
                                merge(GLOBAL, key, setter);
                            } else {
                                globalValue = GLOBAL[key];
                                if (!isUndefined(value)) {
                                    //Work as setter here
                                    globalValue = setter(GLOBAL[key], value, key);
                                    GLOBAL[key] = globalValue;
                                }
                                return globalValue;
                            }
                        };
                    }()),

                    /**
                     * Meta class for modules
                     *
                     * @constructor
                     */
                    Meta: toModule("", Meta),
                    Primitive: toModule("", {}),
                    Object: toModule("", {}),
                    Class: toModule("", {}),

                    namespaceJoin: namespaceJoin,
                    namespaceNormalize: namespaceNormalize,

                    /**
                     * Return current module used
                     */
                    moduleCurrent: function () {
                        return moduleCurrent;
                    },

                    /**
                     * Access the meta module of a function or object
                     *
                     * @param {Object|Function}
                     * @return {stj.Meta}
                     */
                    meta_: function meta_(obj) {
                        return moduleMeta(obj);
                    },
                    isClass: isClass,
                    isModule: isModule,

                    /**
                     * Convert object or function to a module
                     *
                     * @param {string} targetNamespace,
                     * @param {Object|Function} objectOrFunction
                     * @param {Array=} properties array of property names to force import
                     */
                    toModule: toModule,

                    /**
                     * Improved shortcut to script eval
                     *
                     * @param {string} javascriptCode
                     * @param {string=} uriOrFilename
                     */
                    runScript: runScript
                });



                convertModule('Boolean', Boolean, ['toString']);
                convertModule('Function', Function, [ 'apply', 'call', 'bind', 'toString']);
                convertModule('Number', Number, ['toExponential', 'toFixed', 'toLocaleString', 'toPrecision', 'toString']);
                convertModule('RegExp', RegExp, ['exec', 'test']);
                convertModule('Math', Math);
                convertModule('Date', Date, ['now', 'toString', 'parse', 'UTC']);
                convertModule('String', String, [
                    'charAt', 'charCodeAt', 'concat', 'indexOf', 'lastIndexOf', 'match', 'quote', 'replace', 'search',
                    'slice', 'split', 'substr', 'substring', 'trim', 'toLowerCase', 'toUpperCase', 'toString'
                ]);
                convertModule('Object', Object, [
                    'create', 'defineProperty', 'defineProperties', 'keys',
                    'getPrototypeOf', 'getOwnPropertyDescriptor', 'getOwnPropertyNames',
                    'preventExtensions', 'isExtensible', 'seal', 'isSealed', 'freeze', 'isFrozen'
                ]);


                //errors
                convertModule('error.Error', Error, ['toString']);
                convertModules('error', ['EvalError', 'RangeError', 'ReferenceError', 'SyntaxError', 'TypeError', 'URIError']);
                //Generic collections
                convertModule('Array', Array, [
                    'pop', 'push', 'reverse', 'shift', 'sort', 'splice', 'unshift', 'concat', 'join', 'slice',
                    'indexOf', 'lastIndexOf', 'filter', 'forEach', 'every', 'map', 'some', 'reduce', 'reduceRight', 'toString'
                ]);

                //Resolve
                moduleResolveAll();

                // Define globals
                forEach(stj, function (value, key) {
                    if (key.charAt(key.length - 1) === '_') {
                        stj.globals_(key, value);
                    }
                });
            }(
                toModule, //convertModule
                function (root, natives) { //convertModules
                    var i = natives.length - 1, nativeFunction;
                    while (i >= 0) {
                        nativeFunction = natives[i];
                        toModule(namespaceJoin([root, nativeFunction]), GLOBAL[nativeFunction]);
                        i -= 1;
                    }
                }
            ));
        }());

        //=========================================================================
        //                                has
        //=========================================================================
        module_('has', function (has) {
            properties_({
                "global-console": HAS_CONSOLE,
                "global-window": HAS_WINDOW,//browser
                "global-process": HAS_PROCESS,//nodejs
                "object-defineproperty": HAS_DEFINE_PROPERTY,
                "function-name": HAS_FUNCTION_NAME
            });
        });

        //=========================================================================
        //                                filter
        //=========================================================================
        module_('filter', function () {
            var createValidator = function (type, testValue) {
                var testResult = !testValue,
                    methodName = 'is' + type,
                    message = " is not a valid " + type.toLowerCase();

                return function (obj) {
                    if (Object[methodName](obj) === testResult) {
                        throw new TypeError(obj + message);
                    }
                    return obj;
                };
            };

            properties_({
                validateArray: createValidator('Array', true),
                validateBoolean: createValidator('Boolean', true),
                validateNumber: createValidator('Number', true),
                validatePrimitive: createValidator('Primitive', true),
                validateString: createValidator('String', true),
                validateFunction: createValidator('Function', true),
                validateObject: createValidator('Object', true)
            });
        });

        //=========================================================================
        //                                event
        //=========================================================================
        module_('event', function () {
            properties_({
                setTimeout: setTimeout,
                clearTimeout: clearTimeout,
                setInterval: setInterval,
                clearInterval: clearInterval,

                /**
                 * Optimized version of setTimeout(.., 0).
                 * callback will be executed as soon as possible.
                 *
                 * @param {Function} callback
                 * @param {*=} thisp
                 * @return {Function}
                 */
                setNextTick: (function /*factory*/() {
                    if (HAS_PROCESS) {
                        return function setNextTick(callback/*, thisp */) {
                            var thisp = arguments[1];
                            process.nextTick(thisp ? functionBindThis(callback, thisp) : callback);
                        };
                    } else if (typeof MessageChannel !== TYPE_UNDEFINED) {
                        return function setNextTick(callback/*, thisp */) {
                            validateFunction(callback);
                            var channel = new MessageChannel(),
                                thisp = arguments[1];
                            channel.port1.onmessage = function () {
                                callback.call(thisp);
                            };
                            channel.port2.postMessage(0);
                        };
                    } else {
                        var callbackTimeout = function (callback, thisp) {
                            callback.call(thisp);
                        };
                        return function setNextTick(callback/*, thisp */) {
                            validateFunction(callback);
                            setTimeout(callbackTimeout, 0, callback, arguments[1]);
                        };
                    }
                }())
            });
        });

        //=========================================================================
        //                                Object
        //=========================================================================
        class_(Object, function () {
            var objectToString = functionBindToCall(Object.prototype.toString),
                filter = use_('filter');

            extend_({

                /**
                 * Return true if `obj` is a Boolean
                 *
                 * @param {*} obj
                 * @return {boolean}
                 */
                isBoolean: isBoolean,

                /**
                 * Return true if `obj` is an Array
                 *
                 * @param {*} obj
                 * @return {boolean}
                 */
                isArray: isArray,

                /**
                 * Return true if `obj` is an Number
                 *
                 * @param {*} obj
                 * @return {boolean}
                 */
                isNumber: isNumber,

                /**
                 * Return true if `obj` is an Integer
                 *
                 * @param {*} obj
                 * @return {boolean}
                 */
                isInteger: isInteger,

                /**
                 * Return true if `obj` is a string
                 *
                 * @param {*} obj
                 * @return {boolean}
                 */
                isString: isString,

                /**
                 * Return true if `obj` is a Function
                 *
                 * @param {*} obj
                 * @return {boolean}
                 */
                isFunction: isFunction,

                /**
                 * Return true if `obj` is an Object
                 *
                 * @param {*} obj
                 * @return {boolean}
                 */
                isObject: isObject,

                /**
                 * Return true if `obj` is a scalar
                 *
                 * @param {*} obj
                 * @return {boolean}
                 */
                isPrimitive: isPrimitive,

                /**
                 * Return true if `obj` is null
                 *
                 * @param {*} obj
                 * @return {boolean}
                 */
                isNull: isNull,

                /**
                 * Return true if `obj` is undefined
                 *
                 * @param {*} obj
                 * @return {boolean}
                 */
                isUndefined: isUndefined,

                /**
                 * Return true if `object` is instanceof `klass`
                 *
                 * @param {*} obj
                 * @param {Function} klass
                 * @return {boolean}
                 */
                isInstanceOf: function isInstanceOf(obj, klass) {
                    validateFunction(klass);
                    return klass.__isInstanceOf__ ? klass.__isInstanceOf__(obj) : (obj instanceof klass);
                },

                /**
                 * Return true if `obj` if empty
                 *
                 * @param {*} obj
                 * @return {boolean}
                 */
                isEmpty: function isEmpty(obj) {
                    return isUndefined(obj) || isNull(obj) ?
                            true :
                            (obj.isEmpty ? obj.isEmpty() : !Object.size(obj));
                },

                /**
                 * Return true if `obj` is equal to `other`
                 * Call internal obj.isEqual() if exists
                 *
                 * @param {*} obj
                 * @param {*} other
                 * @param {boolean=} _strict
                 * @return {boolean}
                 */
                isEqual: function isEqual(obj, other/*, _strict*/) {
                    var _strict = arguments[2];
                    return (
                        // simple case
                        (obj === other) ? true :

                        //undefined or null
                        (_strict || isUndefined(obj) || isNull(obj)) ? false :

                        //generic handler
                        obj.isEqual ? obj.isEqual(other, _strict) :

                        //default
                        Object._isEqual(obj, other, _strict)
                    );
                },

                isEqualStrict: function isEqualStrict(obj, other) {
                    return obj === other;
                },

                /**
                 * Return representation of the object (for debug)
                 * Call internal obj.toRepresentation() if exists
                 *
                 * @param {*} obj
                 * @return {string}
                 */
                representation: function representation(obj) {
                    var options = arguments[1] || {};
                    //default options
                    merge(options, {
                        keySeparator: ':',
                        propertySeparator: ', '
                    }, false);
                    return isUndefined(obj) || isNull(obj) ?
                            stringify(obj) :
                            (obj.toRepresentation ? obj.toRepresentation(options) : objectToString(obj));
                },

                /**
                 * Return an enumerator for `obj`
                 *
                 * @param {Object} obj
                 * @param {Object=} options
                 * @param {Iterator}
                 */
                /*enumerator: function enumerator(obj, options) {
                    if (obj) {
                        if (obj.enumerator) {
                            return obj.enumerator(options);
                        }
                        throw new Error();
                    }
                },*/

                /**
                 * Iterate on object.
                 * Call internal object.__forEach__(iterator, thisp, options) if exists.
                 *
                 * @param {*} obj
                 * @param {Function} iterator
                 * @param {Object|null|undefined=} thisp
                 * @param {Object=} options
                 * @return {*} thisp.__return__ if defined, else `obj`
                 */
                forEach: forEach,

                /**
                 * Return a hash string for the object.
                 * Call internal object.__hash__() if exists.
                 *
                 * @param {*} obj
                 * @return {string}
                 */
                hash: function hash(obj) {
                    return isPrimitive(obj) ?
                          //1. Primitive case
                               stringify(obj) :
                          //2. Generic case
                               (obj.__hash__ ? obj.__hash__() : Object._hash(obj));
                },

                /**
                 * Clone an object.
                 * Call internal obj.clone() if exists.
                 *
                 * @param {*} obj
                 * @param {boolean=} deep
                 * @return {*}
                 */
                clone: function clone(obj, deep) {
                    return (!obj || isPrimitive(obj)) ?
                            //1. Primitive case
                                obj :
                            //2. Generic case
                                (obj.clone ? obj.clone(deep) : Object._clone(obj, deep));
                },

                /**
                 * Destroy the object
                 *
                 * @param {*} obj
                 * @return {boolean}
                 */
                destroy: function destroy(obj) {
                    if (obj && obj.finalize) {
                        obj.finalize();
                    }
                },

                /**
                 * Return true if object has property
                 */
                has: function has(object, property) {
                    return hasOwn(filter.validateObject(object), property);
                },

                /**
                 * Return the `key` attribute value or `defaultValue` if not defined
                 *
                 * @param {*} obj
                 * @param {string|number|Array} key
                 * @param {*=} _defaultValue
                 * @return {*}
                 */
                get: function get(obj, key/*, _defaultValue*/) {
                    var value,
                        _defaultValue = arguments[2];
                    if (obj.__get__) {
                        value = obj.__get__(key);
                        return isUndefined(value) ? _defaultValue : value;
                    }
                    return getOwn(obj, key, _defaultValue);
                },

                /**
                 * Set the `key` value
                 *
                 * @param {*} obj
                 * @param {string|number|Array} key
                 * @param {*} value
                 * @return {this}
                 */
                set: function set(obj, key, value) {
                    if (obj.__set__) {
                        return obj.__set__(key, value) || value;
                    } else {
                        obj[key] = value;
                        return obj;
                    }
                },

                /**
                 * Unset the `key` value and returns it
                 *
                 * @param {*} obj
                 * @param {string|number|Array} key
                 * @param {*=} _defaultValue
                 * @return {*}
                 */
                unset: function unset(obj, key/*, _defaultValue*/) {
                    var value,
                        _defaultValue = arguments[2];
                    if (obj.__unset__) {
                        value = obj.__unset__(key);
                    } else {
                        value = Object.get(obj, key);
                        delete obj[key];
                    }
                    return isUndefined(value) ? _defaultValue : value;
                },

                /**
                 * Return the `obj` size
                 *
                 * @param {*} left
                 * @param {*} right
                 * @return {number}
                 */
                compare: function compare(left, right) {
                    return (
                        left === right ? 0 :

                        //null and undefined comparison
                        isUndefined(left) || isNull(left) ? -1 :

                        //generic handler
                        left.__compare__ ? left.__compare__(right) :

                        //simple comparison
                        +left > +right ? 1 : -1
                    );
                },

                /**
                 * Return the `obj` size (number of items)
                 *
                 * @return {number}
                 */
                size: function size(obj) {
                    return !obj ?
                            // empty values
                            0 :
                            // generic way
                            toInteger(obj.__size__ ? obj.__size__() : objectKeys(obj).length);
                },

                /**
                 * Return a formatted `obj` using `formatSpec` options
                 *
                 * @param {*} obj
                 * @param {string=} formatSpec
                 * @return {string}
                 */
                format: function format(obj, formatSpec) {
                    return isUndefined(obj) || isNull(obj) ?
                            // empty values
                            '' :
                            // generic way
                            (obj.__format__ ? obj.__format__(formatSpec) : Object._format(obj, formatSpec));
                },

                /**
                 * Mix `extension` into `object`
                 *
                 * @param {Object} object
                 * @param {Object} extension
                 * @param {Function|boolean=} _conflictHandler (objectValue, extensionValue, property) -> value
                 */
                update: function update(object, extension/*, _conflictHandler*/) {
                    return merge(object, extension, arguments[2], true);
                },

                /**
                 * Create a new enumerable calling iterator on every value.
                 *
                 * @param {Object} object
                 * @param {Function} iterator will be called as iterator(value, key, this)
                 * @param {Object=} thisp
                 * @return {Array}
                 */
                map: (function /*factory*/() {
                    var iteratorMap = function (value, key, self) {
                        var newValue = this[0].call(this[1], value, key, self);
                        if (!isUndefined(newValue)) {
                            returnGet(this).push(newValue);
                        }
                    };

                    return function map(object, iterator/*, thisp*/) {
                        validateFunction(iterator);
                        return forEach(object, iteratorMap, returnSet(slice(arguments, 1), []));
                    };
                }()),

                /**
                 * Reduce enumerable
                 *
                 * @param {Object} object
                 * @param {Function} iterator that will be called with (result, value, key, self)
                 * @param {*} firstValue first value passed to iterator
                 * @return {*}
                 */
                reduce: (function /*factory*/() {
                    var iteratorFirst = function (value) {
                            if (!isUndefined(value)) {
                                returnSet(this, value);
                                stopIteration();
                            }
                        },
                        iteratorReduce = function (value, key, self) {
                            returnSet(this, this[0].call(null, returnGet(this), value, key, self));
                        };
                    return function reduce(object, iterator/*, firstValue*/) {
                        var args = slice(arguments, 1),
                            firstValue = args[1];
                        if (isUndefined(firstValue)) {
                            firstValue = forEach(object, iteratorFirst, {});
                        }
                        if (isUndefined(firstValue)) {
                            throw new Error("empty enumerable and firstValue undefined");
                        }
                        validateFunction(iterator);
                        return forEach(object, iteratorReduce, returnSet(args, firstValue));
                    };
                }()),

                /**
                 * Filter all values using `iterator`
                 *
                 * @param {Object} obj
                 * @param {Function=} iterator
                 * @param {Object=} thisp
                 * @return {Array}
                 */
                filter: (function /*factory*/() {
                    var iteratorFilter = function (value, key, self) {
                        if (this[0].call(this[1], value, key, self)) {
                            returnGet(this).push(value);
                        }
                    };
                    return function filter(obj, iterator/*, thisp*/) {
                        iterator = validateFunction(iterator) || Function.Identity;
                        return forEach(obj, iteratorFilter, returnSet(slice(arguments, 1), []));
                    };
                }()),

                /**
                 * Return an array conversion of the object when possible
                 *
                 * @param {*} obj
                 * @return {Array}
                 */
                toArray: function toArray(obj) {
                    return obj.toArray ? obj.toArray() : slice(obj);
                },

                //Internal methods (only for special use i.e. Dictionary)
                _isEqual: function _isEqual(obj, other, _strict) {
                    if (obj === other) {
                        return true;
                    }
                    if (_strict || isUndefined(other) || isNull(other)) {
                        return false;
                    }

                    //Different prototype property
                    if (obj.constructor !== other.constructor) {
                        return false;
                    }

                    var isEqual = Object.isEqual,//Generic is equal
                        objKeys = objectKeys(obj),
                        otherKeys = objectKeys(other),
                        objKeysLength = objKeys.length,
                        objCurrentKey;

                    // having the same number of owned properties (keys incorporates hasOwnProperty)
                    if (objKeysLength !== otherKeys.length) {
                        return false;
                    }

                    // No properties in object
                    if (!objKeysLength) {
                        return true;
                    }

                    //Check that all keys in one is in the other
                    if (!isEqual(objKeys.sort(), otherKeys.sort())) {
                        return false;
                    }

                    /*objKeys.sort();
                    objOther.sort();
                    objKeysLength = lastIndex(objKeys);
                    while (objKeysLength >= 0) {
                        objCurrentKey = objKeys[objKeysLength];
                        if (objCurrentKey !== objOther[objKeysLength]) {
                            return false;
                        }
                        objKeysLength -= 1;
                    }*/

                    //Check all values
                    objKeysLength = objKeys.length - 1;
                    while (objKeysLength >= 0) {
                        objCurrentKey = objKeys[objKeysLength];

                        if (!isEqual(obj[objCurrentKey], other[objCurrentKey])) {
                            return false;
                        }
                        objKeysLength -= 1;
                    }

                    return true;
                },

                _clone: (function /*factory*/() {
                    var iterator = function (value, key) {
                        /*this[0]: deep, this[1]: clone deep or Identity*/
                        Object.set(returnGet(this), key, this[1](value, this[0]));
                    };
                    return function _clone(obj, deep) {
                        var Constructor = obj.constructor || Object;
                        return forEach(obj, iterator, returnSet({
                            0: deep,
                            1: deep ? Object.clone : Function.Identity
                        }, new Constructor()));
                    };
                }()),

                _hash: (function /*factory*/() {
                    var iterator = function (value, key) {
                        var hashKey = stringify(key),
                            hashValue = Object.hash(value);
                        return (hashKey > hashValue) ? hashValue + hashKey : hashKey + hashValue;
                    };
                    return function _hash(obj) {
                        //Default implementation
                        return Object.map(obj, iterator).sort().join('');
                    };
                }()),

                _format: (function /*factory*/() {
                    var FORMAT_SPEC = new RegExp(
                            "^" +
                            "(" +
                            "([^\\{\\}]?)" + //fill
                            "([<>=\\^])" + //align
                            ")?" +
                            "([\\ +-]?)" + //sign
                            "(\\#?)" + //prefix base
                            "(\\0?)" + //zeropad
                            "([0-9]*)" + //width
                            "(\\,?)" +
                            "(\\.([0-9]*))?" + //precision
                            "([a-zA-Z%]?)" + //Type
                            "$"
                        ),
                        BASE_ABBREV = {
                            b: 2,
                            o: 8,
                            u: 10,
                            x: 16,
                            X: 16
                        },
                        BASE_PREFIXES = {
                            '2': '0b',
                            '8': '0o',
                            '16': '0x'
                        },

                        ENUM_PRECISION_FLOATS = choices('fFeE'),
                        ENUM_FORMAT_INT_CHAR = choices('c'),
                        ENUM_FORMAT_INT_BINARY = choices('bouxX'),
                        ENUM_FORMAT_INT_OTHER = choices('di'),
                        ENUM_FORMAT_INT = merge(merge(merge({}, ENUM_FORMAT_INT_CHAR), ENUM_FORMAT_INT_BINARY), ENUM_FORMAT_INT_OTHER),
                        ENUM_FORMAT_FLOAT = choices('eEfFgG%'),
                        ENUM_FORMAT_UPPER = choices('EFG'),
                        ENUM_SIGN = choices('+- '),

                        ALIGN_TO_DIRECTION = {
                            '<': 'r',
                            '>': 'l',
                            '^': 'b'
                        },
                        TYPE_TO_METHOD = {
                            'e': 'toExponential',
                            'g': 'toPrecision',
                            'f': 'toFixed'
                        };

                    return function _format(obj, formatSpec) {

                        //Format spec
                        if (formatSpec && formatSpec.length > 0) {
                            var value = obj,
                                valueIsNumber = isNumber(value),
                                formatSpecParts = formatSpec.match(FORMAT_SPEC),
                                fill = formatSpecParts[2] || ' ',
                                align = formatSpecParts[3] || '>',
                                sign = formatSpecParts[4] || '-',
                                prefixBaseX = formatSpecParts[5],
                                zeroPad = !!formatSpecParts[6],
                                width = formatSpecParts[7] ? +formatSpecParts[7] : 0,
                                thousandSep = formatSpecParts[8] ? ',' : '.',//TODO : implement this
                                type = formatSpecParts[11] || null,
                                precision = !formatSpecParts[10] ? (ENUM_PRECISION_FLOATS[type] ? 6 : (type === 'd') ? 0 : null) : +formatSpecParts[10],
                                signPrefix,

                                isPercentage,
                                padDirection;

                            // Note: undocumented perl feature:
                            if (width < 0) {
                                width = -width;
                                align = '<';//leftJustify= true
                            }

                            // zeroPad option
                            if (zeroPad) {
                                align = '=';
                                fill = '0';
                            }

                            // Align

                            if (valueIsNumber) {
                                value = +value;

                                type = type || (isInteger(value) ? 'd' : 'g');

                                //Integer representation available
                                if (ENUM_FORMAT_INT[type]) {
                                    if (!isInteger(value)) {
                                        throw new ValueError('"' + value + '" must be an integer for ' + type);
                                    }

                                    if (ENUM_FORMAT_INT_CHAR[type]) {
                                        value = String.fromCharCode(+value);

                                    } else if (ENUM_FORMAT_INT_BINARY[type]) {

                                        // Note: casts negative numbers to positive ones
                                        value = value >>> 0;
                                        value = (prefixBaseX && value && BASE_PREFIXES[BASE_ABBREV[type]] || '') +
                                                value.toString(BASE_ABBREV[type]).padLeft(precision || 0, '0');

                                        if (type === 'X') {
                                            value = value.toUpperCase();
                                        }

                                    } else if (ENUM_FORMAT_INT_OTHER[type]) {
                                        value = value | 0;
                                        signPrefix = value < 0 ? '-' : (sign === '-' ? '' : sign);
                                        value = signPrefix +  stringify(abs(value)).padLeft(precision, '0');
                                    }
                                //Float representation available
                                } else if (ENUM_FORMAT_FLOAT[type]) {
                                    isPercentage = (type === '%');
                                    signPrefix = value < 0 ? '-' : (sign === '-' ? '' : sign);
                                    value = abs(isPercentage ? (100 * value) : value);
                                    value = value[TYPE_TO_METHOD[type.toLowerCase()] || TYPE_TO_METHOD.f](precision);
                                    value = signPrefix + value + (isPercentage ? '%' : '');

                                    //Text transform
                                    if (ENUM_FORMAT_UPPER[type]) {
                                        value = value.toUpperCase();
                                    }
                                } else {
                                    throw new ValueError('"' + type + '" representation not supported');
                                }
                            } else {
                                //String representation available
                                value = stringify(value);
                            }

                            //Alignment
                            if (width > value.length) {
                                padDirection = ALIGN_TO_DIRECTION[align];
                                if (padDirection) {
                                    return value.pad(width, fill, padDirection);
                                } else if (align === '=' && valueIsNumber) {
                                    return ENUM_SIGN[value.charAt(0)] ?
                                            //pad sign
                                            value.slice(0, 1) +
                                            ''.pad(width - value.length, fill, 'l') +
                                            value.slice(1):

                                            //default padding
                                            value.pad(width, fill, 'l');
                                } else {
                                    throw new ValueError('"' + align + '" align not supported for "' + value + '".');
                                }
                            }
                            return value;
                        }
                        return stringify(obj);
                    };
                }())
            });
        });

        /**
         * @module stj
         */
        module_(use_("stj"), function (stj) {
            var size = Object.size,
                objectReduce = Object.reduce,
                objectHash = Object._hash,
                objectGet = Object.get,
                objectSet = Object.set,
                objectUnset = Object.unset,
                objectClone = Object._clone,
                objectCompare = Object.compare,
                objectForEach = forEach,
                objectFilter = Object.filter,
                objectMap = Object.map;

            /**
             * @module stj.Class
             */
            module_(stj.Class, {
                __extended__: function __extended__(base) {
                    //TODO use property instead
                    property_(base, 'create', function create(/*arg1, arg2*/) {
                        return base.apply(base, arguments);
                    });
                }

                /**
                 * Return a new instance of this class
                 */

            });

            /**
             * @module stj.Object
             */
            module_(stj.Object, {
                //default comparison
                __compare__: function __compare__(other) {
                    var valueSelf = +this,
                        valueOther = +other;
                    return (valueSelf === valueOther) ?
                            0:
                            (valueSelf < valueOther ? 1 : -1);
                },

                //default iteration
                __forEach__: function __forEach__(iterator, thisp) {
                    for (var key in this) {
                        if (hasOwn(this, key)) {//TODO remove this for performances?
                            iterator.call(thisp, objectGet(this, key), key, this);
                        }
                    }
                },

                //default size implementation
                __size__: (function /*factory*/() {
                    var iteratorSize = function (currentValue) {
                        return currentValue + 1;
                    };
                    return function __size__() {
                        return objectReduce(this, iteratorSize, 0);
                    };
                }()),

                //default hash
                __hash__: function __hash__() {
                    return objectHash(this);
                },

                /**
                 * Return true if
                 *
                 * @param {*} other
                 * @param {boolean=} _strict
                 * @return {boolean}
                 */
                isEqual: function isEqual(other, _strict) {
                    return Object._isEqual(this, other, _strict);
                },

                /**
                 * Return the `key` attribute value
                 *
                 * @param {string|number|Array} key
                 * @param {*=} _defaultValue
                 * @return {*}
                 */
                get: function get(key/*, _defaultValue*/) {
                    return objectGet(this, key, arguments[1]);
                },

                /**
                 * Set the `key` value
                 *
                 * @param {string|number|Array} key
                 * @param {*} value
                 * @return {this}
                 */
                set: function set(key, value) {
                    return objectSet(this, key, value);
                },

                /**
                 * Unset the `key` value and returns it
                 *
                 * @param {string|number|Array} key
                 * @param {*=} _defaultValue
                 * @return {*}
                 */
                unset: function unset(key/*, _defaultValue*/) {
                    return objectUnset(this, key, arguments[1]);
                },

                /**
                 * Return a copy of the scalar
                 *
                 * @return {*}
                 */
                clone: function clone(deep) {
                    return objectClone(this, deep);
                },

                /**
                 * Return object representation
                 *
                 * @return {string}
                 */
                toRepresentation: function toRepresentation(options) {
                    var repr = Object.representation;

                    return this.constructor.getName()  + '({' + objectMap(this, function (value, key) {
                        return repr(key, options) + options.keySeparator + repr(value, options);
                    }).join(options.propertySeparator) + '})';
                }
            });

            /**
             * @module stj.Primitive
             */
            module_(stj.Primitive, {
                /**
                 * Return a number representing comparison
                 *
                 * @param {*} other
                 * @return {number}
                 */
                __compare__: function __compare__(other) {
                    return (this === other) ?
                            0:
                            (this < other ? 1 : -1);
                },

                /**
                 * Return the scalar hash (i.e. string conversion)
                 *
                 * @return {string}
                 */
                __hash__: function __hash__() {
                    return stringify(this);
                },

                /**
                 * Return a copy of the scalar
                 *
                 * @return {*}
                 */
                clone: function clone() {
                    return this;
                },

                /**
                 * Return true if this is equal to other
                 *
                 * @param {*} other
                 * @return {boolean}
                 */
                isEqual: function isEqual(other) {
                    return other === this;
                },

                /**
                 * Return primitive representation
                 *
                 * @return {string}
                 */
                toRepresentation: function toRepresentation(options) {
                    return stringify(this);
                }
            });

            /**
             * @module stj.Enumerable
             */
            module_("Enumerable", function () {
                var ITERATOR_EVERY_SOME = function (value, key, self) {
                        var //iterator, thisp
                            iter = !this[0].call(this[1], value, key, self),
                            result = returnGet(this);
                        if (iter === result) {
                            returnSet(this, !result);
                            stopIteration();
                        }
                    },

                    ITERATOR_MIN_MAX = function (value, key, self) {
                        var
                            iterator = this.it,
                            current = returnGet(this);
                        value = iterator ? iterator.call(this.t, value, key, self) : value;
                        returnSet(this, isUndefined(current) || (this.x * objectCompare(value, current) >= 0) ? value : current);
                    };

                /*class_('Enumerator', function () {
                    properties_({
                        initialize: function initialize(obj, options) {
                            this.object = obj;
                            this.reset();

                            options = options || {};
                            this.reverse = options.reverse || false;
                            this.keysOnly = options.keysOnly || false;
                        },
                        reset: function reset() {
                            this.index = 0;
                            this.keys = Object.objectKeys(this.object);
                            if (this.reverse) {
                                this.keys.reverse();
                            }
                        },
                        hasNext: function hasNext() {
                            return this.index < this.keys.length;
                        },
                        next: function next() {
                            var self = this,
                                keys = self.keys,
                                index = self.index,
                                key;
                            if(index < keys.length) {
                                key = keys[index];
                                self.index += 1;
                                return self.keysOnly ? key : [key, self.object[key]];
                            } else {
                                stopIteration();
                            }
                        }

                        __forEach__: function __forEach__(block) {
                            this.reset();
                            while(value = this.next()) {
                                block.call(thisp, value[0], value[1]);
                            }
                        }
                    });
                });*/

                //include_("stj.Object");
                properties_({

                    /**
                     * Throws a StopIteration error (act as break)
                     *
                     * @return undefined
                     */
                    stopIteration: stopIteration,

                    /*enumerator: function enumerator(options) {
                        var Class = use_('stj.Enumerable.Enumerator');
                        return new Class(this, options);
                    },*/

                    /**
                     * Return true if element is inside this
                     *
                     * @param {*} value
                     * @return {boolean}
                     */
                    contains: function contains(value) {
                        return this.some(Object.isEqualStrict);
                    },

                    /**
                     * Return true if array is empty
                     *
                     * @return {boolean}
                     */
                    isEmpty: function isEmpty() {
                        return this.every(Function.False);
                    },

                    /**
                     * Return the value count
                     *
                     * @return {number}
                     */
                    size: function size() {
                        return Object.size(this);
                    },

                    /**
                     * Remove all values
                     *
                     * @return this
                     */
                    clear: (function /*factory*/() {
                        var iterator = function (_, key, self) {
                            objectUnset(self, key);
                        };
                        return function clear() {
                            return objectForEach(this, iterator);
                        };
                    }()),

                    /**
                     * Return true if any iterator result are true
                     *
                     * @param {Function=} iterator will be called as iterator(value, key, this)
                     * @param {Object=} thisp
                     * @return {boolean}
                     */
                    count: (function /*factory*/() {
                        var iteratorCount = function (value, key, self) {
                                //iterator, thisp
                                if (this[0].call(this[1], value, key, self)) {
                                    returnSet(this, returnGet(this) + 1);
                                }
                            };
                        return function count(iterator/*, thisp*/) {
                            validateFunction(iterator);
                            return iterator ? objectForEach(this, iteratorCount, returnSet(arguments, 0)) : size(this);
                        };
                    }()),

                    /**
                     * Traverse all values calling iterator(value, key)
                     *
                     * @param {Function} iterator will be called as iterator(value, key, this)
                     * @param {Object=} thisp
                     * @param {Object=} options
                     * @return this
                     */
                    forEach: function forEach(iterator/*, thisp, options*/) {
                        var args = arguments;
                        return objectForEach(this, iterator, args[1], args[2]);
                    },

                    /**
                     * Return Array containing all values
                     *
                     * @return {Array}
                     */
                    keys: (function /*factory*/() {
                        var iterator = function (_, key) {
                                return key;
                            };
                        return function keys() {
                            return objectMap(this, iterator);
                        };
                    }()),

                    /**
                     * Return Array containing all values
                     *
                     * @return {Array}
                     */
                    values: function values() {
                        return objectMap(this, Function.Identity);
                    },

                    /**
                     * Return maximum value
                     *
                     * @return {*}
                     */
                    max: function max(iterator/*, thisp*/) {
                        return objectForEach(ITERATOR_MIN_MAX, {
                            it: iterator,
                            t: arguments[1],
                            x: 1//factor
                        });
                    },

                    /**
                     * Return minimum value
                     *
                     * @return {*}
                     */
                    min: function min(iterator/*, thisp*/) {
                        return objectForEach(ITERATOR_MIN_MAX, {
                            it: iterator,
                            t: arguments[1],
                            x: -1//factor
                        });
                    },

                    /**
                     * Return true if all iterator result are true
                     *
                     * @param {Function} iterator will be called as iterator(value, key, this)
                     * @param {Object=} thisp
                     * @return {boolean}
                     */
                    every: function every(iterator/*, thisp*/) {
                        validateFunction(iterator);
                        return objectForEach(this, ITERATOR_EVERY_SOME, returnSet(arguments, true));
                    },

                    /**
                     * Return true if any iterator result are true
                     *
                     * @param {Function} iterator will be called as iterator(value, key, dictionary)
                     * @param {Object=} thisp
                     * @return {boolean}
                     */
                    some: function some(iterator/*, thisp*/) {
                        validateFunction(iterator);
                        return objectForEach(this, ITERATOR_EVERY_SOME, returnSet(arguments, false));
                    },

                    /**
                     * Create a new enumerable calling iterator on every value. Iterator must retun a value array.
                     *
                     * @param {Function} iterator will be called as iterator(value, key, this)
                     * @param {Object=} thisp
                     * @return {Array}
                     */
                    map: function map(iterator/*, thisp*/) {
                        return objectMap(this, iterator, arguments[1]);
                    },

                    /**
                     * Reduce enumerable
                     *
                     * @param {Function} iterator that will be called with (result, value, key, self)
                     * @param {*} firstValue first value passed to iterator
                     * @return {*}
                     */
                    reduce: function reduce(iterator/*, firstValue*/) {
                        return objectReduce(this, iterator, arguments[1]);
                    },

                    /**
                     * Filter all values
                     *
                     * @param {Function=} iterator will be called as iterator(value, key, this)
                     * @param {Object|null|undefined} thisp
                     * @return {stj.Enumerable}
                     */
                    filter: function filter(iterator/*, thisp*/) {
                        return objectFilter(this, iterator, arguments[1]);
                    },

                    /**
                     * String conversion of an Enumerable
                     *
                     * @return {string}
                     */
                    toString: (function /*factory*/() {
                        var iterator = function (value, key) {
                                var result = returnGet(this);
                                returnSet(this, result + (size(result) ? ',' : '') + key + ':' + stringify(value));
                            };
                        return function toString() {
                            return objectForEach(this, iterator, returnSet({}, ''));
                        };
                    }()),

                    /**
                     * Return the first key found for `value`
                     *
                     * @param {*} element
                     * @return {string|number|undefined}
                     */
                    keyOf: (function /*factory*/() {
                        var iterator = function (value, key) {
                            if (value === this[0]) {
                                returnSet(this, key);
                                stopIteration();
                            }
                        };
                        return function keyOf(element) {
                            return objectForEach(this, iterator, returnSet({
                                0: element
                            }));
                        };
                    }()),

                    /**
                     * Pick a random value
                     *
                     * @return {*}
                     */
                    randomAt: function randomAt() {
                        var keys = this.keys(),
                            key = keys[floor(random() * size(keys))];
                        return objectGet(this, key);
                    }

                });
            });

            /**
             * @module stj.Sequence
             */
            module_("Sequence", function () {
                var ITERATOR_SEARCH = function (value, key) {
                        if (value === this[0]) {
                            returnSet(this, key);
                            stopIteration();
                        }
                    };

                include_("stj.Enumerable");
                properties_({
                    __compare__: (function /*factory*/() {
                        var iterator = function (value, index) {
                                var comparison = objectCompare(value, objectGet(this[0], index));
                                if (comparison !== 0) {
                                    returnSet(this, comparison);
                                    stopIteration();
                                }
                            };

                        return function __compare__(other) {
                            return objectForEach(this, iterator, returnSet({
                                0: other
                            }, 0));
                        };
                    }()),

                    //default forEach
                    __forEach__: function __forEach__(iterator, thisp, options) {
                        options = options || {};
                        var length = size(this),
                            reverse, fromIndex = options.fromIndex,
                            first = 0,
                            last = length - 1,
                            i = 0,
                            end;

                        if (length > 0) {

                            if (!options.reverse) {
                                end = last;
                                i = isUndefined(fromIndex) ? first : max(fromIndex, first);
                                while (i <= end) {
                                    iterator.call(thisp, objectGet(this, i), i, this);
                                    i += 1;
                                }
                            } else {
                                end = first;
                                i = isUndefined(fromIndex) ? last : min(fromIndex, last);

                                while (i >= end) {
                                    iterator.call(thisp, objectGet(this, i), i, this);
                                    i += -1;
                                }
                            }
                        }
                    },

                    //default size
                    __size__: function __size__() {
                        return +this.length;
                    },

                    /**
                     * Return a normalized index
                     *
                     * @param {number} i
                     * @return {number}
                     */
                    index: function index(i) {
                        if (i) {
                            var length = size(this);
                            i = toInteger(i);
                            if (i < 0) {
                                i += length;
                            }
                        }
                        return i;
                    },

                    /**
                     * Return the index for `element` or -1 if not found
                     *
                     * @param {*} element
                     * @param {number=} fromIndex
                     * @return {boolean}
                     */
                    indexOf: function indexOf(element/*, fromIndex*/) {
                        return forEach(this, ITERATOR_SEARCH, returnSet({
                            0: element
                        }, -1), {
                            fromIndex: arguments[1],
                            reverse: false
                        });
                    },

                    /**
                     * Return the index for `element` or -1 if not found (reversed)
                     *
                     * @param {*} element
                     * @param {number=} fromIndex
                     * @return {boolean}
                     */
                    lastIndexOf: function lastIndexOf(element/*, fromIndex*/) {
                        return forEach(this, ITERATOR_SEARCH, returnSet({
                            0: element
                        }, -1), {
                            fromIndex: arguments[1],
                            reverse: true
                        });
                    },

                    /**
                     * Return true if element is inside this
                     *
                     * @param {string} substring
                     * @return {boolean}
                     */
                    contains: function contains(element) {
                        return indexOf(this, element) !== -1;
                    },

                    /**
                     * Return true if array is empty
                     *
                     * @return {boolean}
                     */
                    isEmpty: function isEmpty() {
                        return !this.length;
                    },

                    /**
                     * Return true if this is equal to `other`
                     *
                     * @param {*} other
                     * @return {boolean}
                     */
                    isEqual: function isEqual(other) {
                        if (this === other) {
                            return true;
                        }
                        //TODO detect sequence
                        if (isUndefined(other) || isNull(other) ||
                            (this.constructor !== other.constructor) ||
                            (size(this) !== size(other))
                        ) {
                            return false;
                        }
                        return !this.__compare__(other);
                    },

                    /**
                     * Pick a random value
                     *
                     * @return {*}
                     */
                    randomAt: function randomAt() {
                        return objectGet(this, floor(random() * size(this)));
                    },

                    _normalizeSlice: function (index) {
                        var start, end;

                        if (isArray(index)) {
                            start = index[0];
                            end = index[1];

                            if (isUndefined(start) || isNull(start)) {
                                start = 0;
                            }
                            if (isUndefined(end) || isNull(end)) {
                                end = size(this);
                            }

                            index[0] = this.index(start);
                            index[1] = this.index(end) + 1;
                        } else {
                            index = this.index(index);
                        }
                        return index;
                    }


                });
            });
        });

        //===============================================================================
        //                                  Boolean
        //===============================================================================
        class_(Boolean, function () {
            include_("stj.Primitive");
            extend_({
                __isInstanceOf__: isBoolean
            });
        });


        //=================================================================================
        //                                  Function
        //=================================================================================
        class_(Function, function () {
            properties_({
                __hash__: function __hash__() {
                    return stringify(this);
                },

                __size__: function __size__() {
                    return +this.length;
                },

                /**
                 * Return true if other is equal to this
                 *
                 * @param {*} other
                 * @return {boolean}
                 */
                isEqual: function isEqual(other, _strict) {
                    return this === other;
                },

                /**
                 * Return a copy of this function
                 *
                 * @return {Function}
                 */
                clone: function clone(deep) {
                    return this;
                },

                /**
                 * Return true if this function is native
                 *
                 * @return {boolean}
                 */
                isNative: function isNative() {
                    return indexOf(stringify(this), "[native", 9) >= 0;
                },

                /**
                 * Return function size (number of arguments).
                 * This method is also known as "currying".
                 *
                 * @param {Object|Array} args list to pass to function
                 * @return {Function}
                 */
                generate: function generate(args) {
                    var self = this;
                    if (!args) {
                        return self;
                    }
                    args = !isArray(args) ? slice(args, 0) : args;
                    return function () {
                        var a = args.concat(slice(arguments));
                        return self.apply(this, a);
                    };
                },

                /**
                 * Return the function name
                 *
                 * @return {string}
                 */
                getName: function getName() {
                    return functionName(this);
                },

                /**
                 * Return the function parent (inheritance)
                 *
                 * @return {Function=}
                 */
                getParent: function getParent() {
                    return getOwn(prototypeOf(this.prototype), 'constructor', Object);
                },

                /**
                 * Return new function that will bind first parameter to this
                 *
                 * @return {Function}
                 */
                bindToCall: function bindToCall() {
                    return functionBindToCall(this);
                },

                /**
                 * Apply super `methodName` with apply(`thisp`, `args`)
                 *
                 * @param {string} methodName
                 * @param {Object=} thisp
                 * @param {Object=} args
                 * @return {Function|undefined}
                 */
                applySuper: function applySuper(methodName, thisp, args) {
                    var parent = prototypeOf(this.prototype),
                        method = parent && parent[methodName];
                    if (method) {
                        return !args || !args.length ? method.call(thisp) : method.apply(thisp, args);
                    }
                },

                /**
                 * Call super `methodName` with apply(`thisp`, `args`)
                 *
                 * @param {string} methodName
                 * @param {Object=} thisp
                 * @param {*...} arg1
                 * @return {Function|undefined}
                 */
                callSuper: function callSuper(methodName, thisp/*, arg1, arg2*/) {
                    return this.applySuper(methodName, thisp, slice(arguments, 2));
                },

                /**
                 * Return function representation
                 *
                 * @return {string}
                 */
                toRepresentation: function toRepresentation(options) {
                    var result = stringify(this),
                        index = indexOf(result, "{");
                    if (~index) {
                        result = result.substr(0, index) + "{...}";
                    }
                    return result;
                }
            });

            //Some remarkable functions
            extend_({
                __isInstanceOf__: isFunction,

                /**
                 * Return undefined
                 *
                 * @return undefined
                 */
                Void: function Void() {},

                /**
                 * Return same unchanged object
                 *
                 * @return {*}
                 */
                Identity: function Identity(e) {
                    return e;
                },

                /**
                 * Return object negation
                 *
                 * @return {boolean}
                 */
                Not: function Not(e) {
                    return !e;
                },

                /**
                 * Return always true
                 *
                 * @return {boolean}
                 */
                True: function True() {
                    return true;
                },

                /**
                 * Return always false
                 *
                 * @return {boolean}
                 */
                False: function False() {
                    return false;
                }
            });
        });

        //===============================================================================
        //                                  Number
        //===============================================================================
        class_(Number, function () {
            extend_({
                __isInstanceOf__: isNumber
            });
            include_("stj.Primitive");
            properties_({
                /**
                 * Return true for odd numbers
                 *
                 * @return {boolean}
                 */
                isOdd: function isOdd() {
                    return (this % 2) !== 0;
                },

                /**
                 * Return true for even numbers
                 *
                 * @return {boolean}
                 */
                isEven: function isEven() {
                    return (this % 2) === 0;
                }
            });
        });

        //===============================================================================
        //                                  String
        //===============================================================================
        class_(String, function () {


            var size = Object.size,
                isEmpty = Object.isEmpty,
                StringFormatter = class_('Formatter', null, function () {
                    var objectFormat = Object.format,
                        TAG_START = '{',
                        TAG_STOP = '}',
                        REGEXP_ESCAPE = '\\',
                        REGEXP_TAG_TOKENS = new RegExp(REGEXP_ESCAPE + TAG_START + '([^}]*)' +  REGEXP_ESCAPE + TAG_STOP, 'gim'),
                        /*jslint regexp: false*/
                        REGEXP_ARGS = /[.\[]/;
                        /*jslint regexp: true*/

                    properties_({
                        /**
                         * @constructor
                         */
                        initialize: function initialize() {
                            // bind match and cache it
                            this.__match__ = functionBindThis(this._match, this);
                        },

                        /**
                         * Return the formatted `str` result using `args`
                         *
                         * @param {string} str
                         * @param {Object} args
                         * @return {string}
                         */
                        format: function format(str, args) {
                            this.args = args;
                            this.matchCount = 0;
                            this.usedArgs = {};

                            var result = str.replace(REGEXP_TAG_TOKENS, this.__match__);
                            this.checkUnusedArgs(objectKeys(this.usedArgs), args);
                            return result;
                        },

                        /**
                         * Return value from path
                         *
                         * @param {string} path
                         * @param {Object} args
                         * @return {*}
                         */
                        getValue: (function /*factory*/() {
                            var
                                /*jslint regexp: false*/
                                REGEXP_PATH_TOKENS = /\[([^\[]+)\]|([^\.\[]+)/g,
                                /*jslint regexp: true*/
                                breaked = {},
                                root,
                                replacer = function (_, braceToken, dotToken) {

                                    var part = braceToken || dotToken;
                                    part = parseInt(part, 10) || part;
                                    root = isString(root) ? root.charAt(part) : root[part];

                                    if (isUndefined(root)) {
                                        throw breaked;
                                    }
                                };

                            return function getValue(path, args) {
                                root = args;
                                try {
                                    stringify(path).replace(REGEXP_PATH_TOKENS, replacer);
                                } catch (e) {
                                    if (e !== breaked) {
                                        throw e;
                                    }
                                }
                                return root;
                            };
                        }()),

                        /**
                         * Check usedArgs after
                         *
                         * @param {Array} usedArgs
                         * @param {Object} args
                         */
                        checkUnusedArgs: function checkUnusedArgs(usedArgs, args) {
                            //nothing
                        },

                        /**
                         * Format `value` using `formatSpec`
                         *
                         * @param {*} value
                         * @param {string} formatSpec
                         * @return {string}
                         */
                        formatField: function formatField(value, formatSpec) {
                            return objectFormat(value, formatSpec);
                        },

                        /**
                         * Convert `value`
                         *
                         * @param {*} value
                         * @param {string} conversion (s|r|a)
                         * @return {*}
                         */
                        convertField: (function /*factory*/() {
                            var CONVERSION_TO_METHOD = {
                                's': stringify,
                                'r': Object.representation
                            };
                            return function convertField(value, conversion) {
                                var method = CONVERSION_TO_METHOD[conversion];
                                if (method) {
                                    return method(value);
                                }
                                throw new ValueError('"' + conversion + '" is not a valid conversion');
                            };
                        }()),

                        _match: function _match(_, match) {

                            var formatted,
                                args = this.args,
                                parts = match.partition(":"),
                                formatSpec = parts[2],
                                fieldSpec, argPath, value, firstPart;

                            //split on "!"
                            parts = parts[0].partition("!", "r");

                            argPath = stringify(parts[0] || this.matchCount);
                            fieldSpec = parts[2];

                            //get value
                            value = this.getValue(argPath, args);

                            if (!isUndefined(value)) {
                                //Mark first part as used
                                firstPart = argPath.search(REGEXP_ARGS);

                                this.usedArgs[~firstPart ? argPath.slice(0, firstPart) : argPath] = true;
                                this.matchCount += 1;

                                //format field
                                formatted = this.formatField(fieldSpec ? this.convertField(value, fieldSpec) : value, formatSpec);
                            } else {
                                formatted = TAG_START + match + TAG_STOP;
                            }
                            return formatted;
                        }
                    });
                }),

                CHARS_ALPHA = (function /*factory*/() {
                    var chars = {},
                        fromCode = String.fromCharCode, i;
                    for (i = 'A'.charCodeAt(0); i <= 'Z'.charCodeAt(0); i += 1) {
                        chars[fromCode(i)] = true;
                    }
                    for (i = 'a'.charCodeAt(0); i <= 'z'.charCodeAt(0); i += 1) {
                        chars[fromCode(i)] = true;
                    }
                    return chars;
                }()),

                CHARS_DIGIT = (function /*factory*/() {
                    var chars = {}, i = 0;
                    while (i <= 9) {
                        chars['' + i] = true;
                        i += 1;
                    }
                    return chars;
                }());


            extend_({
                __isInstanceOf__: isString,
                formatter: new StringFormatter()
            });


            include_([ "stj.Sequence", "stj.Primitive" ]);
            properties_({
                __get__: function __get__(index) {
                    index = this._normalizeSlice(index);
                    return isArray(index) ? this.slice(index[0], index[1]) : this.charAt(index);
                },

                /**
                 * Return string representation
                 *
                 * @return {string}
                 */
                toRepresentation: function toRepresentation(options) {
                    return '"' + this + '"';
                },

                /**
                 * Join array into a string using this as glue
                 *
                 * @param {Array} array
                 * @return {string}
                 */
                join: function join(array) {
                    return array.join(this);
                },

                /**
                 * Return a repeated string
                 *
                 * @param {number} count
                 * @return {string}
                 */
                repeat: function repeat(count) {
                    var output = '',
                        str = stringify(this);
                    if (count) {
                        while (count > 0) {
                            if (count & 1) {
                                output += str;
                            }
                            count >>= 1;
                            str += str;
                        }
                    }
                    return output;
                },

                /**
                 * Strip `chars` at left
                 *
                 * @param {string=} chars
                 * @return {string}
                 */
                stripLeft: function stripLeft(/*chars*/) {
                    return this.strip(arguments[0], 'l');
                },

                /**
                 * Strip `chars` at right
                 *
                 * @param {string=} chars
                 * @return {string}
                 */
                stripRight: function stripRight(/*chars*/) {
                    return this.strip(arguments[0], 'r');
                },

                /**
                 * Strip characters `chars` at left and right
                 *
                 * @param {string=} chars
                 * @param {string=} mode (left|right|both)
                 * @return {string}
                 */
                strip: (function /*factory*/() {
                    var REGEXP_STRIP_WHITESPACE_LEFT = /^[\s]+/g,
                        REGEXP_STRIP_WHITESPACE_RIGHT = /[\s]+$"/g,
                        ENUM_LEFT_BOTH = choices('lb'),
                        ENUM_RIGHT_BOTH = choices('rb');

                    return function strip(/*chars, mode*/) {
                        var str = stringify(this),
                            args = arguments,
                            chars = args[0],
                            mode = args[1],
                            m = (mode || 'b').charAt(0).toLowerCase();

                        if (chars) {
                            //left
                            if (ENUM_LEFT_BOTH[m]) {
                                str = str.replace(!chars ? REGEXP_STRIP_WHITESPACE_LEFT : new RegExp("^[" + chars + "]+", "g"), "");
                            }

                            //right
                            if (ENUM_RIGHT_BOTH[m]) {
                                str = str.replace(!chars ? REGEXP_STRIP_WHITESPACE_RIGHT : new RegExp("[" + chars + "]+$", "g"), "");
                            }
                        }
                        return str;
                    };
                }()),

                /**
                 * Return left padded string
                 *
                 * @param {number} length
                 * @param {string=} chars
                 * @return {string}
                 */
                padLeft: function padLeft(length, chars) {
                    return this.pad(length, chars, 'l');
                },

                /**
                 * Return right padded string
                 *
                 * @param {number} length
                 * @param {string=} chars
                 * @return {string}
                 */
                padRight: function padRight(length, chars) {
                    return this.pad(length, chars, 'r');
                },

                /**
                 * Return a padded string using `mode`
                 *
                 * @param {number} length
                 * @param {string=} padding
                 * @param {string=} mode (left|right|both)
                 * @return {string}
                 */
                pad: function pad(length, padding, mode) {
                    mode = mode || 'b';
                    padding = isUndefined(padding) ? ' ' : padding;
                    var result = this,
                        diff = length - result.length,
                        begin = mode.charAt(0).toLowerCase(),

                        //Get the pad character count at the left side
                        padLeft = [diff, 0, ceil(diff / 2)][indexOf('lrb', begin)],

                        //for right side
                        padRight = diff - padLeft,
                        paddingLength = size(padding),
                        insert;

                    if (isUndefined(padLeft)) {
                        throw new ValueError('Unknown pad mode "' + mode + '"');
                    }

                    if (padLeft > 0 || padRight > 0) {
                        if (isEmpty(padding)) {
                            throw new ValueError('padding element should not be empty');
                        }

                        while (padLeft > 0) {
                            insert = padding.slice(max(paddingLength - padLeft, 0));
                            result = insert.concat(result);
                            padLeft -= insert.length;
                        }

                        while (padRight > 0) {
                            insert = padding.slice(0, min(padRight, paddingLength));
                            result = result.concat(insert);
                            padRight -= insert.length;
                        }
                    }
                    return result;
                },

                /**
                 * Return an array that split on the first separator occcurence
                 *
                 * @param {string} separator
                 * @param {string=} direction (left|right)
                 * @return {Array}
                 */
                partition: (function /*factory*/() {
                    var DIRECTION_TO_METHOD = {
                        l: indexOf,
                        r: lastIndexOf
                    };

                    return function partition(separator, direction) {
                        direction = direction || 'l';
                        var self = stringify(this),
                            method = DIRECTION_TO_METHOD[direction.charAt(0)],
                            separatorPosition;

                        if (!method) {
                            throw new ValueError(direction + ' is not recognized.');
                        }
                        separatorPosition = method(self, separator);
                        return (separatorPosition >= 0) ?
                                [self.slice(0, separatorPosition), separator, self.slice(separatorPosition + 1)]:
                                [self, '', ''];
                    };
                }()),

                /**
                 * Format string with `args`
                 *
                 * @see http://docs.python.org/release/3.1.3/library/string.html#format-string-syntax
                 *
                 * @param {Array|Object} args
                 * @return {string}
                 */
                format: function format(args) {
                    var self = stringify(this);
                    return (!self.length ||
                            !args ||
                            isEmpty(args) ||
                            indexOf(self, '{') < 0//no tags in the string)
                            ) ?
                           //simple case
                           self :
                           //format case
                           String.formatter.format(self, args);
                },

                /**
                 * Return true if this is starting with substring
                 *
                 * @param {string} substring
                 * @param {number=} start
                 * @param {number=} end
                 * @return {boolean}
                 */
                startsWith: function startsWith(substring/*, start, end*/) {
                    return !indexOf(this.slice(arguments[1] || 0, arguments[2] || size(this)), substring);
                },

                /**
                 * Return true if this is ending with substring
                 *
                 * @param {string} substring
                 * @param {number=} start
                 * @param {number=} end
                 * @return {boolean}
                 */
                endsWith: function endsWith(substring/*, start, end*/) {
                    var part = this.slice(arguments[1] || 0, arguments[2] || size(this)),
                        diff = size(part) - size(substring);
                    return (diff >= 0 && part.lastIndexOf(substring) === diff);
                },

                /**
                 * Return true if this is only constituted with alpha numeric characters
                 *
                 * @return {boolean}
                 */
                isAlphaNumeric: (function /*factory*/() {
                    var iterator = function (character) {
                        return CHARS_ALPHA[character] || CHARS_DIGIT[character];
                    };
                    return function isAlphaNumeric() {
                        return this.every(iterator);
                    };
                }()),

                /**
                 * Return true if this is only constituted with alpha characters
                 *
                 * @return {boolean}
                 */
                isAlpha: (function /*factory*/() {
                    var iterator = function (character) {
                        return CHARS_ALPHA[character];
                    };
                    return function isAlpha() {
                        return this.every(iterator);
                    };
                }()),

                /**
                 * Return true if this is only constituted with digit characters
                 *
                 * @return {boolean}
                 */
                isDigit: (function /*factory*/() {
                    var iterator = function (character) {
                        return CHARS_DIGIT[character];
                    };
                    return function isDigit() {
                        return this.every(iterator);
                    };
                }()),

                /**
                 * Return a capitalized string
                 *
                 * @return {string}
                 */
                capitalize: function capitalize() {
                    var self = stringify(this);
                    return self.charAt(0).toUpperCase() + self.substring(1).toLowerCase();
                }
            });
        });



        //===============================================================================
        //                                  Date
        //===============================================================================
        class_(Date, function () {

            var

            DateFormatter = class_("Formatter", null, function () {
                var objectGet = Object.get,
                    pad = function (value, length) {
                        return stringify(value).pad(length || 2, "0", "l");
                    },
                    DAYS = ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"],
                    MONTHS = [null, "January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"],
                    REGEXP_TIMEZONE = /\b(?:[PMCEA][SDP]T|(?:Pacific|Mountain|Central|Eastern|Atlantic) (?:Standard|Daylight|Prevailing) Time|(?:GMT|UTC)(?:[\-+]\d{4})?)\b/g,
                    /*jslint regexp:false*/
                    REGEXP_FORMAT_SPEC = /d{1,4}|m{1,4}|yy(?:yy)?|([HhMSTt])\1?|[LlosZ]|"[^"]*"|'[^']*'/g,
                    REGEXP_TIMEZONE_CLIP = /[^\-+\dA-Z]/g
                    /*jslint regexp:true*/;

                properties_({
                    initialize: function () {
                        //optimization
                        this.__replacer__ = functionBindThis(function replacer(token, matches) {
                            return stringify(this.formatToken(token));
                        }, this);
                    },

                    format: function format(date, formatSpec) {
                        var utc = false;
                        if (formatSpec.startsWith("UTC:")) {
                            formatSpec = formatSpec.slice(4);
                            utc = true;
                        }

                        this.date = date;
                        this.utc = utc;
                        this.cache = {};

                        return formatSpec.replace(REGEXP_FORMAT_SPEC, this.__replacer__);
                    },

                    formatToken: function formatToken(token) {
                        var value = objectGet(this, token);
                        return isUndefined(value) ? token.slice(1, token.length - 1) : value;
                    },

                    __get__: function __get__(field) {
                        var cache = this.cache,
                            value = getOwn(cache, field),
                            handler;
                        if (isUndefined(value)) {
                            handler = this["_" + field];
                            value = handler ? handler.call(this) : this.date[(this.utc ? "getUTC" : "get") + field]();
                            cache[field] = value;
                        }
                        return value;
                    },
                    getTimezoneOffset: function () {
                        return this.utc ? 0 : objectGet(this, 'TimezoneOffset');
                    },
                    _d: function () {
                        return objectGet(this, 'Date');
                    },
                    _dd: function () {
                        return pad(objectGet(this, 'd'));
                    },
                    _ddd:  function () {
                        return objectGet(this, 'dddd').substr(0, 3);
                    },
                    _dddd: function () {
                        return DAYS[objectGet(this, 'Day')];
                    },
                    _m: function () {
                        return objectGet(this, 'Month') + 1;
                    },
                    _mm: function () {
                        return pad(objectGet(this, 'm'));
                    },
                    _mmm:  function () {
                        return objectGet(this, 'mmmm').substr(0, 3);
                    },
                    _mmmm:  function () {
                        return MONTHS[objectGet(this, 'm')];
                    },
                    _yy: function () {
                        return stringify(objectGet(this, 'yyyy')).slice(2);
                    },
                    _yyyy: function () {
                        return objectGet(this, 'FullYear');
                    },
                    _h: function () {
                        return objectGet(this, 'H') % 12 || 12;
                    },
                    _hh: function () {
                        return pad(objectGet(this, 'h') % 12 || 12);
                    },
                    _H: function () {
                        return objectGet(this, 'Hours');
                    },
                    _HH: function () {
                        return pad(objectGet(this, 'H'));
                    },
                    _M: function () {
                        return objectGet(this, 'Minutes');
                    },
                    _MM: function () {
                        return pad(objectGet(this, 'M'));
                    },
                    _S: function () {
                        return objectGet(this, 'Seconds');
                    },
                    _SS: function () {
                        return pad(objectGet(this, 'S'));
                    },
                    _l: function () {
                        return pad(objectGet(this, 'Milliseconds'), 3);
                    },
                    _L: function () {
                        var value = objectGet(this, 'Milliseconds');
                        return pad(value > 99 ? Math.round(value / 10) : value);
                    },
                    _t: function () {
                        return objectGet(this, 'tt').charAt(0);
                    },
                    _tt: function () {
                        return objectGet(this, 'H') < 12 ? "am"  : "pm";
                    },
                    _T: function () {
                        return objectGet(this, 't').toUpperCase();
                    },
                    _TT: function () {
                        return objectGet(this, 'tt').toUpperCase();
                    },
                    _Z: function () {
                        if (this.utc) {
                            return "UTC";
                        }
                        var matches = stringify(this.date).match(REGEXP_TIMEZONE),
                            last = (matches && matches.pop()) || "";
                        return last.replace(REGEXP_TIMEZONE_CLIP, "");
                    },
                    _o: function () {
                        var offset = this.getTimezoneOffset();
                        return (offset > 0 ? "-" : "+") + pad(floor(abs(offset) / 60) * 100 + abs(offset) % 60, 4);
                    }//,
                    //_s: function () {
                    //    var day = this._d();
                    //    return ["th", "st", "nd", "rd"][day % 10 > 3 ? 0 : (day % 100 - day % 10 !== 10) * day % 10];
                    //}

                });
            }),
            DAYS_IN_MONTHS = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];


            extend_({
                FORMAT_DEFAULT:     "ddd mmm dd yyyy HH:MM:ss",
                FORMAT_DATE_SHORT:  "m/d/yy",
                FORMAT_DATE_MEDIUM: "mmm d, yyyy",
                FORMAT_DATE_LONG:   "mmmm d, yyyy",
                FORMAT_DATE_FULL:   "dddd, mmmm d, yyyy",
                FORMAT_DATE_ISO:    "yyyy-mm-dd",

                FORMAT_TIME_SHORT:  "h:MM TT",
                FORMAT_TIME_MEDIUM: "h:MM:SS TT",
                FORMAT_TIME_LONG:   "h:MM:SS TT Z",
                FORMAT_TIME_ISO:    "HH:MM:SS",

                FORMAT_DATETIME_ISO:    "yyyy-mm-dd'T'HH:MM:SS",
                FORMAT_DATETIME_ISO_UTC: "UTC:yyyy-mm-dd'T'HH:MM:SS'Z'",
                formatter: new DateFormatter()
            });

            //include_("stj.Object");
            properties_({
                __compare__: function __compare__(other) {
                    return (+this) - (+other);
                },

                __hash__: function __hash__() {
                    return stringify(this);
                },

                __format__: function __format__(formatSpec) {
                    return Date.formatter.format(this, stringify(formatSpec || Date.FORMAT_DEFAULT));
                },

                /**
                 * Return date representation
                 *
                 * @return {string}
                 */
                toRepresentation: function toRepresentation(options) {
                    return this.constructor.getName() + '("' + stringify(this) + '")';
                },

                /**
                 * Return true if this is isEqual to `other`
                 *
                 * @param {string} formatSpec
                 * @return {string}
                 */
                format: function format(formatSpec) {
                    return this.__format__(formatSpec);
                },

                /**
                 * Return true if this is isEqual to `other`
                 *
                 * @param {*} other
                 * @return {boolean}
                 */
                isEqual: function (other) {
                    return +this === +other;
                },

                /**
                 * Return a copy of Date object
                 *
                 * @param {boolean} deep
                 * @return {Date}
                 */
                clone: function clone(deep) {
                    return new this.constructor(+this);
                },

                /**
                 * Return true if this is a leap year
                 *
                 * @return {boolean}
                 */
                isLeapYear: function isLeapYear() {
                    var year = this.getFullYear();
                    return  (year % 400 === 0 || (year % 100 !== 0 && year % 4 === 0));
                },

                /**
                 * Return the number of days in month
                 *
                 * @return {number}
                 */
                getDaysInMonth: function getDaysInMonth() {
                    var month = this.getMonth();
                    return month === 1 && this.isLeapYear() ? 29 : DAYS_IN_MONTHS[month];
                }
            });
        });


        //=========================================================================
        //                                Error
        //=========================================================================
        module_(use_('error'), function () {

            class_(Error, function () {
                var errorToString = functionBindToCall(property_('toString'));

                if (!Error.captureStackTrace) {
                    extend_({
                        captureStackTrace: function (obj, stripPoint) {
                            var error = new Error(),
                                stack = getOwn(error, 'stack');
                            if (stack) {

                                stack = stack.split(EOL);

                                // remove one stack level:
                                if (!isUndefined(Components)) {
                                    // Mozilla:
                                    stack.shift();
                                }

                                //Remove first calls
                                stack.splice(0, 2);
                                stack.unshift(stringify(obj));

                                obj.stack = stringify(stack.join(EOL));
                            }
                        }
                    });
                }

                //include_("stj.Object");

                //Patch to use the string.format function
                property_('toString', function toString() {
                    var message = this.message;
                    return errorToString({
                        name: this.name,
                        message: message && message.format ? message.format(this) : message
                    });
                });
            });

            //common code
            var errorClassFactory = function (Constructor) {

                properties_(Constructor.prototype, {
                    name: stj.meta_(Constructor).baseName(),

                    initialize: function initialize(message) {
                        //If object then extends
                        if (isString(message)) {
                            this.message = message;
                        } else {
                            merge(this, message);
                        }
                        Constructor.applySuper('initialize', this, arguments);
                        Error.captureStackTrace(this, Constructor);
                    }
                });
            };

            /**
             * AssertionError class
             *
             * @class error.AssertionError
             * @extends {Error}
             */
            class_('AssertionError', Error, errorClassFactory);

            /**
             * ValueError class
             *
             * @class
             * @extends {Error}
             */
            ValueError = class_('ValueError', Error, errorClassFactory);

            /**
             * RuntimeError class
             *
             * @class
             * @extends {Error}
             */
            class_('RuntimeError', Error, errorClassFactory);

            /**
             * NotImplementedError class
             *
             * @class
             * @extends {Error}
             */
            class_('NotImplementedError', Error, errorClassFactory);

            /**
             * NotImplementedError class
             *
             * @class
             * @extends {Error}
             */
            class_('IOError', Error, errorClassFactory);

            /**
             * SystemExit class
             *
             * @class
             * @extends {Error}
             */
            class_('SystemExit', Error, errorClassFactory);

            /**
             * StopIteration class
             *
             * @class
             * @extends {Error}
             */
            StopIteration = class_('StopIteration', null, function () {
                properties_({
                    name: 'StopIteration'
                });
            });
        });

        //=========================================================================
        //                                Array
        //=========================================================================
        class_(Array, function () {
            properties_({
                __forEach__: property_('forEach'),//force use of native function

                __get__: function __get__(index) {
                    index = this._normalizeSlice(index);
                    return isArray(index) ? this.slice(index[0], index[1]) : this[index];
                },

                __set__: function __set__(index, value) {
                    index = this._normalizeSlice(index);
                    if (isArray(index)) {
                        this.splice.apply(this, index.concat(value));
                    } else {
                        this[index] = value;
                    }
                },

                __unset__: function __unset__(index) {
                    var start = index,
                        count = 1;
                    if (isArray(index)) {
                        start = index[0];
                        count = index[1] - index[0];
                    }
                    return this.splice(start, count);
                },

                toRepresentation: function toRepresentation(options) {
                    return '[' + Object.map(this, function (value, key) {
                        return Object.representation(value, options);
                    }).join(options.propertySeparator) + ']';
                },

                /**
                 * Destroy the array
                 */
                finalize: function finalize() {
                    this.clear();
                },

                /**
                 * Return a new array repeating this `count` times
                 *
                 * @param {number} count
                 * @return {Array}
                 */
                repeat: function repeat(count) {
                    var result = [],
                        i = count || 0;

                    while (i > 0) {
                        result = result.concat(this);
                        i -= 1;
                    }
                    return result;
                },

                /**
                 * Return a new array with duplicated values removed
                 *
                 * @return {Array}
                 */
                unique: (function /*factory*/() {
                    var iterator = function (value, index, self) {
                            var results = returnGet(this);
                            if (!~indexOf(results, value)) {
                                results.push(value);
                            }
                        };
                    return function unique() {
                        return forEach(this, iterator, returnSet({}, []));
                    };
                }()),

                /**
                 * Return flattened array
                 *
                 * @return {Array}
                 */
                flatten: (function /*factory*/() {
                    var iterator = function (value) {
                        returnSet(this, returnGet(this).concat(isArray(value) ? value.flatten() : value));
                    };
                    return function flatten() {
                        return forEach(this, iterator, returnSet({}, []));
                    };
                }()),

                /**
                 * Remove first `element` found in this array
                 *
                 * @param {*} element
                 * @return {*} removed element
                 */
                remove: function remove(element) {
                    return Object.unset(this, indexOf(this, element));
                },

                /**
                 * Empty the array
                 *
                 * @return this
                 */
                clear: function clear() {
                    this.length = 0;
                    return this;
                },

                /**
                 * Left pad this array
                 *
                 * @param {number} length
                 * @param {*=} padding
                 * @return {Array}
                 */
                padLeft: function padLeft(length, padding) {
                    return this.pad(length, padding, 'l');
                },

                /**
                 * Right pad this array
                 *
                 * @param {number} length
                 * @param {*=} padding
                 * @return {Array}
                 */
                padRight: function padRight(length, padding) {
                    return this.pad(length, padding, 'r');
                },

                /**
                 * Pad this array using `mode`
                 *
                 * @param {number} length
                 * @param {*=} padding (left|right|both)
                 * @param {string=} mode
                 * @return {Array}
                 */
                pad: function pad(length, padding, mode) {
                    mode = mode || 'b';
                    var result = slice(this, 0),
                        diff = length - +result.length,
                        begin = mode.charAt(0).toLowerCase(),

                        //Get the pad character count at the left side
                        padLeft = [diff, 0, ceil(diff / 2)][indexOf('lrb', begin)],

                        //for right side
                        padRight = diff - padLeft,
                        insert;

                    if (isUndefined(padLeft)) {
                        throw new ValueError('Unknown pad mode "' + mode + '"');
                    }

                    if (padLeft > 0 || padRight > 0) {
                        if (Object.isEmpty(padding)) {
                            throw new ValueError('padding element should not be empty');
                        }

                        //add left values
                        insert = new Array(padLeft);
                        while (padLeft > 0) {
                            insert[padLeft - 1] = padding;
                            padLeft -= 1;
                        }
                        result = insert.concat(result);


                        //add right values
                        //insert = new Array(padRight);
                        while (padRight > 0) {
                            result.push(padding);
                            padRight -= 1;
                        }
                    }

                    return result;
                },

                /**
                 * Extends this with `array`
                 *
                 * @param {Array} array
                 * @return this
                 */
                update: (function /*factory*/() {
                    var iterator = function (value) {
                            returnGet(this).push(value);
                        };
                    return function update(array) {
                        return forEach(array, iterator, returnSet({}, this));
                    };
                }())
            });

            include_([ "stj.Sequence" ]);
            extend_({
                __isInstanceOf__: isArray
            });
        });
        GLOBAL.__STD__ = true;
        return stj;
    });
}(typeof define !== "undefined" && false ? define : function (name, deps, factory) {
    if (typeof module !== "undefined") {
        module.id = name;
        module.exports = factory();
    } else {
        this[name] = factory();
    }
    if (typeof dojo !== "undefined") {
        /*jslint sub:true */
        dojo["provide"](name);
        /*jslint sub:false */
    }
}, Object, Array, Boolean, Date, Function, Math, Number, RegExp, String, Error, TypeError)//inject natives
);
