/*jslint nodejs: true, indent:4 */
/**
 * Generic Imports
 */
var vows = require('vows'),
    assert = require('assert'),
    env = require('./_env'),
    __filenameTested = env.toFilenameTested(__filename),
    stj = require(env.lib('stj')),
    Deferred = require(__filenameTested),
    isPromise = Deferred.isPromise,

    defer = function (options) {
        return new Deferred(options);
    };


/*******************************************************************************
 * JSLint validation
 ******************************************************************************/
exports.JSLintTest = env.JSLintTest([__filenameTested, __filename]);

/*******************************************************************************
 * DeferredTest
 ******************************************************************************/
exports.DeferredTest = vows.describe('Promise').addBatch({
    'new Deferred())': {
        topic: function () {
            return defer();
        },
        "should be instance of Promise": function (topic) {
            assert.ok(topic instanceof Deferred);
            assert.ok(Deferred.isPromise(topic));

            /*topic = defer();
            assert.ok(topic instanceof Deferred);
            assert.ok(Deferred.isPromise(topic));*/
        }
    },
    'emitSuccess()': {
        topic: function () {
            var self = this,
                report = {
                    promise: new Deferred(),
                    resultCount: 0,
                    errorCount: 0
                };

            Deferred.every([
                report.promise.then(function (val) {
                    throw "throwed_error";
                }).then(null, function (error) {
                    report.error = error;
                    report.errorCount += 1;
                }),
                report.promise.then(function (result) {
                    report.result = result;
                    report.resultCount += 1;
                })
            ]).onAny(function () {
                self.callback(null, report);
            });

            setTimeout(function () {
                report.promise.emitSuccess('successful result');
            }, 1);
        },
        "should have length of 1": function (topic) {
            assert.equal(Deferred.prototype.emitSuccess.length, 1);
        },
        "should be instance of Promise": function (topic) {
            assert.ok(isPromise(topic.promise));
        },
        "should emitSuccess passing value to the callback": function (topic) {
            assert.equal(topic.result, 'successful result');
            assert.equal(topic.resultCount, 1);
        },
        "should catch errors": function (topic) {
            assert.equal(topic.error, 'throwed_error');
            assert.equal(topic.errorCount, 1);
        }
    },
    'emitError()': {
        topic: function () {
            var self = this,
                report = {
                    promise: new Deferred()
                };
            report.promise.then(function (result) {
                report.result = result;
            }, function (error) {
                report.error = error;
                var deferred = new Deferred();
                setTimeout(function () {
                    deferred.emitSuccess('result1');
                }, 1);
                return deferred;
            }).then(function (result) {
                report.result2 = result;
                self.callback(null, report);
            });

            setTimeout(function () {
                report.promise.emitError('error1');
            }, 1);
        },
        "should have length of 1": function (topic) {
            assert.equal(Deferred.prototype.emitError.length, 1);
        },
        "should be instance of Promise": function (topic) {
            assert.ok(isPromise(topic.promise));
        },
        "should emitError passing error to the callback": function (topic) {
            assert.equal(topic.error, 'error1');
            assert.equal(topic.result2, 'result1');
        }
    },
    'then()': {
        topic: function () {
            var self = this,
                report = {
                    promise1: new Deferred()
                };
            report.promise1.then(function (result) {
                report.result1 = result;
                var deferred = new Deferred();
                setTimeout(function () {
                    deferred.emitSuccess('result2');
                }, 1);
                return deferred;
            }).then(function (result) {

                report.result2 = result;
                return "result3";
            }).then(function (result) {
                report.result3 = result;
                self.callback(null, report);
            });

            setTimeout(function () {
                report.promise1.emitSuccess('result1');
            }, 1);
        },
        "should have length of 2": function (topic) {
            assert.equal(Deferred.prototype.then.length, 2);
        },
        "should be instance of Promise": function (topic) {
            assert.ok(isPromise(topic.promise1));
        },
        "should emitSuccess passing result to the callback": function (topic) {
            assert.equal(topic.result1, 'result1');
            assert.equal(topic.result2, 'result2');
            assert.equal(topic.result3, 'result3');
        }
    },
    'cancel()': {
        topic: function () {
            var self = this,
                report = {
                    promise: defer({
                        cancel: function () {
                            return new Error('foobar_cancelled');
                        }
                    })
                };
            report.promise.then(function (result) {
                report.value = 'success';
                self.callback(null, report);
            }, function (error) {
                report.error = error;
                self.callback(null, report);
            });

            setTimeout(function () {
                report.promise.cancel();

                try {
                    report.promise.emitSuccess();
                } catch (e) {
                    //Do nothing
                }
            }, 1);
        },
        "should have length of 0": function (topic) {
            assert.equal(Deferred.prototype.cancel.length, 0);
        },
        "should be instance of Promise": function (topic) {
            assert.ok(isPromise(topic.promise));
        },
        "should emitError passing canceller() result as error": function (topic) {
            assert.ok(topic.error instanceof Error);
            assert.equal(topic.error.message, 'foobar_cancelled');
        }
    },
    'setTimeout()': {
        topic: function () {
            var self = this,
                finished = [],
                report = {
                    status: 'unknown',
                    promise: defer({
                        cancel: function () {
                            return 'cancelled';
                        }
                    }),
                    promiseAborted: defer({
                        cancel: function () {
                            return 'cancelled';
                        }
                    }),
                    promiseAborted2: defer({
                        cancel: function () {
                            return 'cancelled';
                        }
                    })
                };

            //Promise timeout
            finished.push(
                report.promise.then(function (result) {
                    report.status = 'success';
                }, function (error) {
                    report.status = 'error';
                    report.error = error;
                })
            );
            report.promise.setTimeout(1);


            //Promise that will not timeout
            finished.push(
                report.promiseAborted.then(function (result) {
                    report.statusAborted = 'success';
                }, function (error) {
                    report.statusAborted = 'error';
                    report.errorAborted = error;
                })
            );
            report.promiseAborted.setTimeout(1);
            report.promiseAborted.setTimeout(null);

            //Promise that will not timeout too
            finished.push(
                report.promiseAborted2.then(function (result) {
                    report.statusAborted2 = 'success';
                }, function (error) {
                    report.statusAborted2 = 'error';
                    report.errorAborted2 = error;
                })
            );
            report.promiseAborted2.setTimeout(1);
            report.promiseAborted2.setTimeout(Infinity);

            Deferred.every(finished).onAny(function () {
                self.callback(null, report);
            });

            setTimeout(function () {
                try {
                    report.promise.emitSuccess();
                } catch (e1) {
                    //Do nothing
                }

                try {
                    report.promiseAborted.emitSuccess();
                    report.promiseAborted2.emitSuccess();
                } catch (e2) {
                    //Do nothing
                }
            }, 3);

        },
        "should have length of 1": function (topic) {
            assert.equal(Deferred.prototype.setTimeout.length, 1);
        },
        "should return instance of Promise": function (topic) {
            assert.ok(isPromise(topic.promise));
        },
        "should emitError after timeout": function (topic) {
            assert.equal(topic.status, 'error');
        },
        "should remove timeout if called as timeout(null)": function (topic) {
            assert.equal(topic.statusAborted, 'success');
            assert.equal(topic.statusAborted2, 'success');
        }
    },

    'Deferred.when()': {
        topic: function () {
            var self = this,
                report = {
                    promise1: new Deferred(),
                    promise1Result: undefined,

                    promise2: 'i-am-not-a-promise',
                    promise2Result: undefined
                },
                count = 6,
                onResolution = function () {
                    count -= 1;
                    if (count === 0) {
                        self.callback(null, report);
                    }
                };

            //async promise
            report.promise1Result = Deferred.when(report.promise1, function (result) {
                report.promise1AsyncResult1 = result;
                onResolution();
            });

            Deferred.when(report.promise1, function (result) {
                report.promise1AsyncResult2 = result;
                onResolution();
            });
            Deferred.when(report.promise1).then(function (result) {
                report.promise1AsyncResult3 = result;
                onResolution();
            });

            //sync promise
            report.promise2Result = Deferred.when(report.promise2, function (result) {
                report.promise2AsyncResult1 = result;
                onResolution();
            });

            Deferred.when(report.promise2, function (result) {
                report.promise2AsyncResult2 = result;
                onResolution();
            });
            Deferred.when(report.promise2).then(function (result) {
                report.promise2AsyncResult3 = result;
                onResolution();
            });

            setTimeout(function () {
                report.promise1.emitSuccess('successful result');
            }, 1);

        },
        "should have length of 3": function (topic) {
            assert.equal(Deferred.when.length, 3);
        },
        "should return instance of Promise": function (topic) {
            assert.ok(isPromise(topic.promise1Result));
            assert.ok(isPromise(topic.promise2Result));
        },
        "should work with a promise as first argument": function (topic) {
            assert.equal(topic.promise1AsyncResult1, 'successful result');
            assert.equal(topic.promise1AsyncResult2, 'successful result');
        },
        "should work with any other value as first argument": function (topic) {
            assert.equal(topic.promise2AsyncResult1, 'i-am-not-a-promise');
            assert.equal(topic.promise2AsyncResult2, 'i-am-not-a-promise');
        },
        "should emitSuccess/emitError on returned promise": function (topic) {
            assert.equal(topic.promise1AsyncResult3, 'successful result');
            assert.equal(topic.promise2AsyncResult3, 'i-am-not-a-promise');
        }

    },
    'Deferred.every()': {
        topic: function () {
            var self = this,
                promises = [
                    defer(),
                    defer(),
                    defer(),
                    defer(),
                    defer()
                ],
                promisesError = [
                    defer(),
                    defer(),
                    defer(),
                    defer(),
                    defer()
                ],
                report = {
                    returnValue: Deferred.every(promises),
                    returnValueErrored: Deferred.every(promisesError)
                },
                reportCount = 2,
                fulfilled = function (result) {
                    reportCount -= 1;
                    if (!reportCount) {
                        self.callback(null, report);
                    }
                };

            report.returnValue.then(function (result) {
                report.result = result;
                fulfilled();
            }, fulfilled);

            report.returnValueErrored.then(
                fulfilled,
                function (errors) {
                    report.resultError = errors;
                    fulfilled();
                }
            );

            //emit success/error
            promises.forEach(function (promiseObj, index) {
                setTimeout(function () {
                    promiseObj.emitSuccess('result:' + index);
                }, index * 10 + 1);
            });
            promisesError.forEach(function (promiseObj, index) {
                setTimeout(function () {
                    promiseObj.emitError('error:' + index);
                }, index === 3 ? 1 : 2);//force promise 3 to fail first
            });
        },
        "should have length of 1": function (topic) {
            assert.equal(Deferred.every.length, 1);
        },
        "should return instance of Promise": function (topic) {
            assert.ok(isPromise(topic.returnValue));
        },
        "should emitSuccess passing value to the callback": function (topic) {
            assert.deepEqual(topic.result, [
                'result:0',
                'result:1',
                'result:2',
                'result:3',
                'result:4'
            ]);
        },
        "should emitError if at least one promise emit an error": function (topic) {
            assert.isString(topic.resultError);
            assert.equal(topic.resultError, "error:3");
        }
    },
    'Deferred.some()': {
        topic: function () {
            var self = this,
                promises = [
                    defer(),
                    defer(),
                    defer(),
                    defer(),
                    defer(),
                ],
                report = {
                    returnValue: Deferred.some(promises)
                };
            Deferred.when(report.returnValue, function (result) {
                report.result = result;
                self.callback(null, report);
            });

            promises.forEach(function (promiseObj, index) {
                setTimeout(function () {
                    promiseObj.emitSuccess('result:' + index);
                }, index === 3 ? 1 : 2);//force promise 3 to fail first
            });
        },
        "should have length of 1": function (topic) {
            assert.equal(Deferred.some.length, 1);
        },
        "should return instance of Promise": function (topic) {
            assert.ok(isPromise(topic.returnValue));
        },
        "should emitSuccess passing value to the callback": function (topic) {
            assert.equal(topic.result, 'result:3');
        }
    },
    'Object.get(promise)': {
        topic: function () {
            var self = this,
                promiseFoo = defer(),
                report = {
                    returnValue: Object.get(promiseFoo, 'fooKey')
                };

            Deferred.when(report.returnValue, function (result) {
                report.result = result;
                self.callback(null, report);
            });

            setTimeout(function () {
                promiseFoo.emitSuccess({
                    fooKey: 'fooValue'
                });
            }, 1);
        },
        "should return instance of Promise": function (topic) {
            assert.ok(isPromise(topic.returnValue));
        },
        "should emitSuccess passing value to the callback": function (topic) {
            assert.equal(topic.result, 'fooValue');
        }
    },
    'Object.set(promise)': {
        topic: function () {
            var self = this,
                promiseFoo = defer(),
                report = {
                    target: {fooKey: 'fooValue'},
                    returnValue: Object.set(promiseFoo, 'fooKey', 'barValue')
                };

            Deferred.when(report.returnValue, function (result) {
                report.result = result;
                self.callback(null, report);
            });

            setTimeout(function () {
                promiseFoo.emitSuccess(report.target);
            }, 1);
        },
        "should return instance of Promise": function (topic) {
            assert.ok(isPromise(topic.returnValue));
        },
        "should emitSuccess passing value to the callback": function (topic) {
            assert.equal(topic.target.fooKey, 'barValue');
        }
    },

    'Deferred.delay()': {
        topic: function () {
            var self = this,
                report = {
                    currentTime: (new Date()).getTime(),
                    returnValue: Deferred.delay(10),
                    delayedTime: null
                };
            Deferred.when(report.returnValue, function () {
                report.delayedTime = (new Date()).getTime();
                self.callback(null, report);
            });
        },
        "should return instance of Promise": function (topic) {
            assert.ok(Deferred.isPromise(topic.returnValue));
        },
        "should emitSuccess passing value to the callback": function (topic) {
            assert.ok(topic.delayedTime + 1 >= topic.currentTime + 10);//1 is the incertitude strange v8 behavior??
            assert.ok(topic.delayedTime + 1 >= topic.currentTime + 10);
        }
    },

    'Deferred.run()': {
        topic: function () {
            var self = this,
                report = {},
                asyncFunction = function (firstArg, secondArg) {
                    var deferred = defer();
                    setTimeout(function () {
                        report.firstArg = firstArg;
                        report.secondArg = secondArg;
                        deferred.emitSuccess("result");
                    }, 5);
                    return deferred;
                };
            report.promise = Deferred.run(asyncFunction, {myObject: true}, ['foo', 'bar']);
            report.promise.then(function (result) {
                report.returnValue = result;
                self.callback(null, report);
            });
        },
        "should return instance of Promise": function (topic) {
            assert.ok(isPromise(topic.promise));
        },
        "should emitSuccess passing value to the callback": function (topic) {
            assert.equal(topic.firstArg, 'foo');
            assert.equal(topic.secondArg, 'bar');
            assert.equal(topic.returnValue, 'result');

        }

    },

    'Deferred.reduce()': {
        topic: function () {
            var self = this,
                generate = function (delay, suffix) {
                    return function (value) {
                        var concat = defer();
                        Deferred.when(Deferred.delay(delay), function () {
                            concat.emitSuccess(value + suffix);
                        });
                        return concat;
                    };
                },
                promises = [
                    generate(10, '1/'),
                    generate(2, '2/'),
                    generate(3, '3/'),
                    generate(1, '4/'),
                    generate(11, '5/')
                ],
                report = {
                    returnValue: Deferred.reduce(promises, '/')
                };

            Deferred.when(report.returnValue, function (result) {
                report.result = result;
                self.callback(null, report);
            });
        },
        "should return instance of Promise": function (topic) {
            assert.ok(isPromise(topic.returnValue));
        },
        "should emitSuccess passing value to the callback": function (topic) {
            assert.equal(topic.result, '/1/2/3/4/5/');
        }
    }
});