/*jslint nodejs:true, indent:4 */
var vows = require('vows'),
    assert = require('assert'),
    env = require('./_env'),
    __filenameTested = env.toFilenameTested(__filename),
    stj = require(env.lib('stj')),
    Dictionary = require(__filenameTested);

/*******************************************************************************
 * JSLint validation
 ******************************************************************************/
exports.JSLintTest = env.JSLintTest([__filenameTested, __filename]);

/*******************************************************************************
 * Dictionary Test
 ******************************************************************************/
function createDictionary(data) {
    return (new Dictionary(data));
}
exports.DictionaryTest = vows.describe('Dictionary class').addBatch({
    "new Dictionary(data)": {
        topic : function (item) {
            return createDictionary(createDictionary({foo: 'bar', baz: 1, bar: 'toto'}));
        },
        'should construct a copy if data is a Hash' : function (topic) {
            assert.deepEqual(topic, createDictionary({foo: 'bar', baz: 1, bar: 'toto'}));
        }
    },
    "isEmpty()": {
        topic : function (item) {
            return createDictionary({});
        },
        'should return true only for empty Dictionary' : function (topic) {
            assert.equal(createDictionary().isEmpty(), true);
            assert.equal(createDictionary({foo: 'bar', baz: 1, bar: 'toto'}).isEmpty(), false);
        }
    },
    "isEqual()": {
        topic : function (item) {
            return createDictionary({foo: 'bar', baz: 1, bar: 'toto'});
        },
        'should return true if compared to itself' : function (topic) {
            assert.equal(topic.isEqual(topic), true);
        },
        'should return true if compared to a clone' : function (topic) {
            assert.equal(topic.isEqual(createDictionary({foo: 'bar', baz: 1, bar: 'toto'})), true);
        },
        'should return false if types are different' : function (topic) {
            assert.equal(topic.isEqual('fldjslfkjdlks'), false);
            assert.equal(topic.isEqual(1), false);
            assert.equal(topic.isEqual({foo: 'bar', baz: 1, bar: 'toto'}), false);
        },
        'should return false if hash are differents' : function (topic) {
            assert.equal(topic.isEqual(createDictionary({foo: 'bar', baz: 1, bar: 'tot'})), false);
            assert.equal(topic.isEqual(createDictionary({foo: 'bar', baz: 1})), false);
            assert.equal(topic.isEqual(createDictionary({foo: 'bar', baz: 1, bar: 'toto', babar: 'hello'})), false);
        }
    },
    "Object.hash(dict)": {
        topic : function (item) {
            return createDictionary({foo: 'bar', baz: 1, bar: 'toto'});
        },
        'should return empty string for empty hash' : function (topic) {
            assert.equal(Object.hash(createDictionary()), '');
        },
        'should return sorted hash of (key, value) hashes' : function (topic) {
            assert.equal(Object.hash(topic), '1bazbarfoobartoto');
        }
    },
    "get()" : {
        topic : function (item) {
            return createDictionary();
        },
        'should return undefined if key does not exists' : function (topic) {
            assert.equal(topic.get('fldjslfkjdlks'), undefined);
        },
        'should return defaultValue if key does not exists' : function (topic) {
            assert.equal(topic.get('fldjslfkjdlks', 'defaultArg'), 'defaultArg');
        },
        'should return data if key exists' : function (topic) {
            topic.set('myId', 'helloworld');
            assert.equal(topic.get('myId'), 'helloworld');
        }
    },
    "set()" : {
        topic : function (item) {// Topic
            return createDictionary();
        },
        'should return this' : function (topic) {
            assert.equal(topic.set('fldjslfkjdlks', 'jkljlm'), topic);
        },
        'should return set data' : function (topic) {
            topic.set('myId', 'helloworld');
            assert.equal(topic.get('myId'), 'helloworld');
        }
    },
    "isset() " : {
        topic : function (item) {// Topic
            return createDictionary();
        },
        'should return false if key does not exists' : function (topic) {
            assert.equal(topic.isset('fsdqfjdlk'), false);
        },
        'should return true if key exists' : function (topic) {
            topic.set('myId', 'helloworld');
            assert.equal(topic.isset('myId'), true);
        }
    },
    "unset()" : {
        topic : function (item) {// Topic
            return createDictionary();
        },
        'should return default param if key does not exist' : function (topic) {
            assert.equal(topic.unset('jakhhk', 'defaultValue'), 'defaultValue');
        },
        'should delete element and return this element' : function (topic) {
            topic.set('myId', 'helloworld');
            assert.equal(topic.isset('myId'), true);
            var result = topic.unset('myId', 'defaultValue');
            assert.equal(topic.isset('myId'), false);
            assert.equal(result, 'defaultValue');
        }
    },
    "keyOf()" : {
        topic : function (item) {
            return createDictionary({foo: 'foofoo', bar: 1, baz: 'fjdsklfjsdkl'});
        },
        'should return undefined if value is not found' : function (topic) {
            assert.isUndefined(topic.keyOf('nonexistent'));
            assert.isUndefined(topic.keyOf('foo'));
        },
        'should return key if value is found' : function (topic) {
            assert.equal(topic.keyOf('foofoo'), 'foo');
            assert.equal(topic.keyOf(1), 'bar');
        }
    },
    "keys()" : {
        topic : function (item) {
            return createDictionary();
        },
        'should return array containing all data keys' : function (topic) {
            assert.deepEqual(topic.keys(), []);
            topic.set('foo', 1);
            assert.deepEqual(topic.keys(), [ 'foo' ]);
            topic.set('bar', 2);
            assert.deepEqual(topic.keys(), [ 'foo', 'bar' ]);
            topic.unset('foo');
            assert.deepEqual(topic.keys(), [ 'bar' ]);
        }
    },
    "values()" : {
        topic : function (item) {
            return createDictionary();
        },
        'should return array containing all data keys' : function (topic) {
            assert.deepEqual(topic.values(), []);
            topic.set('foo', 1);
            assert.deepEqual(topic.values(), [ 1 ]);
            topic.set('bar', 2);
            assert.deepEqual(topic.values(), [ 1, 2 ]);
            topic.unset('foo');
            assert.deepEqual(topic.values(), [ 2 ]);
        }
    },
    "size()" : {
        topic : function (item) {
            return createDictionary();
        },
        'should return the key count' : function (topic) {
            assert.equal(topic.size(), 0);
            assert.equal(Object.size(topic), 0);
            topic.set('foo', 1);
            assert.equal(topic.size(), 1);
            assert.equal(Object.size(topic), 1);
            topic.set('bar', 2);
            assert.equal(topic.size(), 2);
            assert.equal(Object.size(topic), 2);
            topic.unset('foo');
            assert.equal(topic.size(), 1);
            assert.equal(Object.size(topic), 1);
            topic.unset('foo');
            assert.equal(topic.size(), 1);
            assert.equal(Object.size(topic), 1);
        }
    },
    "update()" : {
        topic : function (item) {
            return createDictionary();
        },
        'should return update with an object' : function (topic) {
            topic.set('foo', 1);

            topic.update();
            assert.equal(topic.get('foo'), 1);

            topic.update({
                bar : 2,
                baz : 3
            });
            assert.equal(topic.get('foo'), 1);
            assert.equal(topic.get('bar'), 2);
            assert.equal(topic.get('baz'), 3);

            topic.update({
                bar : 0,
                baz : 0
            });
            assert.equal(topic.get('foo'), 1);
            assert.equal(topic.get('bar'), 0);
            assert.equal(topic.get('baz'), 0);
        },
        'should extends with a Dictionary' : function (topic) {
            topic.set('foo', 1);

            topic.update();
            assert.equal(topic.get('foo'), 1);

            topic.update(createDictionary({
                bar : 2,
                baz : 3
            }));
            assert.equal(topic.get('foo'), 1);
            assert.equal(topic.get('bar'), 2);
            assert.equal(topic.get('baz'), 3);

            topic.update(createDictionary({
                bar : 0,
                baz : 0
            }));
            assert.equal(topic.get('foo'), 1);
            assert.equal(topic.get('bar'), 0);
            assert.equal(topic.get('baz'), 0);
        }
    },
    "filter()" : {
        topic : function (item) {
            return createDictionary();
        },
        'should return new Hash statisfying iterator' : function (topic) {
            topic.update({
                foo : 1,
                bar : 2,
                baz : 'toto'
            });

            assert.deepEqual(topic.filter(function () {
            }), []);
            assert.deepEqual(topic.filter(function (el) {
                return el === 'toto';
            }), ['toto']);

            topic.set('baz', 'tata');
            assert.deepEqual(topic.filter(function (el) {
                return el === 'toto';
            }), []);
        }
    },
    "clear()" : {
        topic : function (item) {
            return createDictionary({});
        },
        'should return this' : function (topic) {
            assert.equal(topic.clear(), topic);
        },
        'should empty an already empty dictionary without error' : function (topic) {
            topic.clear();
            assert.deepEqual(topic, createDictionary());
        },
        'should empty the content' : function (topic) {
            topic.update({
                foo : 1,
                bar : 2,
                baz : 'toto'
            });

            assert.equal(topic.size(), 3);

            topic.clear();
            assert.equal(topic.size(), 0);
        }
    },
    "clone()" : {
        topic : function (item) {
            var hashObj = createDictionary({});
            hashObj.set('foo', 1);
            hashObj.set('bar', 2);
            hashObj.set('hashObject', createDictionary({
                ahah : 3
            }));
            hashObj.set('anonymous', {
                ohoh : 3
            });
            return hashObj;
        },
        'should return copy of this dictionary' : function (topic) {
            var clone = topic.clone();
            clone.set('foo', 3);

            assert.equal(topic.get('foo'), 1);
            assert.equal(clone.get('foo'), 3);
            assert.equal(clone.get('bar'), 2);
        },
        'should not clone hash values if deep=false' : function (topic) {
            var clone = topic.clone();
            clone.set('foo', 3);

            assert.equal(clone.get('baz'), topic.get('baz'));
        },
        'should clone hash values if deep=true' : function (topic) {
            var clone = topic.clone(true);

            assert.notEqual(clone.get('hashObject').size, undefined);
            assert.deepEqual(clone.get('hashObject'), topic.get('hashObject'));
            assert.notEqual(clone.get('hashObject'), topic.get('hashObject'));

            assert.deepEqual(clone.get('anonymous'), topic.get('anonymous'));
            assert.notEqual(clone.get('anonymous'), topic.get('anonymous'));
        }
    },
    "map()" : {
        topic : function (item) {
            var obj = createDictionary({
                foo: 1,
                bar: 2,
                hashObject: createDictionary({
                    ahah : 3
                }),
                anonymous: {
                    ohoh : 3
                }
            });
            return obj;
        },
        'should map an empty dictionary': function () {
            var result = createDictionary().map(function (value, key) {});

            assert.ok(result instanceof Array);
            assert.deepEqual(result, []);
        },
        'should return a mapped dictionary' : function (topic) {
            var result = topic.map(function (value, key) {
                if (key === 'foo') {
                    return 'helloworld';
                }
                if (value === 2) {
                    return 3;
                }
            });

            assert.ok(result instanceof Array);
            assert.deepEqual(result, ['helloworld', 3]);
        }
    },
    "every()" : {
        topic : function (item) {
            return createDictionary();
        },
        'should always return true for empty dictionary': function (topic) {
            var obj = new Dictionary();
            assert.equal(obj.every(function (value, key) {
                return false;
            }), true);
        },
        'should return true when every value satisfy filter': function (topic) {
            var obj = new Dictionary({
                foo: true,
                bar: true,
                baz: 'toto'
            });
            assert.equal(obj.every(function (value, key) {
                return key.length === 3;
            }), true);
        },
        'should return false when one value does not satisfy filter': function (topic) {
            var obj = new Dictionary({
                foo: true,
                bar: true,
                baz: 'toto'
            });
            assert.equal(obj.every(function (value, key) {
                return value === true;
            }), false);
        }
    },
    "some()" : {
        topic : function (item) {
            return createDictionary();
        },
        'should always return false for empty dictionary': function (topic) {
            var obj = new Dictionary();
            assert.equal(obj.some(function (value, key) {
                return true;
            }), false);
        },
        'should return true when at least value satisfy filter': function (topic) {
            var obj = new Dictionary({
                foo: true,
                bar: true,
                baz: 'toto'
            });
            assert.equal(obj.some(function (value, key) {
                return value === "toto";
            }), true);
        },
        'should return false when all value does not satisfy filter': function (topic) {
            var obj = new Dictionary({
                foo: true,
                bar: true,
                baz: 'toto'
            });
            assert.equal(obj.some(function (value, key) {
                return value === false;
            }), false);
        }
    },
    "toString()" : {
        topic : function (item) {
            return 'toString';
        },
        'should always return "" for empty dictionary': function (topic) {
            var obj = new Dictionary({});
            assert.equal(obj[topic](), '');
        },
        'should always return key1:value1,key2:value2 representation for empty dictionary': function (topic) {
            assert.equal((new Dictionary({'foo': 'foofoo'}))[topic](), 'foo:foofoo');
            assert.equal((new Dictionary({'foo': 1, 'bar': 2}))[topic](), 'foo:1,bar:2');
        }
    }
});