/*jslint nodejs:true, indent:4 */
var vows = require('vows'),
    assert = require('assert'),
    env = require('./_env'),
    __filenameTested = env.lib('stj'),
    nativeMethods = {
        forEach: Array.prototype.forEach,
        indexOf: Array.prototype.indexOf,
        toString: Array.prototype.toString
    },
    stj = require(__filenameTested);

/*******************************************************************************
 * JSLint validation
 ******************************************************************************/
exports.JSLintTest = env.JSLintTest([/*__filenameTested, */__filename]);

/*******************************************************************************
 * ArrayTest class
 ******************************************************************************/
exports.ArrayTest = vows.describe('Array functions').addBatch({
    "[native methods]": {
        topic: function () {
            return Array.prototype;
        },
        'should not change native implementation': function (topic) {
            for (var methodName in nativeMethods) {
                if (nativeMethods.hasOwnProperty(methodName)) {
                    assert.equal(nativeMethods[methodName], topic[methodName]);
                    assert.equal(topic[methodName].toString(), "function " + methodName + "() { [native code] }");
                }
            }
        }
    },
    "isEqual()": {
        topic: function () {
            return Array.prototype.isEqual.bindToCall();
        },
        "should have length of 1": function (topic) {
            assert.equal(Array.prototype.isEqual.length, 1);
        },
        "should return true for same object": function (topic) {
            var array = ['foo', 'bar', 'baz'],
                clone = ['foo', 'bar', 'baz'];
            assert.equal(topic([], []), true);
            assert.equal(topic(array, array), true);
            assert.equal(topic(array, clone), true);
        },
        "should return false for different values": function (topic) {
            var array = ['foo', 'bar', 'baz'];

            assert.equal(topic(array, []), false);
            assert.equal(topic(array, "123"), false);
            assert.equal(topic(array, false), false);
            assert.equal(topic(array, {}), false);
        }
    },
    "clone()": {
        topic: function () {
            return Array.prototype.clone.bindToCall();
        },
        "should have length of 1": function (topic) {
            assert.equal(Array.prototype.clone.length, 1);
        },
        "should return copy of the array object": function (topic) {
            var array = ['foo', 'bar', 'baz', {
                nested: {
                    "arg": true
                }
            }],
            clone = topic(array);
            assert.notEqual(clone, array);
            assert.equal(clone.length, array.length);
            assert.deepEqual(clone, array);
            assert.equal(clone[3], array[3]);
        },
        "should return deep copy of the array object if second argument is true": function (topic) {
            var array = ['foo', 'bar', 'baz', {
                    nested: {
                        "arg": true
                    }
                }],
                clone = topic(array, true);
            assert.notEqual(clone, array);
            assert.equal(clone.length, array.length);
            assert.deepEqual(clone, array);
            assert.notEqual(clone[3].nested, array[3].nested);
            assert.deepEqual(clone[3].nested, array[3].nested);
        }
    },
    "size()": {
        topic: function () {
            return Array.prototype.size.bindToCall();
        },
        "should have length of 0": function (topic) {
            assert.equal(Array.prototype.size.length, 0);
        },
        "should return size of the array": function (topic) {
            assert.equal(topic([]), 0);
            assert.equal(topic(["foo"]), 1);
            assert.equal(topic(["foo", "bar", "baz"]), 3);
        }
    },
    "Object.hash(array)": {
        topic: function () {
            return Object.hash;
        },
        "should return hash of the array": function (topic) {
            assert.equal(topic([]), '');
            assert.equal(topic(["foo"]), "0foo");
            assert.equal(topic(["foo", "bar", "baz"]), "0foo1bar2baz");
        }
    },
    "padLeft()": {
        topic: function () {
            return Array.prototype.padLeft.bindToCall();
        },
        "should have length of 2": function (topic) {
            assert.equal(Array.prototype.padLeft.length, 2);
        },
        "should return same array when padding length is 0": function (topic) {
            assert.deepEqual(topic([], 0), []);
            assert.deepEqual(topic(['foo'], 0), ['foo']);
        },
        "should return same array when length is inferior to the this length": function (topic) {
            assert.deepEqual(topic(["foo", "bar"], 1), ["foo", "bar"]);
            assert.deepEqual(topic(["foo", "bar"], 2), ["foo", "bar"]);
        },
        "should return left padded string when length is superior": function (topic) {
            assert.deepEqual(topic(["foo", "bar", "baz"], 4, "1"), ["1", "foo", "bar", "baz"]);
            assert.deepEqual(topic(["foo", "bar", "baz"], 5, "1"), ["1", "1", "foo", "bar", "baz"]);
        }
    },
    "padRight()": {
        topic: function () {
            return Array.prototype.padRight.bindToCall();
        },
        "should have length of 2": function (topic) {
            assert.equal(Array.prototype.padRight.length, 2);
        },
        "should return same array when padding length is 0": function (topic) {
            assert.deepEqual(topic([], 0), []);
            assert.deepEqual(topic(['foo'], 0), ['foo']);
        },
        "should return same array when length is inferior to the this length": function (topic) {
            assert.deepEqual(topic(["foo", "bar"], 1), ["foo", "bar"]);
            assert.deepEqual(topic(["foo", "bar"], 2), ["foo", "bar"]);
        },
        "should return left padded string when length is superior": function (topic) {
            assert.deepEqual(topic(["foo", "bar", "baz"], 4, "1"), ["foo", "bar", "baz", "1"]);
            assert.deepEqual(topic(["foo", "bar", "baz"], 5, "1"), ["foo", "bar", "baz", "1", "1"]);
        }
    },
    "pad()": {
        topic: function () {
            return Array.prototype.pad.bindToCall();
        },
        "should have length of 3": function (topic) {
            assert.equal(Array.prototype.pad.length, 3);
        },
        "should return pad left when mode is 'l', 'left'": function (topic) {
            var mode = 'left';
            assert.deepEqual(topic(["foo", "bar", "baz"], 4, "1", mode), ["1", "foo", "bar", "baz"]);
            assert.deepEqual(topic(["foo", "bar", "baz"], 5, "1", mode), ["1", "1", "foo", "bar", "baz"]);
        },
        "should return pad right when mode is 'r', 'right'": function (topic) {
            var mode = 'right';
            assert.deepEqual(topic(["foo", "bar", "baz"], 4, "1", mode), ["foo", "bar", "baz", "1"]);
            assert.deepEqual(topic(["foo", "bar", "baz"], 5, "1", mode), ["foo", "bar", "baz", "1", "1"]);
        },
        "should return pad left and right when mode is 'b', 'both'": function (topic) {
            var mode = 'both';
            assert.deepEqual(topic(["foo", "bar", "baz"], 4, "1", mode), ["1", "foo", "bar", "baz"]);
            assert.deepEqual(topic(["foo", "bar", "baz"], 5, "1", mode), ["1", "foo", "bar", "baz", "1"]);
            assert.deepEqual(topic(["foo", "bar", "baz"], 6, "1", mode), ["1", "1", "foo", "bar", "baz", "1"]);
            assert.deepEqual(topic(["foo", "bar", "baz"], 7, "1", mode), ["1", "1", "foo", "bar", "baz", "1", "1"]);
        }
    },
    "clear()": {
        topic: function () {
            return Array.prototype.clear.bindToCall();
        },
        "should have length of 0": function (topic) {
            assert.equal(Array.prototype.clear.length, 0);
        },
        "should return this": function (topic) {
            var array = [];
            assert.equal(topic(array), array);
        },
        "should empty an already empty array": function (topic) {
            var array = [];
            topic(array);
            assert.deepEqual(topic(array), array);
        },
        "should empty any array": function (topic) {
            var array = ['foo', 'bar'];
            topic(array);
            assert.deepEqual(topic(array), []);
            assert.equal(array.length, 0);

            array.push('baz');
            assert.deepEqual(array, ['baz']);
            assert.equal(array.length, 1);

            topic(array);
            assert.deepEqual(topic(array), []);
            assert.equal(array.length, 0);
        }
    },
    "flatten()": {
        topic: function () {
            return Array.prototype.flatten.bindToCall();
        },
        "should have length of 0": function (topic) {
            assert.equal(Array.prototype.flatten.length, 0);
        },
        "should flatten array of []": function (topic) {
            assert.deepEqual(topic([]), []);
            assert.deepEqual(topic([[], [], []]), []);
        },
        "should flatten array of nested arrays": function (topic) {
            assert.deepEqual(topic(['foo', 'bar', ['baz', ['hello'], ['world']]]), ['foo', 'bar', 'baz', 'hello', 'world']);
        }
    },
    "unique()": {
        topic: function () {
            return Array.prototype.unique.bindToCall();
        },
        "should have length of 0": function (topic) {
            assert.equal(Array.prototype.unique.length, 0);
        },
        "should return [] for []": function (topic) {
            assert.deepEqual(topic([]), []);
        },
        "should return array with unique values": function (topic) {
            assert.deepEqual(topic(['foo', 'bar', 'baz']), ['foo', 'bar', 'baz']);
            assert.deepEqual(topic(['foo', 'bar', 'baz', 'baz']), ['foo', 'bar', 'baz']);
            assert.deepEqual(topic(['foo', 'bar', 'baz', 'baz', 'baz']), ['foo', 'bar', 'baz']);
            assert.deepEqual(topic(['foo', 'baz', 'bar', 'baz', ]), ['foo', 'baz', 'bar']);
            assert.deepEqual(topic(['foo', 'foo', 'bar', 'foo', 'baz', 'foo']), ['foo', 'bar', 'baz']);
        }
    },

    "get()": {
        topic: function () {
            return Array.prototype.get.bindToCall();
        },
        'should have length of 1': function (topic) {
            assert.equal(Array.prototype.get.length, 1);
        },
        'should return one value if key is not array': function (topic) {
            assert.isUndefined(topic([], 0));
            assert.isUndefined(topic(["foo", "bar"], 2));
            assert.equal(topic(["foo", "bar"], 0), "foo");
            assert.equal(topic(["foo", "bar"], 1), "bar");

            assert.equal(topic(["foo", "bar", "baz", "hehe"], -1), "hehe");
            assert.equal(topic(["foo", "bar", "baz", "hehe"], -2), "baz");
        },
        'should return an array of sliced values if key is not array': function (topic) {
            assert.deepEqual(topic([], []), []);
            assert.deepEqual(topic(["foo", "bar", "baz"], [0, 0]), ["foo"]);
            assert.deepEqual(topic(["foo", "bar", "baz"], [1, 1]), ["bar"]);
            assert.deepEqual(topic(["foo", "bar", "baz", "hehe"], [1, -2]), ["bar", "baz"]);
            assert.deepEqual(topic(["foo", "bar", "baz"], [1]), ["bar", "baz"]);
            assert.deepEqual(topic(["foo", "bar", "baz", "hehe"], [-2, -1]), ["baz", "hehe"]);
        }
    },

    "set()": {
        topic: function () {
            return Array.prototype.set.bindToCall();
        },
        'should have length of 2': function (topic) {
            assert.equal(Array.prototype.set.length, 2);
        },
        'should return set one value if key is not array': function (topic) {
            var array = [];
            topic(array, 0, 1);
            assert.deepEqual(array, [1]);

            topic(array, 0, 2);
            topic(array, 1, 3);
            assert.deepEqual(array, [2, 3]);

            topic(array, -1, "foo");
            assert.deepEqual(array, [2, "foo"]);
        },
        'should return an array of sliced values if key is not array': function (topic) {
            var array = [1, 2, 3];
            topic(array, [0, 1], []);
            assert.deepEqual(array, [3]);

            topic(array, [0, 1], [1, 2, 3]);
            assert.deepEqual(array, [1, 2, 3]);
        }
    }

});