/*jslint nodejs:true, indent:4 */
var vows = require('vows'),
    assert = require('assert'),
    env = require('./_env'),
    __filenameTested = env.lib('stj'),
    slice = function (array, start, size) {
        return Array.prototype.slice.call(array, start, size);
    },
    nativeMethods = {
        toString: Date.prototype.toString
    },
    stj = require(__filenameTested);

/*******************************************************************************
 * JSLint validation
 ******************************************************************************/
exports.JSLintTest = env.JSLintTest([/*__filenameTested, */__filename]);

/*******************************************************************************
 * DateTest class
 ******************************************************************************/
exports.DateTest = vows.describe('Date functions').addBatch({
    "[native methods]": {
        topic: function () {
            return Date.prototype;
        },
        'should not change native implementation': function (topic) {
            for (var methodName in nativeMethods) {
                if (nativeMethods.hasOwnProperty(methodName)) {
                    assert.equal(nativeMethods[methodName], topic[methodName]);
                    assert.equal(topic[methodName].toString(), "function " + methodName + "() { [native code] }");
                }
            }
        }
    },
    "isEqual": {
        topic: function () {
            return function () {
                return Date.prototype.isEqual.apply(arguments[0], slice(arguments, 1));
            };
        },
        "should have length of 1": function (topic) {
            assert.equal(Date.prototype.isEqual.length, 1);
        },
        "should return true for same object": function (topic) {
            var date = new Date(1984, 10, 17),
                clone = new Date(1984, 10, 17);
            assert.equal(topic(date, date), true);
            assert.equal(topic(date, clone), true);
        },
        "should return false for different values": function (topic) {
            var date = new Date(1984, 10, 17);
            assert.equal(topic(date, new Date(1984, 10, 18)), false);
            assert.equal(topic(date, "123"), false);
            assert.equal(topic(date, false), false);
        }
    },
    "clone()": {
        topic: function () {
            return function () {
                return Date.prototype.clone.apply(arguments[0], slice(arguments, 1));
            };
        },
        "should have length of 1": function (topic) {
            assert.equal(Date.prototype.clone.length, 1);
        },
        "should return copy of the date object": function (topic) {
            var date = new Date(1984, 10, 17);
            assert.equal(topic(date) === date, false);
            assert.deepEqual(topic(date), date);
        }
    },
    "format()": {
        topic: function () {
            return function () {
                return Date.prototype.format.apply(arguments[0], slice(arguments, 1));
            };
        },
        "should have length of 1": function (topic) {
            assert.equal(Date.prototype.format.length, 1);
        },
        "should return a formatted day": function (topic) {
            var date = new Date(1984, 10, 1);
            assert.equal(topic(date, ' d '), ' 1 ');
            assert.equal(topic(date, ' dd '), ' 01 ');
            assert.equal(topic(date, ' ddd '), ' Thu ');
            assert.equal(topic(date, ' dddd '), ' Thursday ');
        },
        "should return a formatted month": function (topic) {
            var date = new Date(1984, 0, 1);
            assert.equal(topic(date, ' m '), ' 1 ');
            assert.equal(topic(date, ' mm '), ' 01 ');
            assert.equal(topic(date, ' mmm '), ' Jan ');
            assert.equal(topic(date, ' mmmm '), ' January ');
        },
        "should return a formatted year": function (topic) {
            var date = new Date(1984, 0, 1);
            assert.equal(topic(date, ' yy '), ' 84 ');
            assert.equal(topic(date, ' yyyy '), ' 1984 ');
        },
        "should return formatted hours": function (topic) {
            var date = new Date(1984, 0, 1, 2, 3, 4);
            assert.equal(topic(date, ' H '), ' 2 ');
            assert.equal(topic(date, ' HH '), ' 02 ');

            date = new Date(1984, 0, 1, 22, 3, 4);
            assert.equal(topic(date, ' h '), ' 10 ');
            assert.equal(topic(date, ' hh '), ' 10 ');
        },
        "should return formatted minutes": function (topic) {
            var date = new Date(1984, 0, 1, 2, 3, 4);
            assert.equal(topic(date, ' M '), ' 3 ');
            assert.equal(topic(date, ' MM '), ' 03 ');
        },
        "should return formatted seconds": function (topic) {
            var date = new Date(1984, 0, 1, 2, 3, 4);
            assert.equal(topic(date, ' S '), ' 4 ');
            assert.equal(topic(date, ' SS '), ' 04 ');
        }
    }
});