/*jslint nodejs:true, indent:4 */
/*global module_, class_, include_, use_, extend_, static_, property_ */
var vows = require('vows'),
    assert = require('assert'),
    env = require('./_env'),
    __filenameTested = env.toFilenameTested(__filename),
    event = require(__filenameTested),
    EventDispatcher = use_('event.Dispatcher'),
    toArray = Object.toArray,
    ValueError = use_('error').ValueError;

/*******************************************************************************
 * JSLint validation
 ******************************************************************************/
exports.JSLintTest = env.JSLintTest([__filenameTested, __filename]);

/*******************************************************************************
 * EventDipatcher class
 ******************************************************************************/
exports.EventDipatcherTest = vows.describe('event.EventDispatcher class').addBatch({
    "new event.Dispatcher()":  {
        topic: function () {
            return new EventDispatcher();
        },
        'should return instance of dispatcher': function (topic) {
            assert.ok(Object.isInstanceOf(topic, EventDispatcher));
        }
    },

    "getListeners()":  {
        topic: function () {
            return new EventDispatcher();
        },
        "should have length of 1": function (topic) {
            assert.equal(EventDispatcher.prototype.getListeners.length, 1);
        },
        'should return undefined for non existing event': function (topic) {
            assert.isUndefined(topic.getListeners('nonexisting'));
        },
        'should return instance of array for existing event': function (topic) {
            assert.ok(topic.getListeners('addListener'), []);
        }
    },

    "addListener()":  {
        topic: function () {
            return new EventDispatcher();
        },
        "should have length of 2": function (topic) {
            assert.equal(EventDispatcher.prototype.addListener.length, 2);
        },
        'should throw error if second argument is not a function': function (topic) {
            assert.throws(function () {
                topic.addListener('addListener', "not a function");
            }, TypeError);
        },
        'should add a new handler for ': function (topic) {
            assert.throws(function () {
                topic.addListener('addListener', "not a function");
            }, TypeError);
        },
        'should return instance of dispatcher': function (topic) {
            assert.equal(topic.getListeners('addListener').length, 0);
            assert.equal(topic.addListener('addListener', function () {}), topic);
            assert.equal(topic.getListeners('addListener').length, 1);

            assert.equal(topic.addListener('addListener', function () {}), topic);
            assert.equal(topic.getListeners('addListener').length, 2);
        },
        'should throw error if strict mode and event is not defined': function (topic) {
            var dispatcherStrict = new EventDispatcher({strict: true});
            assert.doesNotThrow(function () {
                dispatcherStrict.addListener('addListener', function () {});
            });

            assert.throws(function () {
                dispatcherStrict.addListener('non-existing', function () {});
            }, ValueError);
            dispatcherStrict.defineEvent('non-existing');
            assert.doesNotThrow(function () {
                dispatcherStrict.addListener('non-existing', function () {});
            });

        }
    },

    "removeListener()":  {
        topic: function () {
            return new EventDispatcher();
        },
        "should have length of 2": function (topic) {
            assert.equal(EventDispatcher.prototype.removeListener.length, 2);
        },
        'should throw error if second argument is not a function': function (topic) {
            assert.throws(function () {
                topic.removeListener('addListener', "not a function");
            }, TypeError);
        },
        'should return undefined if not found': function (topic) {
            assert.doesNotThrow(function () {
                topic.removeListener('addListener', function () {});
            });

            assert.isUndefined(topic.removeListener('addListener', function () {}));
        },
        'should remove a previously added handler': function (topic) {
            var eventName = 'addListener',
                handler = function () {
                    return 'foo';
                },
                result;

            topic.addListener(eventName, function () {});
            topic.addListener(eventName, handler);
            topic.addListener(eventName, function () {});
            result = assert.equal(topic.getListeners(eventName).indexOf(handler), 1);
            topic.removeListener('addListener', handler);
            assert.equal(topic.getListeners(eventName).length, 2);
            assert.equal(topic.getListeners(eventName).indexOf(handler), -1);
        },
        'should throw error if strict mode and event is not defined': function (topic) {
            var dispatcherStrict = new EventDispatcher({strict: true});
            assert.doesNotThrow(function () {
                dispatcherStrict.removeListener('addListener', function () {});
            });

            assert.throws(function () {
                dispatcherStrict.removeListener('non-existing', function () {});
            }, ValueError);
            dispatcherStrict.defineEvent('non-existing');
            assert.doesNotThrow(function () {
                dispatcherStrict.removeListener('non-existing', function () {});
            });

        }
    },

    "emit()":  {
        topic: function () {
            return new EventDispatcher();
        },
        "should have length of 1": function (topic) {
            assert.equal(EventDispatcher.prototype.emit.length, 1);
        },
        'should emit event to one or many listeners ': function (topic) {
            var args1, args2, argsNotUsed;

            topic.addListener('event-1', function (arg1, arg2, arg3) {
                args1 = arguments;
            });
            topic.addListener('event-1', function (arg1, arg2) {
                args2 = arguments;
            });
            topic.addListener('event-2', function (arg1, arg2) {
                argsNotUsed = 'not used';
            });

            topic.emit('event-1', 'foo', 'bar');
            assert.deepEqual(toArray(args1), ['foo', 'bar']);
            assert.deepEqual(toArray(args2), ['foo', 'bar']);
            assert.isUndefined(argsNotUsed);

            topic.emit('event-1', 'foo', 'bar', 'baz');
            assert.deepEqual(toArray(args1), ['foo', 'bar', 'baz']);
            assert.deepEqual(toArray(args2), ['foo', 'bar', 'baz']);
            assert.isUndefined(argsNotUsed);
        },
        'should throw error if strict mode and event is not defined': function (topic) {
            var dispatcherStrict = new EventDispatcher({strict: true});

            assert.throws(function () {
                dispatcherStrict.emit('non-existing');
            }, ValueError);
            dispatcherStrict.defineEvent('non-existing');
            assert.doesNotThrow(function () {
                dispatcherStrict.emit('non-existing');
            });

        }
    }
});