/*jslint nodejs:true, indent:4 */
var vows = require('vows'),
    assert = require('assert'),
    env = require('./_env'),
    __filenameTested = env.lib('stj'),
    nativeMethods = {
        toString: Function.prototype.toString
    },
    stj = require(__filenameTested),
    func = stj.use_('Function');

/*******************************************************************************
 * JSLint validation
 ******************************************************************************/
exports.JSLintTest = env.JSLintTest([/*__filenameTested, */__filename]);

/*******************************************************************************
 * FunctionTest class
 ******************************************************************************/
exports.FunctionTest = vows.describe('Function functions').addBatch({
    "[native methods]": {
        topic: function () {
            return Function.prototype;
        },
        'should not change native implementation': function (topic) {
            for (var methodName in nativeMethods) {
                if (nativeMethods.hasOwnProperty(methodName)) {
                    assert.equal(nativeMethods[methodName], topic[methodName]);
                    assert.equal(topic[methodName].toString(), "function " + methodName + "() { [native code] }");
                }
            }
        }
    },
    "applySuper()": {
        topic: function () {
            return func.prototype.applySuper.bindToCall();
        },
        "should call parent function": function (topic) {
            var Root = function () {},
                Parent = function () {},
                Child = function (prop) {
                    this.prop = prop;
                },
                result;

            Root.prototype.foo = function (arg1, arg2) {
                return 'root_' + this.prop + '_' + arg1 + '_' + arg2;
            };

            Parent.prototype = new Root();
            Parent.prototype.foo = function (arg1, arg2) {
                return 'parent_' + this.prop + '_' + arg1 + '_' + arg2;
            };

            Child.prototype = new Parent();
            Child.prototype.foo = function (arg1, arg2) {
                return 'child_' + this.prop + '_' + arg1 + '_' + arg2;
            };

            result = new Child('myprop');

            //Check return this
            assert.equal(result.foo('myarg1', 'myarg2'), 'child_myprop_myarg1_myarg2');
            assert.equal(topic(Child, 'foo', result, ['myarg1', 'myarg2']), 'parent_myprop_myarg1_myarg2');
            assert.equal(topic(Parent, 'foo', result, ['myarg1', 'myarg2']), 'root_myprop_myarg1_myarg2');
            assert.equal(topic(Root, 'foo', result, ['myarg1', 'myarg2']), undefined);
        }
    },
    "callSuper()": {
        topic: function () {
            return func.prototype.callSuper.bindToCall();
        },
        "should call parent function": function (topic) {
            var Root = function () {},
                Parent = function () {},
                Child = function (prop) {
                    this.prop = prop;
                },
                result;

            Root.prototype.foo = function (arg1, arg2) {
                return 'root_' + this.prop + '_' + arg1 + '_' + arg2;
            };

            Parent.prototype = new Root();
            Parent.prototype.foo = function (arg1, arg2) {
                return 'parent_' + this.prop + '_' + arg1 + '_' + arg2;
            };

            Child.prototype = new Parent();
            Child.prototype.foo = function (arg1, arg2) {
                return 'child_' + this.prop + '_' + arg1 + '_' + arg2;
            };

            result = new Child('myprop');

            //Check return this
            assert.equal(result.foo('myarg1', 'myarg2'), 'child_myprop_myarg1_myarg2');
            assert.equal(topic(Child, 'foo', result, 'myarg1', 'myarg2'), 'parent_myprop_myarg1_myarg2');
            assert.equal(topic(Parent, 'foo', result, 'myarg1', 'myarg2'), 'root_myprop_myarg1_myarg2');
            assert.equal(topic(Root, 'foo', result, 'myarg1', 'myarg2'), undefined);
        }
    },
    "generate()": {
        topic: function () {
            return func.prototype.generate.bindToCall();
        },
        "should return unchanged this if empty arguments are passed": function (topic) {
            var func = function () {};
            //Check return this
            assert.equal(topic(func), func);
        },
        "should return a new function with default arguments passed": function (topic) {
            var split = function (delimiter, string) {
                    return string.split(delimiter);
                },
                splitOnColons = topic(split, [":"]);
            assert.notEqual(split, splitOnColons);
            assert.deepEqual(split(":", "0:1:2:3:4:5"), [0, 1, 2, 3, 4, 5]);//Safety check
            assert.deepEqual(splitOnColons("0:1:2:3:4:5"), [0, 1, 2, 3, 4, 5]);
        }
    },
    "getName": {
        topic: function () {
            return func.prototype.getName.bindToCall();
        },
        "should return undefined for anonymous functions": function (topic) {
            assert.equal(topic(function () {}), '');
        },
        "should return a new function with default arguments passed": function (topic) {
            assert.equal(topic(function foo() {}), "foo");
            assert.equal(topic(function fooBar() {}), "fooBar");
            assert.equal(topic(function foo_bar() {}), "foo_bar");
        }
    }
});