/*jslint nodejs:true, indent:4 */
var vows = require('vows'),
    assert = require('assert'),
    env = require('./_env'),
    __filenameTested = env.lib('stj'),
    nativeMethods = {
        create: Object.create
    },
    stj = require(__filenameTested),
    object = stj.use_('Object');

/*******************************************************************************
 * JSLint validation
 ******************************************************************************/
exports.JSLintTest = env.JSLintTest([/*__filenameTested, */__filename]);

/*******************************************************************************
 * ObjectTest class
 ******************************************************************************/
exports.ObjectTest = vows.describe('Object functions').addBatch({
    "[native methods]": {
        topic: function () {
            return Object.prototype;
        },
        'should not add new properties into Object.prototype': function (topic) {
            for (var methodName in topic) {
                if (topic.hasOwnProperty(methodName)) {
                    assert.fail("Object.prototype should not be changed for easier compatibility with libraries");
                }
            }
        },
        'should not change native implementation': function (topic) {
            for (var methodName in nativeMethods) {
                if (nativeMethods.hasOwnProperty(methodName)) {
                    assert.equal(nativeMethods[methodName], Object[methodName]);
                    assert.equal(Object[methodName].toString(), "function " + methodName + "() { [native code] }");
                }
            }
        }
    },
    "isEmpty()": {
        topic: function () {
            return object.isEmpty;
        },
        'should have length of 1': function (topic) {
            assert.equal(topic.length, 1);
        },
        'should return true for [], {}, "", null': function (topic) {
            assert.equal(topic(), true);
            assert.equal(topic(null), true);
            assert.equal(topic([]), true);
            assert.equal(topic({}), true);
            assert.equal(topic(""), true);
        },
        'should return false for any other value': function (topic) {
            assert.equal(topic(['foo']), false);
            assert.equal(topic({foo: true}), false);
            assert.equal(topic("123"), false);
        }
    },
    "isArray()": {
        topic: function () {
            return object.isArray;
        },
        'should have length of 1': function (topic) {
            assert.equal(topic.length, 1);
        },
        'should return false for object, strings, number, boolean': function (topic) {
            assert.equal(topic({}), false);
            assert.equal(topic(''), false);
            assert.equal(topic(0), false);
            assert.equal(topic(true), false);
            assert.equal(topic(null), false);
            assert.equal(topic(undefined), false);
            assert.equal(topic(function () {}), false);
        },
        'should return true for []': function (topic) {
            assert.equal(topic([]), true);
        },
        'should return true for an Array subclass': function (topic) {
            var ArraySub = function () {};
            ArraySub.prototype = [];

            assert.equal(topic(new ArraySub()), true);
        }
    },
    "isString()": {
        topic: function () {
            return object.isString;
        },
        'should have length of 1': function (topic) {
            assert.equal(topic.length, 1);
        },
        'should return false for object, arrays, number, boolean': function (topic) {
            assert.equal(topic({}), false);
            assert.equal(topic([]), false);
            assert.equal(topic(1), false);
            assert.equal(topic(true), false);
            assert.equal(topic(null), false);
            assert.equal(topic(undefined), false);
            assert.equal(topic(function () {}), false);
        },
        'should return true for "my string"': function (topic) {
            assert.equal(topic(''), true);
            assert.equal(topic('my string'), true);
            assert.equal(topic(String('my string')), true);
            //assert.equal(topic(new String('my string')), true);
        }
    },
    "isFunction()": {
        topic: function () {
            return object.isFunction;
        },
        'should have length of 1': function (topic) {
            assert.equal(topic.length, 1);
        },
        'should return false for arrays, strings, number, boolean': function (topic) {
            assert.equal(topic(''), false);
            assert.equal(topic([]), false);
            assert.equal(topic(0), false);
            assert.equal(topic(true), false);
            assert.equal(topic(null), false);
            assert.equal(topic(undefined), false);
        },
        'should return true for function () {}': function (topic) {
            assert.equal(topic(function () {}), true);
        }
    },
    "isPrimitive()": {
        topic: function () {
            return object.isPrimitive;
        },
        'should have length of 1': function (topic) {
            assert.equal(topic.length, 1);
        },
        'should return false for object, arrays, functions': function (topic) {
            assert.equal(topic({}), false);
            assert.equal(topic([]), false);
            assert.equal(topic(function () {}), false);
        },
        'should return true for strings, number, booleans, null, undefined': function (topic) {
            assert.equal(topic(true), true);
            assert.equal(topic(''), true);
            assert.equal(topic(0), true);
            assert.equal(topic(null), true);
            assert.equal(topic(undefined), true);
        }
    },
    "isObject()": {
        topic: function () {
            return object.isObject;
        },
        'should have length of 1': function (topic) {
            assert.equal(topic.length, 1);
        },
        'should return false for arrays, strings, number, boolean, null': function (topic) {
            assert.equal(topic(''), false);
            assert.equal(topic(0), false);
            assert.equal(topic(true), false);
            assert.equal(topic(null), false);
            assert.equal(topic(undefined), false);
        },
        'should return true for {} or [] or null or function() {}': function (topic) {
            assert.equal(topic({}), true);
            assert.equal(topic([]), true);
            assert.equal(topic(function () {}), true);
        },
        'should return true for a Object subclass': function (topic) {
            var ObjSub = function () {};
            ObjSub.prototype = {};

            assert.equal(topic(new ObjSub()), true);
        }
    },
    "isInstanceOf()": {
        topic: function () {
            return object.isInstanceOf;
        },
        'should have length of 2': function (topic) {
            assert.equal(topic.length, 2);
        },
        'should return work with Object, Number, Boolean, etc': function (topic) {
            assert.equal(topic({}, Object), true);
            assert.equal(topic("", Object), false);

            assert.equal(topic([], Array), true);
            assert.equal(topic({}, Array), false);

            assert.equal(topic(0, Number), true);
            assert.equal(topic({}, Number), false);

            assert.equal(topic(true, Boolean), true);
            assert.equal(topic({}, Boolean), false);

            assert.equal(topic(new Date(), Date), true);
            assert.equal(topic({}, Date), false);

            assert.equal(topic("string", String), true);
            assert.equal(topic({}, String), false);
        },
        'should call __isInstanceOf__ callback': function (topic) {
            var TestClass = function () {},
                instance = new TestClass();

            assert.equal(topic(instance, TestClass), true);
            TestClass.__isInstanceOf__ = function (obj) {
                return !!obj.duckProperty;
            };
            assert.equal(topic(instance, TestClass), false);

            instance.duckProperty = true;
            assert.equal(topic(instance, TestClass), true);
        }
    },
    "isEqual()": {
        topic: function () {
            return object.isEqual;
        },
        'should have length of 2': function (topic) {
            assert.equal(topic.length, 2);
        },
        'should compare null and undefined': function (topic) {
            assert.equal(topic(null, null), true);
            assert.equal(topic(null, undefined), false);
            assert.equal(topic(undefined, null), false);
            assert.equal(topic(null, 0), false);
            assert.equal(topic(0, null), false);
        },
        'should compare strings': function (topic) {
            assert.equal(topic('', ''), true);
            assert.equal(topic('foo', 'foo'), true);
            assert.equal(topic('', 'fdsfd'), false);
            assert.equal(topic('1', 1), false);
            assert.equal(topic('1', null), false);
            assert.equal(topic('1', ['1']), false);
        },
        'should compare numbers': function (topic) {
            assert.equal(topic(0, 0), true);
            assert.equal(topic(0, 10), false);
            assert.equal(topic(0, '1'), false);
            assert.equal(topic(0, {}), false);
            assert.equal(topic(0, []), false);
        },
        'should compare booleans': function (topic) {
            assert.equal(topic(true, true), true);
            assert.equal(topic(true, false), false);
            assert.equal(topic(true, 'foo'), false);
            assert.equal(topic(true, {}), false);
            assert.equal(topic(true, 0), false);
        },
        'should compare objects': function (topic) {
            assert.equal(topic({}, {}), true);
            assert.equal(topic({
                foo: 'bar'
            }, {
                foo: 'bar'
            }), true);
            assert.equal(topic({
                foo: 'bar'
            }, {}), false);
            assert.equal(topic({}, 0), false);
            assert.equal(topic({}, '123'), false);
        },
        'should compare arrays': function (topic) {
            assert.equal(topic([], []), true);
            assert.equal(topic(['foo'], ['foo']), true);
            assert.equal(topic(['foo'], []), false);
            assert.equal(topic([], 0), false);
            assert.equal(topic([], ''), false);
            assert.equal(topic([], {}), false);
        },
        'should compare functions': function (topic) {
            var original = function (e) {
                    return 'foo';
                },
                clone = function (e) {
                    return 'foo';
                };

            assert.equal(topic(original, original), true);
            //assert.equal(topic(original, clone), true); TODO compare clones?
            assert.equal(topic(original, function () {}), false);
            assert.equal(topic(original, 0), false);
            assert.equal(topic(original, '123'), false);
            assert.equal(topic(original, {}), false);
        }
    },
    "update(object, extension, _conflictHandler)" : {
        topic : function (item) {
            return object.update;
        },
        'should have length of 2': function (topic) {
            assert.equal(topic.length, 2);
        },
        'should extends object and overwrite values when `overwrite` is true' : function (topic) {
            var obj = {
                foo: 1
            };

            topic(obj);
            assert.equal(obj.foo, 1);

            topic(obj, {
                bar : 2,
                baz : 3,
                array1: ['foo']
            }, true);
            assert.equal(obj.foo, 1);
            assert.equal(obj.bar, 2);
            assert.equal(obj.baz, 3);
            assert.deepEqual(obj.array1, ['foo']);

            topic(obj, {
                bar : 0,
                baz : 0,
                array1: ['foo', 'bar']
            });
            assert.equal(obj.foo, 1);
            assert.equal(obj.bar, 0);
            assert.equal(obj.baz, 0);
            assert.deepEqual(obj.array1, ['foo', 'bar']);
        },
        'should extends object and keep values when `overwrite` is false' : function (topic) {
            var obj = {
                foo: 1,
                bar : 2,
                baz : 3,
                array1: ['foo']
            };

            topic(obj, {
                foo : 0,
                baz : 0,
                helloworld: "hello",
                array1: ['foo', 'bar'],
                arrayNew: ['baz']
            }, false);
            assert.equal(obj.foo, 1);
            assert.equal(obj.bar, 2);
            assert.equal(obj.baz, 3);
            assert.equal(obj.helloworld, "hello");
            assert.deepEqual(obj.array1, ['foo']);
            assert.deepEqual(obj.arrayNew, ['baz']);
        },
        'should resolve conflict using `overwrite` if function' : function (topic) {
            var obj = {
                foo: 1,
                bar : 2,
                baz : 3
            };

            topic(obj, {
                foo : 0,
                baz : 0,
                helloworld: "hello"
            }, function (myValue, extValue, property) {
                if (property === "baz") {
                    return extValue;
                } else {
                    return myValue;
                }
            });
            assert.equal(obj.foo, 1);
            assert.equal(obj.bar, 2);
            assert.equal(obj.baz, 0);
            assert.equal(obj.helloworld, "hello");
        }
    },
    "hash()": {
        topic: function () {
            return object.hash;
        },
        'should have length of 1': function (topic) {
            assert.equal(topic.length, 1);
        },
        'should return hash string for native types': function (topic) {
            assert.equal(topic("toto"), "toto");
            assert.equal(topic(123), "123");
            assert.equal(topic({toto: 'foo', tata: 'bar'}), "bartatafoototo");
            assert.equal(topic({
                __hash__: function () {
                    return 'customhash';
                }
            }), "customhash");
        }
    },
    "size()": {
        topic: function () {
            return object.size;
        },
        'should have length of 1': function (topic) {
            assert.equal(topic.length, 1);
        },
        'should return hash string for native types': function (topic) {
            assert.equal(topic([]), 0);
            assert.equal(topic([1, 2]), 2);
            assert.equal(topic(""), 0);
            assert.equal(topic("abc"), 3);
            assert.equal(topic({
                __size__: function () {
                    return 10;
                }
            }), 10);
        }
    },
    "clone()": {
        topic: function () {
            return object.clone;
        },
        'should have length of 2': function (topic) {
            assert.equal(topic.length, 2);
        },
        'should clone only first level': function (topic) {
            assert.equal(topic("string"), "string");
            assert.equal(topic(1), 1);

            var cloned = {
                    "test": true,
                    "1": 2,
                    "object": {
                        'deep': {
                            'verydeep': true
                        }
                    }
                },
                clone = topic(cloned);
            assert.notEqual(cloned, clone);
            assert.deepEqual(cloned, clone);
            assert.equal(cloned.object.deep, clone.object.deep);
        },
        'should clone deeply if true is passed as second argument': function (topic) {
            var cloned = {
                    "test": true,
                    "1": 2,
                    "object": {
                        'deep': {
                            'verydeep': true
                        }
                    }
                },
                clone = topic(cloned, true);
            assert.notEqual(cloned, clone);
            assert.deepEqual(cloned, clone);
            assert.notEqual(cloned.object.deep, clone.object.deep);
            assert.deepEqual(cloned.object.deep, clone.object.deep);
        }
    },
    "has()": {
        topic: function () {
            return object.has;
        },
        'should have length of 2': function (topic) {
            assert.equal(topic.length, 2);
        },
        'should throw error for non objects': function (topic) {
            assert.throws(function () {
                topic(null, 'p');
            }, TypeError);
            assert.throws(function () {
                topic(true, 'p');
            }, TypeError);
            assert.throws(function () {
                topic("string", 'p');
            }, TypeError);
            assert.throws(function () {
                topic(10, 'p');
            }, TypeError);
        },
        'should return result of hasOwnProperty': function (topic) {
            assert.equal(topic({}, 'p'), false);
            assert.equal(topic({p: undefined}, 'p'), true);
        }
    },
    "forEach()": {
        topic: function () {
            return object.forEach;
        },
        'should have length of 2': function (topic) {
            assert.equal(topic.length, 2);
        },
        'should not throw error for non objects': function (topic) {
            assert.doesNotThrow(function () {
                topic(undefined, function () {});
            }, TypeError);
            assert.doesNotThrow(function () {
                topic(null, function () {});
            }, TypeError);
            assert.doesNotThrow(function () {
                topic(true, function () {});
            }, TypeError);
            assert.doesNotThrow(function () {
                topic(10, function () {});
            }, TypeError);
        },
        'should throw error if second parameter is not function': function (topic) {
            assert.throws(function () {
                topic(null, 'nonfunction');
            }, TypeError);
        }
    },
    "map()": {
        topic: function () {
            return object.map;
        },
        'should have length of 2': function (topic) {
            assert.equal(topic.length, 2);
        },
        'should not throw error for non objects': function (topic) {
            assert.doesNotThrow(function () {
                topic(undefined, function () {});
            }, TypeError);
            assert.doesNotThrow(function () {
                topic(null, function () {});
            }, TypeError);
            assert.doesNotThrow(function () {
                topic(true, function () {});
            }, TypeError);
            assert.doesNotThrow(function () {
                topic(10, function () {});
            }, TypeError);
        },
        'should throw error if second parameter is not function': function (topic) {
            assert.throws(function () {
                topic(null, 'nonfunction');
            }, TypeError);
        }
    },
    "reduce()": {
        topic: function () {
            return object.reduce;
        },
        'should have length of 2': function (topic) {
            assert.equal(topic.length, 2);
        },
        'should not throw error for non objects': function (topic) {
            assert.doesNotThrow(function () {
                topic(undefined, function () {}, 'first');
            }, TypeError);
            assert.doesNotThrow(function () {
                topic(null, function () {}, 'first');
            }, TypeError);
            assert.doesNotThrow(function () {
                topic(true, function () {}, 'first');
            }, TypeError);
            assert.doesNotThrow(function () {
                topic(10, function () {}, 'first');
            }, TypeError);
        },
        'should throw error if second parameter is not function': function (topic) {
            assert.throws(function () {
                topic(null, 'nonfunction');
            }, TypeError);
        }
    },
    "filter()": {
        topic: function () {
            return object.filter;
        },
        'should have length of 2': function (topic) {
            assert.equal(topic.length, 2);
        },
        'should not throw error for non objects': function (topic) {
            assert.doesNotThrow(function () {
                topic(undefined, function () {});
            }, TypeError);
            assert.doesNotThrow(function () {
                topic(null, function () {});
            }, TypeError);
            assert.doesNotThrow(function () {
                topic(true, function () {});
            }, TypeError);
            assert.doesNotThrow(function () {
                topic(10, function () {});
            }, TypeError);
        },
        'should throw error if second parameter is not function': function (topic) {
            assert.throws(function () {
                topic(null, 'nonfunction');
            }, TypeError);
        }
    },
    "representation()": {
        topic: function () {
            return object.representation;
        },
        'should have length of 1': function (topic) {
            assert.equal(topic.length, 1);
        },
        'should return representation of primitives': function (topic) {
            assert.equal(topic(undefined), "undefined");
            assert.equal(topic(null), "null");
            assert.equal(topic(true), "true");
            assert.equal(topic(false), "false");
            assert.equal(topic(10), "10");
            assert.equal(topic("string"), '"string"');
        },
        'should return representation of objects': function (topic) {
            var now = new Date();

            assert.equal(topic(function named_function(arg1, arg2, /*comment*/ arg3) {
                return 'lala';
            }), 'function named_function(arg1, arg2, /*comment*/ arg3) {...}');
            //assert.equal(topic({key1:"str", key2:1}), '{key1:"string", key2:1}');

            assert.equal(topic(["str", 1]), '["str", 1]');
            assert.equal(topic(now), 'Date("' + now + '")');
        }
    }
});