/*jslint nodejs:true, indent:4 */
var vows = require('vows'),
    assert = require('assert'),
    env = require('./_env'),
    __filenameTested = env.toFilenameTested(__filename),
    stj = require(__filenameTested),
    slice = function (array, start, size) {
        return Array.prototype.slice.call(array, start, size);
    },
    use_ = stj.use_,
    include_ = stj.include_,
    extend_ = stj.extend_,
    property_ = stj.property_,
    properties_ = stj.properties_;


/*******************************************************************************
 * JSLint validation
 ******************************************************************************/
exports.JSLintTest = env.JSLintTest([__filenameTested, __filename]);

/*******************************************************************************
 * StdTest class
 ******************************************************************************/
exports.StdTest = vows.describe('Std module').addBatch({
    "use_()": {
        topic: function () {
            return stj.use_;
        },
        "should return a module": function (topic) {
            assert.doesNotThrow(function () {
                topic('error');
            });
            assert.doesNotThrow(function () {
                topic('error.Error');
            });
            assert.equal(topic('error.Error'), Error);
        },
        "should throw error for a non existing module": function (topic) {
            assert.throws(function () {
                topic('nonexisting');
            });
            assert.throws(function () {
                topic('nonexisting', false);
            });
        },
        "should throw error if result is not a module": function (topic) {
            //Preparation
            property_(topic("error"), 'nonModule', "error_nonModule");
            assert.throws(function () {
                topic('error.nonModule');
            });


            stj.property_(topic("error.Error"), 'nonModule', "error_Error_nonModule");
            assert.throws(function () {
                topic('error.Error.nonModule');
            });
        },
        "should return undefined for a non existing module (with _noError=true)": function (topic) {
            assert.isUndefined(topic('nonexisting', true));
        }
    },
    "module_()": {
        topic: function () {
            return stj.module_;
        },
        "should throw error for duplicate module": function (topic) {
            assert.throws(function () {
                topic('error');
            });

            topic('error.existing');

            assert.throws(function () {
                topic('error.existing');
            });

            assert.throws(function () {
                property_(stj.use_('error'), "existing", "foo");
            });

            assert.throws(function () {
                stj.module_(stj.use_('error'), function () {
                    topic('existing', {});
                });

            });
        },
        "should throw error when opening non module": function (topic) {
            assert.throws(function () {
                topic({});
            });
            assert.throws(function () {
                topic(function () {});
            });
        },
        "should return a new module object": function (topic) {
            var foo = function () {},
                result = topic('', {
                    foo: foo
                });
            assert.equal(result.foo, foo);
            assert.deepEqual(result, {foo: foo});
            assert.isObject(result);

            assert.doesNotThrow(function () {
                topic('');
            });
        },
        "should return a named module when specified": function (topic) {
            var foo = function () {},
                result = topic('foo.bar.baz', {
                    foo: foo
                }),
                expected = stj.use_('foo.bar.baz');
            assert.equal(result.foo, foo);
            assert.deepEqual(result, {foo: foo});
            assert.equal(result, expected);
            assert.equal(stj.use_('foo.bar').baz, expected);

            assert.throws(function () {
                topic('foo.bar.baz');
            });
        }
    },
    "class_()": {
        topic: function () {
            return stj.class_;
        },
        "should throw error for duplicate class": function (topic) {
            var error = stj.use_('error');

            assert.throws(function () {
                topic('error.Error');
            });

            assert.throws(function () {
                property_(error, "Error", "foo");
            });

            assert.throws(function () {
                stj.module_(error, function () {
                    stj.class_('Error', null, function () {});
                });

            });
        },
        "should throw error when opening non module": function (topic) {
            assert.throws(function () {
                topic({});
            });
            assert.throws(function () {
                topic(function () {});
            });
        },
        "should create a new class": function (topic) {
            var Class = stj.class_("Test.NewClass", null, function () {
                properties_({
                    "foo": function () {}
                });
                property_("bar", function () {});
            });
            assert.isFunction(Class.prototype.foo);
            assert.isFunction(Class.prototype.bar);
            assert.equal(Class.name, "NewClass");
        },
        "should return a new constructor function": function (topic) {
            var Class = topic(''),
                obj;
            property_(Class.prototype, "initialize", function (arg1, arg2) {
                this.count = (this.count || 0) + 1;
                this.arg1 = arg1;
                this.arg2 = arg2;
            });
            assert.isFunction(Class);
            assert.doesNotThrow(function () {
                topic('');
            });

            //Test instance
            obj = new Class("hehe", "hoho");
            assert.ok(obj instanceof Class);
            assert.equal(obj.arg1, "hehe");
            assert.equal(obj.arg2, "hoho");
            assert.equal(obj.count, 1);

            //Test direct function call
            obj = Class("hihi", "huhu");
            assert.ok(obj instanceof Class);
            assert.equal(obj.arg1, "hihi");
            assert.equal(obj.arg2, "huhu");
            assert.equal(obj.count, 1);
        },
        "should return a named function": function (topic) {
            assert.equal(topic('foo.bar.named1').getName(), "named1");
            assert.equal(topic('foo.bar.named2').getName(), "named2");
        },
        "should return a named class when specified": function (topic) {
            var result = topic('foo.bar.mytype'),
                expected = stj.use_('foo.bar.mytype');

            assert.equal(result, expected);
            assert.equal(stj.use_('foo.bar').mytype, expected);
            assert.throws(function () {
                topic('foo.bar.mytype');
            });
        },
        "should handle inheritance for instance methods": function (topic) {
            var Parent = stj.class_('stj-test.Parent'),
                Child = stj.class_('stj-test.Child', Parent),
                instanceChild = new Child();

            stj.class_(Parent, function () {
                property_('foo', function () {
                    return 'parent_foo';
                });
                property_('bar', function () {
                    return 'parent_bar';
                });
            });


            stj.class_(Child, function (_child, _super) {
                property_('bar', function () {
                    return 'child_bar';
                });
                property_('baz', function () {
                    return 'child_baz';
                });
            });


            assert.equal(instanceChild.foo(), 'parent_foo');
            assert.equal(instanceChild.bar(), 'child_bar');
            assert.equal(instanceChild.baz(), 'child_baz');
        },

        "should call declaration block as (currentClass, metaModule)": function (topic) {
            var Parent = stj.class_('stj-test.Parent2'),
                Child = stj.class_('stj-test.Child2', Parent),
                instanceChild = new Child(),
                blockArgs;

            stj.class_(Child, function (_child, _meta) {
                blockArgs = arguments;
            });

            assert.equal(blockArgs[0], Child);
            assert.equal(blockArgs[1], stj.meta_(Child));
        }

    },
    "property_()": {
        topic: function () {
            return stj.property_;
        },
        "should define a new attribute inside module": function (topic) {
            var result = stj.module_(''),
                method1 = function () {};

            assert.isUndefined(result.foo);
            topic(result, "foo", method1);
            assert.equal(result.foo, method1);
        },
        "should overwrite already defined attribute": function (topic) {
            var result = stj.module_(''),
                method1 = function () {},
                method2 = function () {};

            assert.isUndefined(result.foo);
            topic(result, "foo", method1);
            topic(result, "foo", method2);
            assert.equal(result.foo, method2);
        },
        "should work as getter when no value is specified": function (topic) {
            var result = stj.module_(''),
                method1 = function () {};

            topic(result, "foo", method1);
            assert.equal(result.foo, method1);
            assert.equal(topic(result, 'foo'), method1);
        }
    },
    /*"static_()": {
        topic: function () {
            return stj.static_;
        },
        "should define a new attribute for class": function (topic) {
            var result = stj.class_(''),
                method1 = function () {};

            assert.isUndefined(result.foo);
            topic(result, "foo", method1);
            assert.equal(result.foo, method1);
        },
        "should overwrite already defined attribute": function (topic) {
            var result = stj.class_(''),
                method1 = function () {},
                method2 = function () {};

            assert.isUndefined(result.foo);
            topic(result, "foo", method1);
            topic(result, "foo", method2);
            assert.equal(result.foo, method2);
        },
        "should work as getter when no value is specified": function (topic) {
            var result = stj.class_(''),
                method1 = function () {};

            topic(result, "foo", method1);
            assert.equal(result.foo, method1);
            assert.equal(topic(result, 'foo'), method1);
        }
    },*/
    "include_()": {
        topic: function () {
            return stj.include_;
        },

        "should throw error when extending non module": function (topic) {
            assert.throws(function () {
                topic('non module', function () {});
            });

            assert.throws(function () {
                topic({}, function () {});
            });

            /*assert.doesNotThrow(function () {
                topic(stj.module_(''), function () {});
            });*/
        },

        "should not overwrite property if already present": function (topic) {
            var mod = stj.module_('', {
                    existing: "lala"
                }),
                mod2 = stj.module_('', {
                    existing: "foo"
                });

            stj.module_(mod, function () {
                include_(mod2);
            });
            assert.equal(mod.existing, "lala");
        },

        "should extend a module with an Object": function (topic) {
            var mod = stj.module_(''),
                inc, ext;

            stj.module_(mod, function () {
                properties_({
                    __included__: function (base) {
                        inc = base;
                    },
                    __extended__: function (base) {
                        inc = base;
                    },
                    foo: function () {},
                    bar: function () {}
                });
            });
            assert.isFunction(mod.foo);
            assert.isFunction(mod.bar);
            assert.isUndefined(inc);
            assert.isUndefined(ext);
        },

        /*"should extend a module with an Object new block": function (topic) {

            var Test = stj.class_('', null, function () {
                    include_(function () {
                        property_('my_member', function () {

                        });
                    });
                }),
                instance = new Test();

            assert.isUndefined(Test.my_member);
            assert.isFunction(instance.my_member);
        },*/

        "should extend a module with another module": function (topic) {
            var mixin1 = stj.module_(''),
                mixin2 = stj.module_(''),
                child = stj.module_(''),
                inc, ext;


            stj.module_(mixin1, function () {
                properties_({
                    __included__: function (arg) {
                        inc = arg;
                    },
                    __extended__: function (arg) {
                        ext = arg;
                    },
                    foo: function () {
                        return 'mixin1_foo';
                    },
                    bar: function () {
                        return 'mixin1_bar';
                    },
                    baz: function () {
                        return 'mixin1_baz';
                    },
                    helloworld: function () {
                        return 'mixin1_helloworld';
                    }
                });
            });

            stj.module_(mixin2, function () {
                properties_({
                    foo: function () {
                        return 'mixin2_foo';
                    },
                    helloworld2: function () {
                        return 'mixin2_helloworld';
                    }
                });
            });

            stj.module_(child, function () {
                properties_(child, {
                    baz: function () {
                        return 'child_baz';
                    }
                });
                include_([mixin1, mixin2]);
            });

            assert.equal(child.foo(), 'mixin2_foo');
            assert.equal(child.bar(), 'mixin1_bar');
            assert.equal(child.baz(), 'child_baz');
            assert.equal(child.helloworld(), 'mixin1_helloworld');
            assert.equal(child.helloworld2(), 'mixin2_helloworld');
            assert.equal(inc, child);
            assert.isUndefined(ext);

            property_(child, 'bar', function () {
                return 'child_bar';
            });
            assert.equal(child.bar(), 'child_bar');

            //
            property_(mixin1, 'newMethod', function () {});
            assert.isFunction(mixin1.newMethod);
            assert.isFunction(child.newMethod);
        },
        "should extend a class with an Object": function (topic) {
            var Test = stj.class_(''),
                instance = new Test();

            stj.class_(Test, function () {
                properties_({
                    foo: function () {},
                    bar: function () {}
                });
            });


            assert.isFunction(instance.foo);
            assert.isFunction(instance.bar);

            assert.isUndefined(instance.baz);
            properties_(Test.prototype, {
                baz: function () {}
            });
            assert.isFunction(instance.baz);
        },
        "should extend a class with another module": function (topic) {
            var Test = stj.class_(''),
                instance = new Test(),
                included = stj.module_(''),
                inc, ext;

            stj.module_(included, function () {
                properties_({
                    __included__: function (arg) {
                        inc = arg;
                    },
                    __extended__: function (arg) {
                        ext = arg;
                    },
                    foo: function () {
                        return 'included_foo';
                    },
                    bar: function () {
                        return 'included_bar';
                    },
                    baz: function () {
                        return 'included_baz';
                    }
                });
            });

            stj.class_(Test, function () {
                properties_({

                    foo: function () {
                        return 'test_foo';
                    },
                    bar: function () {
                        return 'test_bar';
                    },
                    helloworld: function () {
                        return 'helloworld';
                    }
                });
                include_(included);
            });

            assert.equal(instance.foo(), 'test_foo');
            assert.equal(instance.helloworld(), 'helloworld');
            assert.equal(instance.bar(), 'test_bar');
            assert.equal(instance.baz(), 'included_baz');

            //hooks
            assert.equal(inc, Test.prototype);
            assert.isUndefined(ext);

            //
            property_(included, 'newMethod', function () {});
            assert.isFunction(instance.newMethod);
            assert.isFunction(instance.newMethod);
        }
    },

    "extend_()": {
        topic: function () {
            return stj.extend_;
        },
        "should throw error when extending non class": function (topic) {

            assert.throws(function () {
                topic(stj.module_(''), function () {});
            });

            assert.throws(function () {
                topic('non module', function () {});
            });
        },
        "should extend a new class with a new block": function (topic) {

            var Test = stj.class_('', null, function () {
                    extend_(function () {
                        property_('my_static', function () {

                        });
                    });
                }),
                instance = new Test();

            assert.isFunction(Test.my_static);
            assert.isUndefined(instance.my_static);
        },
        "should extend a class with a new block": function (topic) {
            var Test = stj.class_(''),
                included = stj.module_(''),
                instance = new Test(),
                inc, ext;


            stj.module_(included, function () {
                properties_({
                    __included__: function (arg) {
                        inc = arg;
                    },
                    __extended__: function (arg) {
                        ext = arg;
                    },
                    foo: function () {
                        return 'included_foo';
                    },
                    bar: function () {
                        return 'included_bar';
                    },
                    baz: function () {
                        return 'included_baz';
                    }
                });
            });
            topic(Test, function () {
                property_('my_static', function () {

                });

                properties_({
                    'my_inclusion_1': function () {},
                    'my_inclusion_2': function () {}
                });
                include_(included);
            });
            properties_(Test.prototype, {
                method: function () {}
            });

            //hooks
            assert.isUndefined(inc);
            assert.equal(ext, Test);


            assert.isFunction(Test.my_static);
            assert.isUndefined(instance.my_static);
            assert.isFunction(Test.my_inclusion_1);
            assert.isUndefined(instance.my_inclusion_1);
            assert.isFunction(Test.my_inclusion_2);
            assert.isUndefined(instance.my_inclusion_2);
            assert.isUndefined(Test.method);
            assert.isFunction(instance.method);
            assert.isFunction(Test.foo);
            assert.isFunction(Test.bar);
        }
    },

    /*"globals_()": { //TODO
        topic: function () {
            return stj.globals_;
        },
    },*/

    "meta information": {
        topic: function () {
            return stj.use_;
        },
        "should return a module": function (topic) {
            assert.equal(topic('error.Error'), Error);
        },
        /*"should return a named module": function (topic) {
            assert.equal(topic('error.Error').displayName, "error.Error");
        },*/
        "should return a named meta module": function (topic) {
            assert.equal(stj.meta_(topic('error.Error')).displayName, "error.Error");
        }
    }

});