/*jslint nodejs:true, indent:4 */
var vows = require('vows'),
    assert = require('assert'),
    env = require('./_env'),
    __filenameTested = env.lib('stj'),
    nativeMethods = {
        indexOf: String.prototype.indexOf,
        toString: String.prototype.toString
    },
    stj = require(__filenameTested),
    object = stj.use_('Object'),
    string = stj.use_('String');
/*******************************************************************************
 * JSLint validation
 ******************************************************************************/
exports.JSLintTest = env.JSLintTest([/*__filenameTested, */__filename]);

/*******************************************************************************
 * StringTest class
 ******************************************************************************/
exports.StringTest = vows.describe('String functions').addBatch({
    "[native methods]": {
        topic: function () {
            return string.prototype;
        },
        'should not change native implementation': function (topic) {
            for (var methodName in nativeMethods) {
                if (nativeMethods.hasOwnProperty(methodName)) {
                    assert.equal(nativeMethods[methodName], topic[methodName]);
                    assert.equal(topic[methodName].toString(), "function " + methodName + "() { [native code] }");
                }
            }
        }
    },
    "isEmpty()": {
        topic: function () {
            return string.prototype.isEmpty.bindToCall();
        },
        'should have length of 0': function (topic) {
            assert.equal(string.prototype.isEmpty.length, 0);
        },
        'should return true only for ""': function (topic) {
            assert.equal(topic(''), true);
            assert.equal(topic('test'), false);
            assert.equal(topic('TEST'), false);
        }
    },

    "repeat()": {
        topic: function () {
            return string.prototype.repeat.bindToCall();
        },
        'should have length of 1': function (topic) {
            assert.equal(string.prototype.repeat.length, 1);
        },
        "should return empty string if repeated <= 0 times": function (topic) {
            assert.equal(topic("foo", -1), "");
            assert.equal(topic("foo", 0), "");
        },
        "should return same string if repeated 1 times": function (topic) {
            assert.equal(topic("foo", 1), "foo");
        },
        "should return repeated string if repeated N times": function (topic) {
            assert.equal(topic("foo", 2), "foofoo");
            assert.equal(topic("foo", 3), "foofoofoo");
        }
    },
    "padLeft()": {
        topic: function () {
            return string.prototype.padLeft.bindToCall();
        },
        'should have length of 2': function (topic) {
            assert.equal(string.prototype.padLeft.length, 2);
        },
        "should return same string when padding length is 0": function (topic) {
            assert.equal(topic("", 0), "");
            assert.equal(topic("foo", 0), "foo");
        },
        "should return same string when length is inferior to the this length": function (topic) {
            assert.equal(topic("foobar", 2), "foobar");
            assert.equal(topic("foobar", 6), "foobar");
        },
        "should throw error if string is empty": function (topic) {
            assert.equal(topic("foo", 0, ""), "foo");
            assert.throws(function () {
                topic("mystring", 13, "");
            });
        },
        "should return left padded string when length is superior": function (topic) {
            assert.equal(topic("foo", 4), " foo");
            assert.equal(topic("foo", 5), "  foo");
            assert.equal(topic("foo", 5, '_'), "__foo");
            assert.equal(topic("foo", 5, '-='), "-=foo");
            assert.equal(topic("foo", 6, '-='), "=-=foo");
        }
    },
    "padRight()": {
        topic: function () {
            return string.prototype.padRight.bindToCall();
        },
        'should have length of 2': function (topic) {
            assert.equal(string.prototype.padRight.length, 2);
        },
        "should return same string when padding length is 0": function (topic) {
            assert.equal(topic("", 0), "");
            assert.equal(topic("foo", 0), "foo");
        },
        "should return same string when length is inferior to the this length": function (topic) {
            assert.equal(topic("foobar", 2), "foobar");
            assert.equal(topic("foobar", 6), "foobar");
        },
        "should throw error if string is empty": function (topic) {
            assert.equal(topic("foo", 0, ""), "foo");
            assert.throws(function () {
                topic("mystring", 13, "");
            });
        },
        "should return right padded string when length is superior": function (topic) {
            assert.equal(topic("foo", 4), "foo ");
            assert.equal(topic("foo", 5), "foo  ");
            assert.equal(topic("foo", 5, '_'), "foo__");
            assert.equal(topic("foo", 5, '-='), "foo-=");
            assert.equal(topic("foo", 6, '-='), "foo-=-");
        }
    },
    "pad()": {
        topic: function () {
            return string.prototype.pad.bindToCall();
        },
        'should have length of 3': function (topic) {
            assert.equal(string.prototype.pad.length, 3);
        },
        "should return pad left when mode is 'l', 'left'": function (topic) {
            var mode = 'left';
            assert.equal(topic("foo", 4, ' ', mode), " foo");
            assert.equal(topic("foo", 5, ' ', mode), "  foo");
            assert.equal(topic("foo", 5, '_', mode), "__foo");
            assert.equal(topic("foo", 5, '-=', mode), "-=foo");
            assert.equal(topic("foo", 6, '-=', mode), "=-=foo");
        },
        "should return pad right when mode is 'r', 'right'": function (topic) {
            var mode = 'right';
            assert.equal(topic("foo", 4, ' ', mode), "foo ");
            assert.equal(topic("foo", 5, ' ', mode), "foo  ");
            assert.equal(topic("foo", 5, '_', mode), "foo__");
            assert.equal(topic("foo", 5, '-=', mode), "foo-=");
            assert.equal(topic("foo", 6, '-=', mode), "foo-=-");
        },
        "should return pad left and right when mode is 'b', 'both'": function (topic) {
            var mode = 'both';
            assert.equal(topic("foo", 4, ' ', mode), " foo");
            assert.equal(topic("foo", 5, ' ', mode), " foo ");
            assert.equal(topic("foo", 7, '_', mode), "__foo__");
            assert.equal(topic("foo", 7, '-=', mode), "-=foo-=");
            assert.equal(topic("foo", 8, '-=', mode), "=-=foo-=");
            assert.equal(topic("foo", 9, '-=', mode), "=-=foo-=-");
        }
    },
    "stripLeft()": {
        topic: function () {
            return string.prototype.stripLeft.bindToCall();
        },
        'should have length of 0': function (topic) {
            assert.equal(string.prototype.stripLeft.length, 0);
        },
        "should return empty string for empty string": function (topic) {
            assert.equal(topic("", "  "), "");
        },
        "should return same string if replacement is empty": function (topic) {
            assert.equal(topic("mystring", ""), "mystring");
        },
        "should return same string if replacement is not found at right": function (topic) {
            assert.equal(topic("mystring", "foo"), "mystring");
            assert.equal(topic("mystring", "tri"), "mystring");
        },
        "should strip left for non empty string": function (topic) {
            assert.equal(topic("mystring", "mystr"), "ing");
        }
    },
    "stripRight()": {
        topic: function () {
            return string.prototype.stripRight.bindToCall();
        },
        'should have length of 0': function (topic) {
            assert.equal(string.prototype.stripRight.length, 0);
        },
        "should return empty string for empty string": function (topic) {
            assert.equal(topic("", "  "), "");
        },
        "should return same string if replacement is empty": function (topic) {
            assert.equal(topic("mystring", ""), "mystring");
        },
        "should return same string if replacement is not found at right": function (topic) {
            assert.equal(topic("mystring", "foo"), "mystring");
            assert.equal(topic("mystring", "tri"), "mystring");
            assert.equal(topic("mystring", "myst"), "mystring");
        },
        "should strip left for non empty string": function (topic) {
            assert.equal(topic("mystring", "ing"), "mystr");
        }
    },
    "strip()": {
        topic: function () {
            return string.prototype.strip.bindToCall();
        },
        'should have length of 0': function (topic) {
            assert.equal(string.prototype.strip.length, 0);
        },
        "should return empty string for empty string": function (topic) {
            assert.equal(topic("", "  "), "");
        },
        "should return same string if replacement is empty": function (topic) {
            assert.equal(topic("mystring", ""), "mystring");
        },
        "should return same string if replacement is not found at right or left": function (topic) {
            assert.equal(topic("mystring", "foo"), "mystring");
            assert.equal(topic("mystring", "tri"), "mystring");
            assert.equal(topic("mystring", "yst"), "mystring");
        },
        "should strip left for non empty string": function (topic) {
            assert.equal(topic("mystring", "ing"), "mystr");
            assert.equal(topic("mystring", "myst"), "ring");
        }
    },
    "format()": {
        topic: function () {
            return string.prototype.format.bindToCall();
        },
        'should have length of 1': function (topic) {
            assert.equal(string.prototype.format.length, 1);
        },
        'should return unchanged string when no args are passed': function (topic) {
            var str = 'mystring';
            assert.equal(topic(str), str);
        },
        'should return formatted string when passing array': function (topic) {
            var str = 'mystring {0} ahah {2} ohoh {1} {flag}';
            assert.equal(topic(str, []), str);
            assert.equal(topic(str, [undefined, 'test', false]), 'mystring {0} ahah false ohoh test {flag}');
        },
        'should return formatted string when passing object': function (topic) {
            var str = 'mystring {flag1} ahah {flag3} ohoh {1}';
            assert.equal(topic(str, {}), str);
            assert.equal(topic(str, {flag3: 'test3'}), 'mystring {flag1} ahah test3 ohoh {1}');
            assert.equal(topic(str, {flag3: 'test3', flag1: undefined}), 'mystring {flag1} ahah test3 ohoh {1}');
            assert.equal(topic(str, {1: 'test2'}), 'mystring {flag1} ahah {flag3} ohoh test2');
        },
        'should return formatted string for nested formatters patterns': function (topic) {
            var str = 'mystring {{flag1} ahah {flag3} ohoh {1}}';
            assert.equal(topic(str, {flag3: 'test3'}), 'mystring {{flag1} ahah test3 ohoh {1}}');
        },
        'should return formatted string !s(string), !r(representation) flag': function (topic) {
            var str = 'mystring {0!s} ahah {1!r}';
            assert.equal(topic(str, []), str);
            assert.equal(topic(str, [1234, {hello: 'world'}]), 'mystring 1234 ahah [object Object]');
        },
        'should return formatted string with positional arguments when {} is encountered': function (topic) {
            var str = 'mystring {!s} sep {} {!r}';
            assert.equal(topic(str, []), str);
            assert.equal(topic(str, [1, 2, '3']), 'mystring 1 sep 2 "3"');
        },
        'should return formatted string with when path is provided (like 0.argument or arg[0])': function (topic) {
            var args = {
                    0: {
                        foo: ':-)'
                    },
                    bar: {
                        baz: 'hello!'
                    },
                    "very_special.char ?": {
                        result: 'hello!'
                    },
                    "baz": "mystring"
                };
            assert.equal(topic('', args), '');
            assert.equal(topic('{0.foo}', args), ":-)");
            assert.equal(topic('{bar.baz}', args), "hello!");

            assert.equal(topic('{[0][foo]}', args), ":-)");
            assert.equal(topic('{[very_special.char ?][result]!s}', args), "hello!");

            assert.equal(topic('{baz}', args), "mystring");
            assert.equal(topic('{baz.0}', args), "m");
            assert.equal(topic('{baz[0]}', args), "m");
        },

        'should return formatted string for align instruction': function (topic) {
            assert.equal(topic('{:<20}', []), '{:<20}');
            assert.equal(topic('{:<20}', ['leftaligned']), "leftaligned         ");
            assert.equal(topic('{:>20}', ['rightaligned']), "        rightaligned");
            assert.equal(topic('{:^20}', ['centered']), "      centered      ");
        },

        'should return formatted integer': function (topic) {
            assert.equal(topic("{0} {1}", [42, -42]), "42 -42");
            assert.equal(topic("{0:-} {1:-}", [42, -42]), "42 -42");
            assert.equal(topic("{0:+} {1:+}", [42, -42]), "+42 -42");
            assert.equal(topic("{0: } {1: }", [42, -42]), " 42 -42");
            assert.equal(topic("int: {0:d};  hex: {0:x};  oct: {0:o};  bin: {0:b}", [42]), "int: 42;  hex: 2a;  oct: 52;  bin: 101010");
            assert.equal(topic("int: {0:d};  hex: {0:#x};  oct: {0:#o};  bin: {0:#b}", [42]), "int: 42;  hex: 0x2a;  oct: 0o52;  bin: 0b101010");
        },

        'should return formatted float': function (topic) {
            assert.equal(topic("{:+f}; {:+f}", [3.14, -3.14]), "+3.140000; -3.140000");
            assert.equal(topic("{: f}; {: f}", [3.14, -3.14]), " 3.140000; -3.140000");
            assert.equal(topic("{:-f}; {:-f}", [3.14, -3.14]), "3.140000; -3.140000");

            assert.equal(topic("{:-.2f}; {:-.3f}", [3.14, -3.14]), "3.14; -3.140");
            assert.equal(topic("{:+.2f}", [3.166]), "+3.17");

            assert.equal(topic("{:.2%}", [(19.5 / 22)]), "88.64%");
        }

    },

    "startsWith()": {
        topic: function () {
            return string.prototype.startsWith.bindToCall();
        },
        'should have length of 1': function (topic) {
            assert.equal(string.prototype.startsWith.length, 1);
        },
        'should return true if this is starting with first parameter': function (topic) {
            var str = 'mystring';
            assert.equal(topic(str, ''), true);
            assert.equal(topic(str, 'mys'), true);
            assert.equal(topic(str, 'mystr'), true);
            assert.equal(topic(str, 'str'), false);
            assert.equal(topic(str, 'ing'), false);
            assert.equal(topic(str, 'mystring'), true);
        },
        'should return true if this is starting with substring using start & end parameters': function (topic) {
            var str = 'mystring';
            //Use start, end parameters
            assert.equal(topic(str, 'mys', 0, str.length), true);
            assert.equal(topic(str, 'ys', 1, str.length), true);
            assert.equal(topic(str, 'yst', 1, str.length), true);
            assert.equal(topic(str, 'yst', 2, str.length), false);
            assert.equal(topic(str, 'mystring', 0, str.length - 1), false);

            assert.equal(topic(str, 'mys', 0, 3), true);
            assert.equal(topic(str, 'mys', 0, 2), false);
            assert.equal(topic(str, 'g', str.length - 1), true);
        }
    },

    "endsWith()": {
        topic: function () {
            return string.prototype.endsWith.bindToCall();
        },
        'should have length of 1': function (topic) {
            assert.equal(string.prototype.endsWith.length, 1);
        },
        'should return true if this is starting with first parameter': function (topic) {
            var str = 'mystring';
            assert.equal(topic(str, ''), true);
            assert.equal(topic(str, 'ing'), true);
            assert.equal(topic(str, 'ring'), true);
            assert.equal(topic(str, 'str'), false);
            assert.equal(topic(str, 'mystr'), false);
            assert.equal(topic(str, 'mystring'), true);
        },
        'should return true if this is starting with substring using start & end parameters': function (topic) {
            var str = 'mystring';
            //Use start, end parameters
            assert.equal(topic(str, 'ing', 0, str.length), true);
            assert.equal(topic(str, 'rin', 2, str.length - 1), true);
            assert.equal(topic(str, 'rin', 2, str.length - 3), false);
            assert.equal(topic(str, 'mystring', 0, str.length - 1), false);

            assert.equal(topic(str, 'mys', 0, 3), true);
            assert.equal(topic(str, 'mys', 0, 2), false);
            assert.equal(topic(str, 'g', str.length - 1), true);
            assert.equal(topic(str, 'm', 0, 1), true);
        }
    },

    "isAlphaNumeric()": {
        topic: function () {
            return string.prototype.isAlphaNumeric.bindToCall();
        },
        'should have length of 0': function (topic) {
            assert.equal(string.prototype.isAlphaNumeric.length, 0);
        },
        'should return true if containing alpha characters': function (topic) {
            assert.equal(topic(''), true);
            assert.equal(topic('test'), true);
            assert.equal(topic('TEST'), true);
            assert.equal(topic('Test'), true);
            assert.equal(topic('Test123'), true);
            assert.equal(topic('Test#'), false);
            assert.equal(topic('123'), true);
        }
    },

    "isAlpha()": {
        topic: function () {
            return string.prototype.isAlpha.bindToCall();
        },
        'should have length of 0': function (topic) {
            assert.equal(string.prototype.isAlpha.length, 0);
        },
        'should return true if containing alpha characters': function (topic) {
            assert.equal(topic(''), true);
            assert.equal(topic('test'), true);
            assert.equal(topic('TEST'), true);
            assert.equal(topic('Test'), true);
            assert.equal(topic('Test123'), false);
            assert.equal(topic('Test#'), false);
            assert.equal(topic('123'), false);
        }
    },

    "isDigit()": {
        topic: function () {
            return string.prototype.isDigit.bindToCall();
        },
        'should have length of 0': function (topic) {
            assert.equal(string.prototype.isDigit.length, 0);
        },
        'should return true if containing only digit characters': function (topic) {
            assert.equal(topic('test'), false);
            assert.equal(topic('TEST'), false);
            assert.equal(topic('Test'), false);
            assert.equal(topic('Test123'), false);
            assert.equal(topic('Test#'), false);
            assert.equal(topic('123'), true);
            assert.equal(topic('123.12'), false);
            assert.equal(topic('123,12'), false);
            assert.equal(topic(''), true);
        }
    },

    "get()": {
        topic: function () {
            return string.prototype.get.bindToCall();
        },
        'should have length of 1': function (topic) {
            assert.equal(string.prototype.get.length, 1);
        },
        'should return one value if key is not array': function (topic) {
            assert.equal(topic('test', 0), "t");
            assert.equal(topic('test', 1), "e");
            assert.equal(topic('test', 2), "s");
            assert.equal(topic('test', 5), "");
            assert.equal(topic('test', 8), "");

            assert.equal(topic('test', -1), "t");
            assert.equal(topic('test', -2), "s");
        },
        'should return an array of sliced values if key is not array': function (topic) {
            assert.deepEqual(topic("", []), "");
            assert.deepEqual(topic("test", [0, 0]), "t");
            assert.deepEqual(topic("test", [1, 1]), "e");
            assert.deepEqual(topic("test", [1, -2]), "es");
            assert.deepEqual(topic("test", [1]), "est");
        }
    }
});